# -- binding.tcl
# Bindings des touches (editeurs, menu, ...)
#
# Auteur:               Pascal Bouvier
# Creation:             25/09/96
# Modifications:
#
# (c) PrologIA 1996
# -----------------------------------------------------------------------------


@com {
On definit les classes d'evenements E :
- ted_commun
- ted_motif
- ted_emacs
- ted_mac

- tty_commun
- tty_motif
- tty_emacs
- tty_mac

L'enchainement des bindings est :

	.ed.t   *_common   <E>   moved   .ed   all 

au lieu de :

	.ed.t   Text   .ed   all

Syntaxe :

	bind:addkey  classe  event  nom  script

Exemple :

	bind:addkey ted_common <Shift-Key-Right> select-to-next-char {
		tkTextKeySelect %W [%W index {insert + 1c}]
		}

Il y a aussi 'bind:addbutton'
}

#
#
#
proc bind:init {} {
	catch {unset bind:tab_event_name}
	catch {unset bind:tab_name_event}
}


#
#
#
proc bind:addkey {class event name script} {
	global bind:tab_event_name bind:tab_name_event
	bind $class $event $script
	set bind:tab_event_name($class,$event) $name
	lappend bind:tab_name_event($class,$name) $event
}

#
#
#
proc bind:addbutton {class event name script} {
	global bind:tab_event_name bind:tab_name_event
	bind $class $event $script
	set bind:tab_event_name($class,$event) $name
	lappend bind:tab_name_event($class,$name) $event
}

#
#
#
proc bind:giveevent {w name} {
	global bind:tab_name_event
	set lret {}
	foreach class [bindtags $w] {
		if { ! [catch {set bind:tab_name_event($class,$name)} ret]} {
			set lret [concat $lret $ret]
			}
		}
	return $lret
}

#
#
#
proc bind:event_to_shortcut {event} {
	set len [string len $event]
#	incr len -2
#	set l [split [string range $event 1 $len] -]

	set L [translit "<" " " $event]
	set L [translit ">" " " $L]

set ret ""
set nb 0
foreach l $L {
	set l [split $l -]
	if $nb {append ret " "}
	incr nb

	while {"$l" != ""} {
		set elem [lindex $l 0]
		set l [lrange $l 1 end]
		switch -exact -- $elem {
			Control { append ret C- }
			Meta { append ret M- }
			Shift { append ret S- }
			Key break
			default { set l $elem; break }
			}
		}
	while {"$l" != ""} {
		set elem [lindex $l 0]
		set l [lrange $l 1 end]
		switch -glob -- $elem {
			Delete { append ret DEL }
			BackSpace { append ret BS }
			Insert { append ret INS }
			Select { append ret SEL }
			Prior { append ret PREV }
			Tab { append ret TAB }
			Return { append ret RET }
			KP_Enter { append ret ENTER }

			space { append ret SPACE }
			greater { append ret > }
			less { append ret < }
			slash { append ret / }
			backslash { append ret \\ }
			bracketleft { append ret \[ }
			bracketright { append ret ] }
			F[0-9]* { append ret $elem }
			? { append ret $elem }
			default { append ret [string toupper $elem] }
			}
		}
	}
	return $ret
}


# -----------------------------------------------------------------------------
# Menus 
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# ajoute une commande dans le menu m associe au texte t
# menu:add menu text prefix commande autres-args
# -----------------------------------------------------------------------------
proc menu:add {m t prefix com args} {
	global font_for_menus
	set Lcom [list $m add command -font $font_for_menus  \
		-label [lang: $com] \
		-u [lang:u $com] \
		-command [list $prefix:$com $t] \
		-acc [bind:listevents_stringshortcuts [bind:giveevent $t $com]]]
	eval [concat $Lcom $args]
	}

# -----------------------------------------------------------------------------
# menu:addcheck menu text nom variable autres-args
# -----------------------------------------------------------------------------
proc menu:addcheck {m t nom var args} {
	global font_for_menus
	set Lcom [list $m add checkbutton -font $font_for_menus -selectcolor green  \
		-label [lang: $nom] \
		-u [lang:u $nom] \
		-variable $var]
	eval [concat $Lcom $args]
	}

# -----------------------------------------------------------------------------
# menu:addradio menu text nom variable autres-args
# -----------------------------------------------------------------------------
proc menu:addradio {m t nom var args} {
	global font_for_menus
	set Lcom [list $m add radiobutton -font $font_for_menus -selectcolor green  \
		-label [lang: $nom] \
		-u [lang:u $nom] \
		-variable $var]
	eval [concat $Lcom $args]
	}

#
#
#
proc bind:listevents_stringshortcuts {Lev} {
	set L ""
	foreach ev $Lev {
		append L "  " [bind:event_to_shortcut $ev]
		}
	return $L
	}

#------------------------------------------------------------------------------
# Etant Donnee un chaine (abstraite) donne sa traduction dans la
# langue courante.
#------------------------------------------------------------------------------
proc lang: {str} {
	global lang
	if {$str == ""} {
		return ""
		} \
	else {
		return [lindex $lang($str) 0]
		}
}

#------------------------------------------------------------------------------
# meme chose, mais pour le caractere a souligner (si pertinent)
# pour les underlines (-1 s'il n'y en a pas)
#------------------------------------------------------------------------------
proc lang:u {w} {
	global lang
	set u [lindex $lang($w) 1]
	if {"$u"==""} {
		return -1
		} \
	else {
		return $u
		}
}

# -- EOF binding.tcl ----------------------------------------------------------

