# -- bugs.tcl
# Les fenetres pour les rapports de bug
#
# Auteur:		Pascal Bouvier
# Creation:		11/10/95
# Modifications:	
#			PBO 29/03/96 mettre le user name dans le nouveau champ "Copy to:"
#
# (c) PrologIA 1995
# -----------------------------------------------------------------------------


# ==== G L O B A L E S =======================================

global bug:bugid
set bug:bugid 0

global bug:mailexe
# mailx pour hp
set bug:mailexe "mail"

global bug:mailp4
set bug:mailp4 "p4bugs@prologianet.univ-mrs.fr"

global bug:sender

if [catch {set bug:sender $env(USER)}] {
	if [catch {set bug:sender $env(USERNAME)}] {
		if [catch {set bug:sender $env(LOGNAME)}] {
			set bug:sender ""
			}
		}
	}
#unix: set bug:sender $env(USER)

global bug:severity

global bug:topic

global bug:also
set bug:also 1

# ==== P R O C S =======================================

# -----------------------------------------------------------------------------
# Identification unique d'objets. (fenetres de bug)
# -----------------------------------------------------------------------------
proc bug:newid {} {
	global bug:bugid
	return [incr bug:bugid]
}

# -----------------------------------------------------------------------------
# Le dialogue-mail pour les rapports de bug
# MENU
# -----------------------------------------------------------------------------
proc bug:Dial_Bug {} {
	global bug:mailp4 bug:topic bug:severity
	
	set id [bug:newid]
	set w [toplevel .bug-$id]
	set titre [format [lang: T-mail-report] $id]
	wm title $w $titre
	wm minsize $w 0 0

	set bug:topic "..."
	set bug:severity [lang: BUG.unknown]

	#
	# l'entree pour l'adresse du mail
	#
	set fadr [frame $w.fadr]
		set f [frame $fadr.fsnd]
			label $f.l -text [lang: BUG.send-to]
			entry $f.e -textvar bug:mailp4 -relief ridge -width 30
			pack $f.l -side left -fill both
			pack $f.e -side left -fill both -expand 1
		pack $f -side top -fill both -expand 1
	#
	# l'entree pour l'envoyeur du mail
	#
	set fexp [frame $w.fexp]
		set f [frame $fexp.fsnd]
			label $f.l -text [lang: BUG.copy-to]
			entry $f.e -textvar bug:sender -relief ridge -width 30
			pack $f.l -side left -fill both
			pack $f.e -side left -fill both -expand 1
		pack $f -side top -fill both -expand 1

	set cbalso [checkbutton $w.cbalso -text [lang: BUG.self-copy] -variable bug:also]

	#
	# Les themes:
	# Language {general,Primitives,Iso}, Graphical Env {general,editor,console,}
	#
	set ftopic [frame $w.ftopic]
		label $ftopic.l -text [lang: BUG.choose-topic]
		set m $ftopic.mbm.m
		menubutton $ftopic.mbm -menu $m -relief raised -bd 1 \
			-textvariable bug:topic -indicator on
		menu $m -tearoff 0
		$m add radiobutton -label [lang: BUG.lang:general] \
				-variable bug:topic -indicator off
		$m add radiobutton -label [lang: BUG.lang:p4-bips] \
				-variable bug:topic -indicator off
		$m add radiobutton -label [lang: BUG.lang:iso-bips] \
				-variable bug:topic -indicator off
		$m add radiobutton -label [lang: BUG.grenv:general] \
				-variable bug:topic -indicator off
		$m add radiobutton -label [lang: BUG.grenv:editor] \
				-variable bug:topic -indicator off
		$m add radiobutton -label [lang: BUG.grenv:console] \
				-variable bug:topic -indicator off
		$m add radiobutton -label [lang: BUG.grenv:debugger] \
				-variable bug:topic -indicator off
		pack $ftopic.l $ftopic.mbm -side left

	#
	# La gravite:
	# Unknown, suggestion, annoyance, avoidable bug, unavoidable bug,
	# App hang/crash App Exit
	#
	set fgrav [frame $w.fgrav]
		label $fgrav.l -text [lang: BUG.severity]
		set m $fgrav.mbm.m
		menubutton $fgrav.mbm -menu $m -relief raised -bd 1 \
			-textvariable bug:severity -indicator on
		menu $m -tearoff 0
		$m add radiobutton -label [lang: BUG.unknown] \
				-variable bug:severity -indicator off
		$m add radiobutton -label [lang: BUG.suggestion] \
				-variable bug:severity -indicator off
		$m add radiobutton -label [lang: BUG.annoyance] \
				-variable bug:severity -indicator off
		$m add radiobutton -label [lang: BUG.avoidable-bug] \
				-variable bug:severity -indicator off
		$m add radiobutton -label [lang: BUG.unavoidable-bug] \
				-variable bug:severity -indicator off
		$m add radiobutton -label [lang: BUG.app-hang-crash] \
				-variable bug:severity -indicator off
		$m add radiobutton -label [lang: BUG.app-exits] \
				-variable bug:severity -indicator off
		pack $fgrav.l $fgrav.mbm -side left

	#
	# La description
	#
	set desc [label $w.ldesc -text [lang: BUG.please-describe] -anch w]
	set fdesc [frame $w.fdesc]
		text $fdesc.t -width 60 -hei 30 -relief sunken -bd 1 \
				-yscroll "$fdesc.sv set"
		scrollbar $fdesc.sv -orient vert -command "$fdesc.t yview" -relief sunken
#				-bg #e300f300f500 -activeback #00002e2e0000
		if [catch "exec uname -a" uname] {
			set uname [lang: unavailable]
			}
		$fdesc.t insert insert "\n\n\n\n\n\nUNAME: $uname\n\n"
		$fdesc.t mark set insert 1.0
		pack $fdesc.sv -side right -fill y
		pack $fdesc.t -side left -fill both -expand 1
		focus $fdesc.t

	#
	# Les boutons du bas
	#
	set fbut [frame $w.fbut]
		button $fbut.bsend -text [lang: BUG.send-mail] \
				-command "bug:sendmail $fdesc.t; destroy $w"
		button $fbut.bcan -text [lang: cancel] -command "destroy $w"
		pack $fbut.bsend $fbut.bcan -side left -padx 5m -pady 2m -ipadx 2m -ipady 1m
	pack $fadr $fexp $ftopic $fgrav $desc $fbut \
			-side top -fill both -anc w -pady 2m -padx 2m
	pack $cbalso -after $fexp \
			-side top -anc w -pady 2m -padx 2m
	pack $fdesc -after $desc -side top -fill both -expand 1 -anc w \
			-pady 2m -padx 2m
}

# -----------------------------------------------------------------------------
# On envoie par mail le texte de l'objet t
# -----------------------------------------------------------------------------
proc bug:sendmail {t} {
	global bug:mailp4 bug:also bug:sender

	if [catch "bug:send_a_mail ${bug:mailp4} $t" u] {
		tk_dialog .ma "Error" \
			"Error during sendmail to ${bug:mailp4} :\n $u" error 0 Ok
		}
	if ${bug:also} {
		if [catch "bug:send_a_mail ${bug:sender} $t" u] {
			tk_dialog .ma "Error" \
				"Error during sendmail to ${bug:sender} :\n $u" error 0 Ok
			}
		}
}

# -----------------------------------------------------------------------------
# L'action effective de l'envoi de mail a une personne donnee
# -----------------------------------------------------------------------------
proc bug:send_a_mail {dest t} {
	global bug:mailp4 bug:severity bug:topic bug:mailexe
	set subject [lang: BUG.report-for-p4] 
	exec ${bug:mailexe} -s "$subject (${bug:severity})" \
			$dest << "TOPIC: ${bug:topic}\n\
							\n[$t get 1.0 end]"
}

#-- EOF bugs.tcl -------------------------------------------------------------
