# -- choose.tcl
# Le selecteur de fichier pour ouvrir, sauver ou choisir directory
# recupere d'apres Mario Jorge Silva, Berkeley.
#
# Auteur:		Pascal Bouvier
# Creation:		../05/95
# Modifications:	
#			 	PB ../05/95	profonds remaniements
#			 	PB 01/11/95	glob au lieu de /bin/ls
#			 	PB 02/11/95	installation d'un filtre incremental
#			 	PB 29/04/96	verification d'existance/lisibilite du fichier dans getfile
#				PBO 04/10/96 tk4.1 + bindings
#               PBO 08/11/96 ESC sur efilter ; autoscroll du path courant
#
# (c) PrologIA 1995
# -----------------------------------------------------------------------------


# ==== G L O B A L E S =======================================

set stdfile:id 0

if 0 {
# use the "option" command for further configuration

option add *Listbox*font \
    "-*-helvetica-medium-r-normal-*-12-*-*-*-p-*-iso8859-1" startupFile
option add *Entry*font \
    "-*-helvetica-medium-r-normal-*-12-*-*-*-p-*-iso8859-1" startupFile
option add *Label*font \
    "-*-helvetica-medium-r-normal-*-12-*-*-*-p-*-iso8859-1" startupFile
}

set font_for_choose "-*-helvetica-bold-r-normal-*-14-*-*-*-p-*-iso8859-1"

# ==== P R O C S =======================================

# -----------------------------------------------------------------------------
# this is the default proc called when error is detected
# indicate your own as an argument to choose
# -----------------------------------------------------------------------------
proc stdfile:get:errorHandler {errorMessage} {
    puts stdout "error: $errorMessage"
    catch { cd ~ }
}

# -----------------------------------------------------------------------------
# Identification unique d'objets.
# -----------------------------------------------------------------------------
proc stdfile:newid {} {
	global stdfile:id
    return [incr stdfile:id]
}

# -----------------------------------------------------------------------------
#UNUSED
# Ceci cree la boite de selection de fichier.
# choose(file) contient le fichier choisi
# choose(dir) contient le directory
# retourne dans choose(file) --NONE-- si cancel
# -----------------------------------------------------------------------------
#proc stdfile:getdir {label {cmd ""}} {
#	global choose
#	set res [stdfile:choose $label $cmd stdfile:listdirs get]
#	if { "$res" == "." } {
#		return $choose(dir)
#		}
#	return $res
#}


# -----------------------------------------------------------------------------
# sfputfile
# -----------------------------------------------------------------------------
proc stdfile:put {label {cmd ""} {listcmd stdfile:listfiles}} {
	return [stdfile:choose $label $cmd $listcmd put]
}

# -----------------------------------------------------------------------------
# sfgetfile
# -----------------------------------------------------------------------------
proc stdfile:get {label {cmd ""} {listcmd stdfile:listfiles}} {
	return [stdfile:choose $label $cmd $listcmd get]
}

# -----------------------------------------------------------------------------
# Le gros travail est fait ici
# -----------------------------------------------------------------------------
proc stdfile:choose {label {cmd ""} {listcmd stdfile:listfiles} {op get}} {
	upvar #0 font_for_choose ffc
	
	set w .choose[stdfile:newid]
	# TMP 
	catch {stdfile:free $w}
	
	toplevel $w
	wm geo $w +300+300
	grab $w
	wm title $w [lang: T-choose]
	
	# path independent names for the widgets
	global choose
	
	set choose(entry) $w.file.eframe.entry
	set choose(efilter) $w.file.eframe.efilt
	set choose(list) $w.file.sframe.list
	set choose(scroll) $w.file.sframe.scroll
	set choose(ok) $w.fbut.fok.bok
	set choose(cancel) $w.fbut.bcan
	set choose(dirlabel) $w.file.dirlabel
	set choose(cmd) $listcmd
	set choose(nodots) 1
	set choose(onlydirs) 0
	set choose(sort) 0
	set choose(filter) ""
	set choose(endfilter) "*"
	set choose(get) [expr \"$op\" == \"get\"]
	# resultat
	set choose(file) ""
	set choose(dir)  ""
	
	# widgets
	frame $w.file -bd 10
	frame $w.fbut -bd 10
	pack $w.file $w.fbut -side left -fill y -expand 1

	frame $w.file.eframe
	frame $w.file.sframe
	# read-only : juste le chemin courant
	entry $w.file.dirlabel -textvar choose(dir) \
		-font $ffc -bd 2 -relief groove -state disabled
#
# A JETER (ca marche pas)
#	trace variable $choose(dir) w "choose:tracedirlabel"
#	proc choose:tracedirlabel {_n1 _n2 _op} {
#		global choose
#		$choose(dirlabel) xview end
#		}

	pack $w.file.eframe -side top -anch w
	pack $w.file.sframe -side top -fill x
	pack $w.file.dirlabel -side top -anc w -fill x

	label $w.file.eframe.label -anchor w -width 24 -text $label -font $ffc
	entry $w.file.eframe.entry -relief sunken -font $ffc -backgr ivory3 \
		-textvar choose(file)
	label $w.file.eframe.lfilt -anchor w -width 24 -text [lang: CHO.incr-filter] -font $ffc
	entry $w.file.eframe.efilt -relief sunken -font $ffc -backgr ivory2 \
		-textvar choose(filter)

	pack $w.file.eframe.label -side top -expand 1 -anch w
	pack $w.file.eframe.entry -side top -fill x -anch w
	pack $w.file.eframe.lfilt -side top -expand 1 -anch w
	pack $w.file.eframe.efilt -side top -fill x -anch w

	scrollbar $w.file.sframe.yscroll -relief sunken \
		-command "$w.file.sframe.list yview"
	listbox $w.file.sframe.list -relief sunken -font $ffc \
		-yscroll "$w.file.sframe.yscroll set" 

	pack $w.file.sframe.yscroll -side right -fill y
	pack $w.file.sframe.list -side left -fill both -expand 1

	#
	# boutons
	#
	frame $w.fbut.fok -borderwidth 2 -relief sunken
	
	button $w.fbut.fok.bok -text [lang: ok] -relief raised -font $ffc \
			-command "stdfile:get:Ok $w \"$cmd\""
	button $w.fbut.bcan -text [lang: cancel] -relief raised -padx 10 -font $ffc \
			-command "stdfile:get:Cancel $w"
	button $w.fbut.bup -text [lang: up] -relief raised -padx 10 -font $ffc \
			-command "stdfile:get:Up"

	pack $w.fbut.fok.bok -padx 2 -pady 2 -fill x
	
	pack $w.fbut.fok -fill x -pady 1m
	pack $w.fbut.bcan $w.fbut.bup -fill x -side top -pady 1m

	set f $w.fbut
	checkbutton $f.cbnodot -text [lang: CHO.hide-dot-files] \
			-variable choose(nodots) \
			-command {$choose(cmd) $choose(dir)}
	checkbutton $f.cbodirs -text [lang: CHO.only-dirs] \
			-variable choose(onlydirs) \
			-command {$choose(cmd) $choose(dir)}
	checkbutton $f.cbsort -text [lang: CHO.alpha-sort] \
			-variable choose(sort) \
			-command {$choose(cmd) $choose(dir)}
	pack $f.cbnodot -fill x -side bottom
	pack $f.cbodirs $f.cbsort -fill x -side bottom

	#
	# Fill the listbox with a list made from the directory
	#
	set choose(dir) [pwd]
	$choose(dirlabel) xview end
	$choose(cmd) $choose(dir)

	#
	# Set up bindings for the browser.
	#
	bind $w <Return> {$choose(ok) invoke}
	bind $w <Escape> {$choose(cancel) invoke}

	foreach event [lmatch-glob [bind Entry] *Key*] {
		bind $choose(efilter) $event "
			[bind Entry $event]
			\$choose(cmd) \$choose(dir)
			break"
		}
	bind $choose(efilter) <Return> {stdfile:setifsingle ; break}
	bind $choose(efilter) <Shift-Return> {
		set choose(endfilter) ""
		$choose(cmd) $choose(dir)
		break
		}
	bind $choose(efilter) <Tab> { }
	bind $choose(efilter) <Shift-Tab> { }
	bind $choose(efilter) <Escape> { }

	bind $choose(list) <Button-1> {
		eval [bind Listbox <Button-1>]
#3		%W select from [%W nearest %y]
		$choose(entry) delete 0 end
		$choose(entry) insert 0 [string range [%W get [%W nearest %y]] 2 end]
		}
	bind $choose(list) <Double-ButtonPress-1> {
#3		%W select from [%W nearest %y]
		$choose(entry) delete 0 end
		$choose(entry) insert 0 [string range [%W get [%W nearest %y]] 2 end]
		$choose(ok) invoke
		}

	# set kbd focus to entry widget
	focus $choose(entry)

	if {$cmd == ""} {
		tkwait window $w
		return $choose(file)
		}
	return ""
}

# -----------------------------------------------------------------------------
# On libere les variables et on tue la fenetre
# -----------------------------------------------------------------------------
proc stdfile:free {w} {
	destroy $w
}

# -----------------------------------------------------------------------------
# Met dans l'entry l'unique element de la liste sinon ne fait rien.
# -----------------------------------------------------------------------------
proc stdfile:setifsingle {} {
	global choose
	if {[$choose(list) size] == 1} {
		set elt [string range [$choose(list) get 0] 2 end]
		$choose(entry) delete 0 end
		$choose(entry) insert 0 $elt
		$choose(ok) invoke
		}
}

# -----------------------------------------------------------------------------
# Bouton Cancel
# -----------------------------------------------------------------------------
proc stdfile:get:Cancel {w} {
	global choose
	set choose(file) "--NONE--"
	set choose(dir)  "--NONE--"
	stdfile:free $w
}

# -----------------------------------------------------------------------------
# Bouton Up
# -----------------------------------------------------------------------------
proc stdfile:get:Up {} {
	global choose
	$choose(entry) delete 0 end
	$choose(entry) insert 0 ".."
	$choose(ok) invoke
}

# -----------------------------------------------------------------------------
# Bouton OK ou quand on fait double-clic
# -----------------------------------------------------------------------------
proc stdfile:get:Ok {w cmd} {
	global choose
	set selected [$choose(entry) get]

	#
	# some nasty file names may cause "file isdirectory" to return an error
	#
	set sts [catch {file isdirectory $selected} errorMessage]
	if { $sts != 0 } then {
		stdfile:get:errorHandler $errorMessage
		stdfile:free $w
		return
		}
	#
	# Si c'est un directory, on affiche son contenu (sauf .) et on sort
	# TMP:
	# il y aura un probleme avec "." quand on choisi un directory
	###if {[file isdirectory $selected]  &&  "$selected" != "."}### 
	if {[file isdirectory $selected]} {
		if { [file readable $selected] && [file executable $selected] } {
			cd $selected
			set dir [pwd]
			set choose(dir) $dir
			$choose(dirlabel) xview end
			set choose(file) ""
			set choose(filter) ""
			$choose(cmd) $dir
			return
			} \
		else {
			set lab [lang: CHO.unreadable-dir]
			dial:alert [format $lab $selected] [lang: CHO.select-file-error]
			return
			}
		}
	#
	# Sinon c'est un fichier (faut verifier)
	# par defaut, on ne fait que remplir result et tuer le dialogue
	#
	if { $choose(get) } {
		if { ! [file exists $selected] } {
			set lab [lang: CHO.unexist-file]
			dial:alert [format $lab $selected] [lang: CHO.select-file-error]
			return
			}
		if { ! [file readable $selected] } {
			set lab [lang: CHO.unreadable-file]
			dial:alert [format $lab $selected] [lang: CHO.select-file-error]
			return
			}
		} \
	else {
		# sfputfile
		if { [file exists $selected]  &&  ! [file writable $selected] } {
			set lab [lang: CHO.unwritable-file]
			dial:alert [format $lab $selected] [lang: CHO.select-file-error]
			return
			}
		}

	if {$cmd == ""} {
		set choose(file) $selected
		stdfile:free $w
		return
		}
	$cmd -whofor- $selected
}

#
#   fichier
# / directory
#
proc stdfile:listfiles dir {
	global choose
	$choose(list) delete 0 end
	
	set filtre "$choose(filter)$choose(endfilter)"
	if $choose(nodots) {
		set L [glob -nocomplain -- $filtre]
		} \
	else {
		set L [lmatch-glob [glob -nocomplain -- .* *] $filtre]
		# on vire .
		if {[string compare [lindex $L 0] "."] == 0} {
			set L [lrange $L 1 end]
			}
		}
	set L [lsort $L]

	if $choose(onlydirs) {
		set Ld {}
		foreach i $L {
			if [file isdirectory $i] {
				lappend Ld $i
				}
			}
		set L $Ld
		}

	if $choose(sort) {
		set L [lsort -command choose:nccmp $L]
		}

	foreach i $L {
		if [file isdirectory $i] {
			$choose(list) insert end "/ $i"
			} \
		else {
			$choose(list) insert end "  $i"
			}
		}
	set choose(endfilter) *
}

#
#   fichier
# / directory
#
proc stdfile:listdirs dir {
	global choose
	$choose(list) delete 0 end

	set olddir [pwd]
	cd $dir

	if $choose(nodots) {
		set L [glob -nocomplain -- *]
		$choose(list) insert end "/ ."
		$choose(list) insert end "/ .."
		} \
	else {
		set L [glob -nocomplain -- .* *]
		}

	if $choose(sort) {
		set L [lsort -command choose:nccmp $L]
		}

	foreach i $L {
		if [file isdirectory $i] {
			$choose(list) insert end "/ $i"
			}
		}
	cd $olddir
}

#
# Pour comparer deux chaines (nom de fichier) en ignorant la case
# sert dans:
#     lsort -command choose:nccmp $List
#
proc choose:nccmp {A B} {
	return [string compare [string tolower $A] [string tolower $B]]
}

# -- EOF choose.tcl -----------------------------------------------------------
