# -- commun.tcl
# Bindings des touches (editeurs, menu, ...) pour tous
# (mouvements et selections)
#
# Auteur:               Pascal Bouvier
# Creation:             25/09/96
# Modifications:
#
# (c) PrologIA 1996
# -----------------------------------------------------------------------------


@com {
On definit les classes d'evenements E :
- ted_common
- ted_motif
- ted_emacs
- ted_mac

- tty_common
- tty_motif
- tty_emacs
- tty_mac

L'enchainement des bindings est :
	.ed.t   *_common   <E>   moved   .ed   all 

au lieu de :
	.ed.t   Text   .ed   all


Syntaxe :

	bind:addkey  classe  event  nom  script

Exemple :

	bind:addkey ted_common <Shift-Key-Right> select-to-next-char {
		tkTextKeySelect %W [%W index {insert + 1c}]
		}
}



#
# GENERAL
#

bind:addkey ted_common <Key-KP_Enter> none {# nothing}

bind:addkey ted_common <Key-Escape> none {# nothing}

bind:addkey ted_common <Control-Key> none {# nothing}

bind:addkey ted_common <Meta-Key> none {# nothing}

bind:addkey ted_common <Alt-Key> none {# nothing }


#
# KEY F6 (matching-pairs  {} () and [])
#

bind:addkey ted_common <Key-F6> balance {
    edc:balance %W
}


#
# KEY SELECT
#

bind:addkey ted_common <Shift-Key-Select> select-to-here {
    set tkPriv(selectMode) char
    tkTextKeyExtend %W insert
}


bind:addkey ted_common <Key-Select> select-from-here {
    %W mark set anchor insert
}


#
# KEY TAB FOCUS
#


bind:addkey ted_common <Control-Shift-Key-Tab> previous-focus {
    focus [tk_focusPrev %W]
}

bind:addkey ted_common <Control-Key-Tab> next-focus {
    focus [tk_focusNext %W]
}

bind:addkey ted_common <Shift-Key-Tab> none {
    # Needed only to keep <Tab> binding from triggering;  doesn't
    # have to actually do anything.
}



#
# KEYS END HOME 
#

bind:addkey ted_common <Control-Shift-Key-End> select-to-end {
    tkTextKeySelect %W {end - 1 char}
}

bind:addkey ted_common <Control-Key-End> goto-end {
    tkTextSetCursor %W {end - 1 char}
}

bind:addkey ted_common <Control-Shift-Key-Home> select-to-begin {
    tkTextKeySelect %W 1.0
}

bind:addkey ted_common <Control-Key-Home> goto-begin {
    tkTextSetCursor %W 1.0
}

bind:addkey ted_common <Shift-Key-End> select-to-eol {
    tkTextKeySelect %W {insert lineend}
}

bind:addkey ted_common <Key-End> goto-eol {
    tkTextSetCursor %W {insert lineend}
}

bind:addkey ted_common <Shift-Key-Home> select-to-bol {
    tkTextKeySelect %W {insert linestart}
}

bind:addkey ted_common <Key-Home> goto-bol {
    tkTextSetCursor %W {insert linestart}
}


#
# KEYS PREV NEXT (PAGES) 
#

bind:addkey ted_common <Control-Key-Next> see-next-page {
    %W yview scroll 1 page
}

bind:addkey ted_common <Control-Key-Prior> see-previous-page {
    %W yview scroll -1 page
}

bind:addkey ted_common <Shift-Key-Next> select-to-next-page {
    tkTextKeySelect %W [tkTextScrollPages %W 1]
}

bind:addkey ted_common <Key-Next> goto-next-page {
    tkTextSetCursor %W [tkTextScrollPages %W 1]
}

bind:addkey ted_common <Shift-Key-Prior> select-to-previous-page {
    tkTextKeySelect %W [tkTextScrollPages %W -1]
}

bind:addkey ted_common <Key-Prior> goto-previous-page {
    tkTextSetCursor %W [tkTextScrollPages %W -1]
}


#
# KEY UP DOWN LEFT RIGHT (ARROWS)
#

bind:addkey ted_common <Control-Shift-Key-Down> select-to-next-para {
    tkTextKeySelect %W [tkTextNextPara %W insert]
}

bind:addkey ted_common <Control-Shift-Key-Up> select-to-previous-para {
    tkTextKeySelect %W [tkTextPrevPara %W insert]
}

bind:addkey ted_common <Control-Shift-Key-Right> select-to-next-word {
    tkTextKeySelect %W [%W index {insert wordend}]
}

bind:addkey ted_common <Control-Shift-Key-Left> select-to-previous-word {
    tkTextKeySelect %W [%W index {insert - 1c wordstart}]
}

bind:addkey ted_common <Control-Key-Down> goto-next-para {
    tkTextSetCursor %W [tkTextNextPara %W insert]
}

bind:addkey ted_common <Control-Key-Up> goto-begin-para {
    tkTextSetCursor %W [tkTextPrevPara %W insert]
}

bind:addkey ted_common <Control-Key-Right> goto-next-word {
    tkTextSetCursor %W [%W index {insert wordend}]
}

bind:addkey ted_common <Control-Key-Left> goto-previous-word {
    tkTextSetCursor %W [%W index {insert - 1c wordstart}]
}

bind:addkey ted_common <Shift-Key-Down> select-to-next-line {
    tkTextKeySelect %W [tkTextUpDownLine %W 1]
}

bind:addkey ted_common <Shift-Key-Up> select-to-previous-line {
    tkTextKeySelect %W [tkTextUpDownLine %W -1]
}

bind:addkey ted_common <Shift-Key-Right> select-to-next-char {
    tkTextKeySelect %W [%W index {insert + 1c}]
}

bind:addkey ted_common <Shift-Key-Left> select-to-previous-char {
    tkTextKeySelect %W [%W index {insert - 1c}]
}

bind:addkey ted_common <Key-Down> goto-next-line {
    tkTextSetCursor %W [tkTextUpDownLine %W 1]
}

bind:addkey ted_common <Key-Up> goto-previous-line {
    tkTextSetCursor %W [tkTextUpDownLine %W -1]
}

bind:addkey ted_common <Key-Right> goto-next-char {
    tkTextSetCursor %W insert+1c
}

bind:addkey ted_common <Key-Left> goto-previous-char {
    tkTextSetCursor %W insert-1c
}


#
# BUTTONS
#


bind:addbutton ted_common <B2-Motion> scan-text {
    if !$tk_strictMotif {
	if {(%x != $tkPriv(x)) || (%y != $tkPriv(y))} {
	    set tkPriv(mouseMoved) 1
	}
	if $tkPriv(mouseMoved) {
	    %W scan dragto %x %y
	}
    }
}

bind:addbutton ted_common <Button-2> button2 {
    if !$tk_strictMotif {
	%W scan mark %x %y
	set tkPriv(x) %x
	set tkPriv(y) %y
	set tkPriv(mouseMoved) 0
    }
}


bind:addbutton ted_common <Control-Button-1> goto-click-keep-selection {
    %W mark set insert @%x,%y
}

bind:addbutton ted_common <ButtonRelease-1> none {
    tkCancelRepeat
}

bind:addbutton ted_common <B1-Enter> none {
    tkCancelRepeat
}

bind:addbutton ted_common <B1-Leave> none {
    set tkPriv(x) %x
    set tkPriv(y) %y
    tkTextAutoScan %W
}

bind:addbutton ted_common <Triple-Shift-Button-1> select-to-click-line {
    set tkPriv(selectMode) line
    tkTextSelectTo %W %x %y
}

bind:addbutton ted_common <Double-Shift-Button-1> select-to-click-word {
    set tkPriv(selectMode) word
    tkTextSelectTo %W %x %y
}

bind:addbutton ted_common <Shift-Button-1> select-to-click-char {
    tkTextResetAnchor %W @%x,%y
    set tkPriv(selectMode) char
    tkTextSelectTo %W %x %y
}

bind:addbutton ted_common <Triple-Button-1> select-line {
    set tkPriv(selectMode) line
    tkTextSelectTo %W %x %y
    catch {%W mark set insert sel.first}
}

bind:addbutton ted_common <Double-Button-1> select-word {
    set tkPriv(selectMode) word
    tkTextSelectTo %W %x %y
    catch {%W mark set insert sel.first}
}

bind:addbutton ted_common <B1-Motion> select {
    set tkPriv(x) %x
    set tkPriv(y) %y
    tkTextSelectTo %W %x %y
}

bind:addbutton ted_common <Button-1> goto-click {
    tkTextButton1 %W %x %y
    %W tag remove sel 0.0 end
}



# -- EOF commun.tcl ----------------------------------------------------------
