# -- dbgcom.tcl
# Quoi: 
# Les commandes du debugger Prolog IV
#
# Auteur:			Pascal Bouvier
# Creation:			07/05/95
# Modifications:	none
# 
# (c) PrologIA 1995
# -----------------------------------------------------------------------------


# -----------------------------------------------------------------------------
# Commandes de base.
# -----------------------------------------------------------------------------
proc dbgcom:step {{opt ""}} {
	dbg:envoie "step $opt \n"
}

proc dbgcom:next {{opt ""}} {
	dbg:envoie "next $opt \n"
}

proc dbgcom:cont {{opt ""}} {
	dbg:envoie "cont $opt \n"
}

proc dbgcom:where {} {
	dbg:envoie "where\n"
}

proc dbgcom:chpt {} {
	dbg:envoie "chpt\n"
}

proc dbgcom:up {} {
	dbg:envoie "up\n"
}

proc dbgcom:down {} {
	dbg:envoie "down\n"
}

proc dbgcom:killquery {} {
	dbg:envoie "kill\n"
}

# -----------------------------------------------------------------------------
# Ecrit la valeur des variables
# -----------------------------------------------------------------------------
proc dbgcom:printvars {} {
	dbg:envoie "printvar\n"
}

# -----------------------------------------------------------------------------
# Ecrit la valeur d'une variable selectionnee,
# ou toutes si la selection est vide.
# -----------------------------------------------------------------------------
proc dbgcom:printselvar {} {
	if {[catch {selection get} sel]} {
		set sel ""
		}
	dbg:envoie "printvar [string trim $sel] \n"
}


# -----------------------------------------------------------------------------
# Ecrit la valeur des variables (avec la simplification)
# -----------------------------------------------------------------------------
proc dbgcom:sdvars {} {
	dbg:envoie "sd\n"
}

# -----------------------------------------------------------------------------
# Ecrit la valeur d'une variable selectionnee, (avec la simplification)
# ou toutes si la selection est vide.
# -----------------------------------------------------------------------------
proc dbgcom:sdselvar {} {
	if {[catch {selection get} sel]} {
		set sel ""
		}
	dbg:envoie "sd [string trim $sel] \n"
}

# -----------------------------------------------------------------------------
# Met un spy sur le predicat selectionne,
# ou la boite courant si la selection est vide
# -----------------------------------------------------------------------------
proc dbgcom:spysel {} {
	if {[catch {selection get} sel]} {
		set sel ""
		}
	dbg:envoie "spy \{[string trim $sel]\} \n"
}

# -----------------------------------------------------------------------------
# Le lancement effectif de l'executable
# On ouvre le fichier pipe
# TMP:
# -----------------------------------------------------------------------------
#proc dbgcom:oldrun {} {
#	global dbgvar f1
#	catch {dbgcom:kill}
#	dbg:puts_error "\n-------- Running $dbgvar(nameexec) --------\n"
#	if {[catch "open \"$dbgvar(execpipe)\" {RDWR NONBLOCK}" f1]} {
#		return 0
#		}
#	process:record [pid $f1]
#	fcntl $f1 NONBLOCK 1
#	set dbgvar(isrunning) 1
#	set dbgvar(istalking) 1
#	dbg_g
#	return 1
#}

# -----------------------------------------------------------------------------
# L'arret definitif et brutal de l'executable
# -----------------------------------------------------------------------------
#proc dbgcom:oldkill {} {
#	global f1 dbgvar
#	# SIGTERM (terminate)
#	kill SIGTERM [pid $f1]
#	set dbgvar(isrunning) 0
#	dbg:setstate "Ready to run"
#}

# -----------------------------------------------------------------------------
# Indique l'exe qu'on va debugger (ne le lance pas)
# (il faudrait breaker sur le lancement (to_begin ?) )
# il faudrait au moins montrer le source du lancement
# Verifie l'existance et l'executabilite
# -----------------------------------------------------------------------------
#proc dbgcom:debug {exe} {
#	global dbgvar
#	catch {dbgcom:kill}
#	# TMP: 2
#	set dbgvar(nameexec) $exe
#	if { "[string index $exe 0]" == "/"} {
#		set fexe $dbgvar(nameexec)
#		} \
#	else {
#		set fexe $dbgvar(pathexec)/$dbgvar(nameexec)
#		}
#	if { ![file exists $fexe] \
#		|| ![file isfile $fexe] \
#		|| ![file readable $fexe] \
#		|| ![file executable $fexe] } {
#		dbg:puts_error "Can't run file \"$fexe\""
#		return 0
#		}
#	set dbgvar(execpipe) "|$fexe -g -tcl"
#	dbg:setstate "Ready to run"
#	return 1
#}

# -----------------------------------------------------------------------------
# On charge le fichier texte (menu.px)
# TMP: faudrait passer par les fonctions de l'editeur pour charger
# -----------------------------------------------------------------------------
proc dbg:loadfile {fullname name} {
	global dbgvar
	set f [open $fullname r]
	.dbgp4.frEdit.t delete 1.0 end
	.dbgp4.frEdit.t insert insert [read $f]
	close $f
	set dbgvar(currin_file) $name
}

# -----------------------------------------------------------------------------
# On change de fichier source name : 
#	- on le montre dans la fenetre et
#	- on charge les tables qui lui sont associees
# -----------------------------------------------------------------------------
proc dbgcom:file {name {masterfile {}} {incl 0}} {
	global dbgvar
	if [is-fullpath $name] {
		set fullname $name
		} \
	else {
		set fullname $dbgvar(pathexec)/$name
		}

	if {! [is-existing&readable&regular $fullname] } {\
		dbg:puts_error "Can't open file $fullname"
		return 0
		}
	dbg:loadfile $fullname $name

	if {"$masterfile" == ""} {
		set mastinc [dbg_find_master $name]
		if {$mastinc == ""} {
			set masterfile $name
			set incl 1
			} \
		else {
			lassign $mastinc masterfile incl
			}
		}
	if {$incl == 0} {
		dbg:puts_error "Bad argument incl in 'dbgcom:file' command"
		}

	#
	# Lit les tables des regions de texte
	#
	set n [file root $masterfile]
	if [is-fullpath $n] {
		set fullname $n
		} \
	else {
		set fullname $dbgvar(pathexec)/$n
		}
	dbg_read_xdbg_table $fullname.sym

	#
	# Lit la table dans le fichier .c pour montrer les litteraux debuggables
	#
# TMP:
#	dbg_read_cdbg_table $fullname.g.c
#	global $fullname.g.c
#	dbg_showstoppable [set $fullname.g.c] $n.px $incl
	return 1
}


#
# Vire toutes les colorations de ports
#
proc dbg:remove_tagport {} {
	foreach p {CALL EXIT EXITED REDO FAIL OK NO CALLER RULE} {
		catch {.dbgp4.frEdit.t tag remove stopped$p 1.0 end}
		}
}

#
#
#
proc dbg:add_tagport {port deb fin} {
	.dbgp4.frEdit.t tag add stopped$port $deb $fin
	.dbgp4.frEdit.t yview "stopped$port.first -10l"
}

# -----------------------------------------------------------------------------
# Met a jour le debugger source en fonction de l'endroit ou on est dans le
# debugger boites.
# Pour l'instant, file est de la forme fic/0  (nrev.px/0)
# info est un grand nombre (1000008)
# port elt {CALL EXIT EXITED REDO FAIL OK NO CALLER}
# -----------------------------------------------------------------------------
proc dbgcom:showline {file info port} {
	global dbgvar
# avant, on avait toto.p4/0 
#	set file [string range $file 0 [expr [string last / $file]-1]]

	catch "dbg:init"
#Test
	update idletasks

# TMP:
	puts $file:$info:$port
# TMP:
	dbg:remove_tagport
	if { $info < 0 } {
		dbg:puts_error "\"$file\" : No source line available ($info)"
		set dbgvar(stopin_line) ?
		set dbgvar(stopin_file) $file
		} \
	else {
		if { $info > 999999 } {
			set idx $info
			global dbg dbginclude
			set incl [expr $info / 1000000]
			if { $incl != 1 } {
#				set fichier [lindex $dbginclude($file,$incl) 0]
				if { [catch "lindex \$dbginclude($file,$incl) 0" fichier] } {
					dbgcom:file $file $file 1
					if { [catch "lindex \$dbginclude($file,$incl) 0" fichier] } {
						dbg:puts_error "dbg: I'am lost with all these files ! ($file)"
						return
						}
					}
				} \
			else {
				set fichier $file
				}
			if { "$fichier" != "$dbgvar(currin_file)" } {
				dbgcom:file $fichier $file $incl
				}
			set dbgvar(stopin_file) $fichier

			set couple [lindex $dbg($file,$incl) [expr $idx % 1000000]]
			lassign $couple deb fin
			set dbgvar(stopin_line) [expr int($deb)]
			dbg:add_tagport $port $deb $fin
			}
		}
}

proc dbgcom:killtable {s} {
}
proc dbgcom:killunit {s} {
}



# -- EOF dbgcom.tcl -----------------------------------------------------------
