# -- debug.tcl
# Debugger Prolog IV
#
# Auteur:		Pascal Bouvier
# Creation:		../08/94
# Modifications:	
#				PB 07/05/95 Deuxieme jet (changements/integration)
#
# (c) PrologIA 1995
# -----------------------------------------------------------------------------

global srcdir

source $srcdir/dbgcom.tcl
source $srcdir/dbgtop4.tcl

#
# c'est la version "avec compile" du debugger Prolog IV
#
set dbg:version 2.0

#
# le tableau des variables du debugger graphique
#
# dbgvar(state)				# "State: Off"
# dbgvar(dir) 				# ?? repertoire courant ??
# dbgvar(stopin_file)		#
# dbgvar(stopin_predicate)	#
# dbgvar(stopin_line)		#
# dbgvar(currin_file)		# currently in ...
# dbgvar(currin_predicate)	#
# dbgvar(currin_line)		#

# -----------------------------------------------------------------------------
# Appelee une seule fois, pour la creation de la fenetre
# -----------------------------------------------------------------------------
proc dbg:init {} {
	global dbg:version super:bitmapdir

	#
	# Fenetre principale
	#
	set d .dbgp4
	toplevel $d
	wm title $d "Prolog IV Debugger v${dbg:version}"
	wm minsize $d 0 0 
	wm iconbitmap $d @${super:bitmapdir}/p4dbg.icn
	wm iconname $d "DBG IV"
# TMP:
# pour remettre current_wout comme il faut:
	wm protocol $d WM_DELETE_WINDOW {\
		if { ${conn:current_wout} == ".dbgp4.frtty.t" } {
			set conn:current_wout .cons.frtty.t
			}
		destroy .dbgp4
		}
	
	#
	# Les variables du debugger
	#
	dbg:init_fields

	# frame menu
	frame $d.fm -relief raised -bd 1
	# frame directory
	frame $d.fdir
	# frame stopped in file
	frame $d.fsif
	# frame currently in file
	frame $d.fcif
	#
	# frame texte source
	#
	set fsrc [dbg:init:sourcefile $d]
##	frame $d.ft
	# frame buttons
	frame $d.fb1 -relief sunken -bd 1
	frame $d.fb2 -relief sunken -bd 1

	#
	# frame debugger I/O
	#
	set ftty [dbg:init:dbginout $d]
##	frame $d.fd

	# label status
	label $d.ls -textvar dbgvar(state)
	
# TMP:
	pack $d.fm -side top -fill x
	pack $d.fdir -side top -fill x -padx 1m
	pack $d.fsif -side top -fill x -padx 1m
	pack $d.fcif -side top -fill x -padx 1m

	pack $fsrc -side top -fill both -expand 1

	pack $d.fb1 -side top -fill x -padx 1m
	pack $d.fb2 -side top -fill x -padx 1m
	pack $ftty -side top -fill both -expand 1
	pack $d.ls -side right -padx 1m

	#
	# Diverses choses temporaires
	#
	global conn:current_wout
	radiobutton $d.fm.cb -text [lang: PCON.output-here] -variable conn:current_wout  \
		-activeback #eed5b7 -activeforegr Black \
		-fg Black -selectcolor green \
		-value $ftty.t
#		-command {if $dbgvar(istalking) dbg_g}
	button $d.fm.debug -text [lang: PCON.debug] -command {conn:debug-on} \
		-bd 1 -padx 5 -pady 0
	button $d.fm.stop -text [lang: PCON.stop] -command {conn:ctrl-c} \
		-bd 1 -padx 5 -pady 0
	pack $d.fm.stop $d.fm.debug $d.fm.cb -padx 2m -side right

	#
	# L'interieur des frames
	#
	dbg:init:buttons1 $d.fb1
	dbg:init:buttons2 $d.fb2
#	dbg:init:menus $d.fm
	dbg:init:dir $d.fdir
	dbg:init:stopin $d.fsif
	dbg:init:currin	$d.fcif

	#
	# Les extra-tags du texte "source file"
	#
	set t $d.frEdit.t
	dbg:config_tagport $t
}

#
#
#
proc dbg:config_tagport {t} {
	#
	# les endroits ou on pourrait s'arreter
	#
	$t tag config stoppable -background #c0ffc0

	#
	# les ports
	#
	if {[winfo depth .] == 1} {
		#
		# Noir et Blanc
		#
		$t tag config stoppedCALL -relief raised -borderwidth 1 -backgr white
		$t tag config stoppedEXIT -relief raised -borderwidth 1 -backgr white
		$t tag config stoppedEXITED -relief raised -borderwidth 1 -backgr white
		$t tag config stoppedREDO -relief raised -borderwidth 1 -backgr white
		$t tag config stoppedFAIL -relief raised -borderwidth 1 -backgr white
		$t tag config stoppedOK -relief raised -borderwidth 1 -backgr white
		$t tag config stoppedNO -relief raised -borderwidth 1 -backgr white
		$t tag config stoppedCALLER -relief raised -borderwidth 1 -backgr white
		$t tag config stoppedRULE -relief raised -borderwidth 1 -backgr white
		} \
	else {
		#
		# Couleurs
		#
		$t tag config stoppedCALL -background blue -foreground white
		$t tag config stoppedEXIT -background darkgreen -foreground white
		$t tag config stoppedEXITED -background darkgreen -foreground white
		$t tag config stoppedREDO -background purple -foreground white
		$t tag config stoppedFAIL -background red -foreground white
		$t tag config stoppedOK -background green -foreground black
		$t tag config stoppedNO -background red -foreground white
		$t tag config stoppedCALLER -background lightblue -foreground black
		$t tag config stoppedRULE -background lightblue -foreground black
		}

	# sel au dessus de tous
	$t tag raise sel
}

# -----------------------------------------------------------------------------
# Reinitialise a rien les variables d'etat du debugger
# TMP:
#
proc dbg:init_fields {} {
	global dbgvar dbginclude

	#
	# on initialise le tableau des includes
	#
	set dbginclude("") ""

	set dbgvar(state) "State: Off"
	set dbgvar(dir) [pwd]
	
	set dbgvar(stopin_file) ""
	set dbgvar(stopin_predicate) ""
	set dbgvar(stopin_line) ""
	
	set dbgvar(currin_file) ""
	set dbgvar(currin_predicate) ""
	#
	# Le chemin et le nom de l'executable
	#
	set dbgvar(pathexec) "."
	set dbgvar(nameexec) "pascal.g.out"
	
	#
	# Le nom de la commande pipee
	#
	set dbgvar(execpipe) ""
}


# -----------------------------------------------------------------------------
# Frame "source file" (le panel du fichier presente)
# -----------------------------------------------------------------------------
proc dbg:init:sourcefile {root} {
	set fr [ed:newfrEdit $root 0 1 1]
	$fr.t config -hei 15
	ed:setdirty $fr.t ReadOnly
	return $fr
}

# -----------------------------------------------------------------------------
# Frame "debugger input/output"
# -----------------------------------------------------------------------------
proc dbg:init:dbginout {root} {
	set fr [tty:newframe $root]
	$fr.t config -bd 2 -hei 15 -width 70 -relief sunken
#	$fr.sv config -bg #ffe4c4
	return $fr
}

# -----------------------------------------------------------------------------
# frame "buttons"
#
proc dbg:init:buttons1 {f} {
	set L "
	[button $f.b1 -text "Step" -command {dbgcom:step}] \
	[button $f.b2 -text "Next" -command {dbgcom:next}] \
	[button $f.b3 -text "Cont" -command {dbgcom:cont}] \
	[button $f.b5 -text "Spy sel. pred." -command {dbgcom:spysel}] \
	[button $f.b6 -text "Stop at" -state disabled] \
	[button $f.b7 -text "Where" -command {dbgcom:where}] \
	[button $f.b8 -text "Chpt" -command {dbgcom:chpt}] \
	[button $f.b9 -text "Up" -command {dbgcom:up}] \
	[button $f.b10 -text "Down" -command {dbgcom:down}] \
	[button $f.b11 -text "kill" -command {dbgcom:killquery}] \
	"
	eval pack $L -side left -ipadx 1m
}

proc dbg:init:buttons2 {f} {
	set L "
	[button $f.b4b -text "Print vars" -command {dbgcom:printvars}] \
	[button $f.b4 -text "Print sel. var." -command {dbgcom:printselvar}] \
	[button $f.b4t -text "Subdomain of vars" -command {dbgcom:sdvars}] \
	[button $f.b4q -text "Subdomain of sel. var." -command {dbgcom:sdselvar}] \
	"
	eval pack $L -side left -ipadx 1m
}

# Pas fait: OBSOLETE
#	[button $f.b8 -text "Run" -command {dbgcom:run}]
#	[button $f.b9 -text "Kill" -command {dbgcom:kill}]

# -----------------------------------------------------------------------------
# Frame "menus" (bidon pour l'instant, car ne fait rien)
#
proc dbg:init:menus {f} {
	button $f.b1 -text "File"
	button $f.b2 -text "Breakpoint"
	pack $f.b1 $f.b2 -side left -padx 2m -pady 1m
}

# -----------------------------------------------------------------------------
# Frame "directory"
#
proc dbg:init:dir {f} {
	label $f.l -text "Directory:"
	label $f.e -textvar dbgvar(dir) -anchor w
	pack $f.l -side left -padx 1m
	pack $f.e -side left -padx 1m -expand 1 -fill x
}

# -----------------------------------------------------------------------------
# frame "stopped in file"
#
proc dbg:init:stopin {f} {
	label $f.lf -text "Stopped in File:"
	label $f.ef -textvar dbgvar(stopin_file) -anchor w
#	label $f.lp -text "Predicate:"
#	label $f.ep -textvar dbgvar(stopin_predicate)
	label $f.ll -text "Line:"
	label $f.el -textvar dbgvar(stopin_line) -wid 4 -anchor e

	pack $f.lf -side left -padx 1m
	pack $f.ef -side left -fill x -padx 1m -expand 1
	pack $f.ll -side left -padx 1m
	pack $f.el -side right -fill x -padx 1m
}

# -----------------------------------------------------------------------------
# frame "currently in file"
#
proc dbg:init:currin {f} {
	label $f.lf -text "Currently in File:"
	label $f.ef -textvar dbgvar(currin_file) -anchor w
#	label $f.lp -text "Predicate:"
#	label $f.ep -textvar dbgvar(currin_predicate)
	pack $f.lf -side left -padx 1m
	pack $f.ef -side left -fill x -padx 1m -expand 1
}


# ===== D I V E R S ===========================================================


# -----------------------------------------------------------------------------
# Envoie la chaine sur le canal de communication
# -----------------------------------------------------------------------------
proc dbg:envoie s {
	global f1 dbgvar tab_answer
#	if { ! $dbgvar(isrunning) } {
#		dbg:puts_error "No program is running !"
#		return 0
#		}

	#
	# si on a pu demander si le dbg attendait
	set replyid [conn:pro_fun isdbgwaiting "" $f1]
	if {$replyid == 0} {bip;bip;return}

	set answer $tab_answer($replyid)
	unset tab_answer($replyid)
	if {$answer == 0} {bip; return}

	dbg:puts_user $s ""
	dbg:setstate "Executing"
#
# NEW SIGUSR1
	conn:envoie $s $f1
#	envoie $s 
}

proc dbg:setstate {{s Ready}} {
	global dbgvar
	set dbgvar(state) "State: $s"
}

#TMP
proc dbg:setprompt p {
	dbg:puts $p {}
	dbg:setstate
}

#
# Utilisee par la table de symbole (.sym)
#
proc lsetend {i L} {
	global dbg
	foreach e $L {
		lappend dbg($i) $e
		}
}
#
# flegme
#
proc ss {} {destroy .d; source debug.tcl}


#
#-- EOF debug.tcl ------------------







