# -- edit.tcl
# L'editeur multifenetre pour Prolog IV
#
# Auteur:		Pascal Bouvier
# Creation:		../01/95
# Modifications:	
#				PBO ../05/95 2eme chantier
#				PBO 05/10/95 Save All et exit
#				PBO 06/10/95 Arrange les alertes
#				PBO 06/02/96 variable prolog:syntaxp4 pour parse
#				PBO 28/04/96 ReadOnly pour dirty
#				PBO 04/10/96 tk4.1 et remaniement menu et bindings
#				PBO 21/10/98 dans edit:Open on ne testait pas les chemins
#
# (c) PrologIA 1995
# -----------------------------------------------------------------------------


# =========================================================
#	                  L ' E D I T E U R
# =========================================================


# ==== G L O B A L E S =======================================

global tcl_platform

# La globale edit:id est un numero croissant de creation d'objet
global edit:id

# BINP4 est le directory des executables du kit: xxx/bin

# Ces tables gerent les editeurs
global edit_eds
global edit_files

# Listes pour gerer les notifieurs
global edit_callbacks


# Les differents parametrages modifables par l'utilisateur
# indexes par: font, bg, fg, lang
#
global edit:pref
set edit:pref(font) "Courier 12 Bold"
set edit:pref(lang) "Prolog"
set edit:pref(bg) "white"
set edit:pref(fg) "black"

global edit:tmp

global misc:pref
global misc:tmp

if {$tcl_platform(platform) == "unix"} {
	set misc:pref(emacslike) 1
} else {
	set misc:pref(emacslike) 0
	}


# ==== P R O C S =======================================

# -----------------------------------------------------------------------------
# Identification unique d'objets.
# -----------------------------------------------------------------------------
proc edit:newid {} {
	global edit:id
	return [incr edit:id]
}

# -----------------------------------------------------------------------------
# Initialise l'Editeur (qui gere les editeurs)
# retourne le menubouton de l'Editeur, installe.
# -----------------------------------------------------------------------------
proc edit:init {rootfm} {
	global srcdir
	global edit:id
	set edit:id 0
	
	set mf [edit:create_file_menu $rootfm]
#	pack $mf -side left -padx 2m -pady 2m

	source $srcdir/commun.tcl
	source $srcdir/emacs.tcl
	source $srcdir/pc.tcl
#3	source $srcdir/keys.tcl
	source $srcdir/edcom.tcl
	source $srcdir/procolor.tcl
	ed:dyncol:Prolog:init

	#
	# Le scrap global pour le Cut/Copy/Paste
	#
	global edit:scrap
	set edit:scrap {}

	#
	# la classe pour mettre a jour le voyant de position
	#
	bind moved <Key> {[winfo parent %W].fs.pos config -text [%W index insert]}

	#
	# Pour les notifications de creation/destruction d'editeur
	# et la gestion des editeurs
	global edit_eds edit_files edit_callbacks
	set edit_eds(0) 0; unset edit_eds(0)
	set edit_files(0) 0; unset edit_files(0); 
	set edit_callbacks {}

	#
	# Backup des editeurs
	#
	global ed:bkdelayms
	after ${ed:bkdelayms} "ed:backuptexts"

	return $mf
}

# -----------------------------------------------------------------------------
# Sortie de l'Editeur (confirmations, sauve ...)
# retourne 0 si on peut sortir, 1 si on ne veut plus (ou si erreur)
# -----------------------------------------------------------------------------
proc edit:exit {} {
	if [edit:Need_saving] {
		global edit_save
		set d [edit:build_dial_askreview]
		update
		grab -global $d
		tkwait window $d
	
		switch -exact $edit_save {
			CANCEL {
				return 1
				}
			REVIEW {
				if {[edit:Review_unsaved] == 0} {
					return 1
					}
				}
			SAVEALL {
				edit:SaveAll
				}
			DISCARDALL {
				}
			}
		}
	return 0
}

# -----------------------------------------------------------------------------
# Cree le menu 'File' de l'Editeur
# -----------------------------------------------------------------------------
proc edit:create_file_menu {rootfm} {
	upvar #0 font_for_menus f
	set mb $rootfm.mbfile
	set m $mb.mfile
	menubutton $mb -menu $m -text [lang: TB.M-file] \
			-padx 5 -pady 5 -relief raised -bd 2 -font $f

	menu $m
	$m add command -font $f -label [lang: new] \
			-command edit:New
	$m add command -font $f -label [lang: open] \
			-command edit:Open
	$m add sepa
	$m add command -font $f -label [lang: save-all] \
			-command edit:SaveAll
	$m add sepa
	$m add command -font $f -label [lang: close-unedited] \
			-command edit:CloseUnedited
	$m add command -font $f -label [lang: iconify-unedited] \
			-command edit:IconifyUnedited
	return $mb
}

@com {
Cette commande cree une nouvelle fenetre dans laquelle est cree un panel d'edition.
Elle est lancee depuis un menu.
}

# -----------------------------------------------------------------------------
# installe le long titre dans la fenetre
# et nomme egalement l'icone de la fenetre
# icone qu'avec le nom (sans le chemin) PBO 04/03/96
# -----------------------------------------------------------------------------
proc edit:setlongtitle {ed file} {
	set top [winfo toplevel $ed]
	set lt [edit:longtitle $file]
	wm title $top $lt
#	wm iconname $top $lt
	wm iconname $top [edit:shorttitle $file]
}

# -----------------------------------------------------------------------------
# Cree un nouvel editeur.
# Le nom de fichier par defaut est "Untitled-n" du directory courant
# un argument inutile pour coherence avec les autres commandes
# MENU
# -----------------------------------------------------------------------------
proc edit:New {{fromed {}}} {
	set n [edit:newid]
	set ed [ed:new .ed$n]

	set file [pwd]/Untitled-$n

	edit:setlongtitle $ed $file
	edit:recordfile $ed $file
	focus $ed.t
	return $ed
}

# -----------------------------------------------------------------------------
# Ferme un editeur (demande pour sauver)
# retourne 1 si ok, 0 si cancel
# MENU
# -----------------------------------------------------------------------------
proc edit:Close {ed} {
	set can ""
	if {! [edit:isdirty $ed] || [set can [edit:demander_sauver $ed]] != "CANCEL"} {
		edit:unrecordfile $ed
		ed:close_window $ed
		}
	return [expr [string compare $can CANCEL] != 0]
}

# -----------------------------------------------------------------------------
# Ferme les editeurs interactivement
# retourne 1 si ok, 0 si cancel
# MENU
# -----------------------------------------------------------------------------
proc edit:Review_unsaved {} {
	global edit_eds
	foreach ed [array names edit_eds] {
		set top [winfo toplevel $ed]
		wm deiconify $top
		raise $top
		if {[edit:Close $ed] == 0} {
			return 0
			}
		}
	return 1
}

# -----------------------------------------------------------------------------
# Il y a-t-il des editeurs a sauver ? (1 si oui, 0 sinon)
# MENU
# -----------------------------------------------------------------------------
proc edit:Need_saving {} {
	global edit_eds
	foreach ed [array names edit_eds] {
		if { [edit:isdirty $ed] } {
			return 1
			}
		}
	return 0
}

# -----------------------------------------------------------------------------
# Fait choisir un fichier et tente de le charger dans un editeur
# MENU
# -----------------------------------------------------------------------------
proc edit:Open {} {
	set file [stdfile:get [lang: CHO.open-file] ""]
	if {$file == "--NONE--"} return
	global choose
	if {[file pathtype $file] == "relative"} {
		set file [file join $choose(dir) $file]
		}
	edit:open_window $file
}

# -----------------------------------------------------------------------------
# Ouvre un nouvel editeur et y charge le fichier file, a moins qu'il ne soit
# deja montre quelque part.
# -----------------------------------------------------------------------------
proc edit:open_window {file} {
	if {[set ed [edit:deja_ouvert $file]] != ""} {
		set top [winfo toplevel $ed]
		raise $top
		wm deiconify $top
		return $ed
		}
	set ed [edit:New]

	if { ! [ed:load_file $ed $file] } {
		edit:Close $ed
		return
		}
	global $ed
	set ${ed}(named) 1

	edit:unrecordfile $ed
	edit:setlongtitle $ed $file
	edit:recordfile $ed $file
	return $ed
}

# -----------------------------------------------------------------------------
# Fait choisir un fichier et le charge dans un editeur donne
# Demande confirmation si modifications anterieures non-sauvees
# MENU
# -----------------------------------------------------------------------------
proc edit:LoadFile {ed} {
	if { ! [edit:Clear $ed] } return

	set file [stdfile:get [lang: CHO.load-file] ""]
	if {$file == "--NONE--"} return
	global choose
	set file $choose(dir)/$file

	if { ! [ed:load_file $ed $file] } return

	global $ed
	set ${ed}(named) 1

	edit:unrecordfile $ed
	edit:setlongtitle $ed $file
	edit:recordfile $ed $file

	focus $ed.t
	$ed.t mark set insert 1.0
	eda:displayinsert $ed.t
	return $ed
}

# -----------------------------------------------------------------------------
# Vide un Editeur (se remet en "Untitled-n")
# Demande et sauver s'il y a lieu
# retourne 1 si ok, 0 si cancelle
# MENU
# -----------------------------------------------------------------------------
proc edit:Clear {ed} {
	if {[edit:isdirty $ed]  &&  [edit:demander_sauver $ed] == "CANCEL"} {
		return 0
		}
	ed:empty_window $ed
	ed:setdirty $ed.t ""
	edit:unrecordfile $ed

	global $ed
	set ${ed}(named) 0

	regexp {.ed([0-9]*)} $ed _bid n
	set file [pwd]/Untitled-$n
	edit:setlongtitle $ed $file
	edit:recordfile $ed $file
	return 1
}

# -----------------------------------------------------------------------------
# Sauve le contenu d'un editeur
# s'il n'a pas encore de nom, appelle SaveAs
# retourne 1 si ok, 0 sinon
# MENU
# -----------------------------------------------------------------------------
proc edit:Save {ed {interactif 1}} {
	global $ed
	if [set ${ed}(named)] {
		#
		# Il a deja un nom
		#
		global edit_eds
		if { ! [ed:save_file $ed $edit_eds($ed)] } {
			if {$interactif} {
				edit:dial:problemesave $ed {}
				}
			return 0
			}
		ed:setdirty $ed.t ""
		return 1
		} \
	else {
		#
		# Il n'a pas encore de nom
		#
		return [edit:SaveAs $ed $interactif]
		}
}

# -----------------------------------------------------------------------------
# Sauve le contenu d'un editeur en le renommant
# demande le nom d'un fichier de sortie
# renomme l'editeur en fonction de ce nom
# retourne 1 si ok, 0 sinon
# MENU
# -----------------------------------------------------------------------------
proc edit:SaveAs {ed {interactif 1}} {
	set file [stdfile:put [lang: CHO.save-as] ""]
	if {$file == "--NONE--"} {return 0}

	global choose
	set file $choose(dir)/$file
	if { ! [ed:save_file $ed $file] } {
		if {$interactif} {
			edit:dial:problemesave $ed $file as
			}
		return 0
		}

	global $ed
	set ${ed}(named) 1

	edit:unrecordfile $ed
	edit:setlongtitle $ed $file
	edit:recordfile $ed $file
	ed:setdirty $ed.t ""
	return 1
}

# -----------------------------------------------------------------------------
# Sauve le contenu de tous les editeurs pas encore sauves.
# retourne 1 si ok, 0 sinon
# MENU
# -----------------------------------------------------------------------------
proc edit:SaveAll {{interactif 1}} {
	global edit_eds
	foreach ed [array names edit_eds] {
		if {[edit:isdirty $ed]} {
			edit:Save $ed $interactif
			}
		}
}

# -----------------------------------------------------------------------------
# Ferme toutes les fenetres non-modifiees
# retourne 1 si ok, 0 sinon
# MENU
# -----------------------------------------------------------------------------
proc edit:CloseUnedited {{interactif 1}} {
	global edit_eds
	foreach ed [array names edit_eds] {
		if {! [edit:isdirty $ed]} {
			edit:Close $ed
			}
		}
}

# -----------------------------------------------------------------------------
# Iconify toutes les fenetres non-modifiees
# retourne 1 si ok, 0 sinon
# MENU
# -----------------------------------------------------------------------------
proc edit:IconifyUnedited {{interactif 1}} {
	global edit_eds
	foreach ed [array names edit_eds] {
		if {! [edit:isdirty $ed]} {
			wm iconify [winfo toplevel $ed]
			}
		}
}

# -----------------------------------------------------------------------------
# Sauve le contenu d'un editeur
# demande le nom d'un fichier de sortie
# retourne 1 si ok, 0 sinon
# MENU
# -----------------------------------------------------------------------------
proc edit:SaveTo {ed {interactif 1}} {
	set file [stdfile:put [lang: CHO.save-to] ""]
	if {$file == "--NONE--"} return

	global choose
	set file $choose(dir)/$file
	if { ! [ed:save_file $ed $file] } {
		if {$interactif} {
			edit:dial:problemesave $ed $file to
			}
		return 0
		}
	return 1
}


# -----------------------------------------------------------------------------
# Relais texte->Editeur
# -----------------------------------------------------------------------------
proc ed:New {t} { return [edit:New [winfo parent $t]] }	
proc ed:Close {t} { return [edit:Close [winfo parent $t]] }	
proc ed:LoadFile {t} { return [edit:LoadFile [winfo parent $t]] }	
proc ed:Clear {t} { return [edit:Clear [winfo parent $t]] }	
proc ed:Save {t} { return [edit:Save [winfo parent $t]] }	
proc ed:SaveAs {t} { return [edit:SaveAs [winfo parent $t]] }	
proc ed:SaveTo {t} { return [edit:SaveTo [winfo parent $t]] }	


# -----------------------------------------------------------------------------
# Appelee si probleme pendant sauve interactif
# -----------------------------------------------------------------------------
proc edit:dial:problemesave {ed fname {verb to}} {
	global edit_eds
	upvar #0 font_for_menus fnt

	if {$fname != ""} {
		set file [wm title [winfo toplevel $ed]]
		set mess "An error occurs while saving window:\n    \"$file\" \n\
			$verb file:\n    \"$fname\"  !"
		} \
	else {
		set file $edit_eds($ed)
		set mess "An error occurs while saving file \"$file\" !"
		}
	dial:alert $mess "Problem during Save"
}


# -----------------------------------------------------------------------------
# Enregistrement/enlevement d'un fichier dans l'Editeur
# Dit si un fichier est deja charge dans un editeur
# -----------------------------------------------------------------------------
proc edit:recordfile {ed file} {
	global edit_files edit_eds edit_callbacks
	set edit_files($file) $ed
	set edit_eds($ed) $file
	foreach cb $edit_callbacks {
		catch "${cb}_record $ed $file"
		}
}
#
proc edit:unrecordfile ed {
	global edit_files edit_eds edit_callbacks
	foreach cb $edit_callbacks {
		catch "${cb}_unrecord $ed"
		}
	unset edit_files($edit_eds($ed))
	unset edit_eds($ed)
}

#
proc edit:recordcallback {cb} {
	global edit_callbacks
	set edit_callbacks [union $edit_callbacks $cb]
}

#
proc edit:unrecordcallback {cb} {
	global edit_callbacks
	set i [lsearch -exact $edit_callbacks $cb]
	if {$i != -1} {
		set edit_callbacks [lreplace $edit_callbacks $i $i]
		}
}

# -----------------------------------------------------------------------------
# Dit si un fichier est deja charge dans un editeur
# -----------------------------------------------------------------------------
proc edit:deja_ouvert file {
	global edit_files
	if {[catch {set edit_files($file)} res] == 1} {
		return ""
		} \
	else {
		return $res
		}
}

#
# Vire les token-tags d'une region donnee
#
# TMP:
proc ed:cleartag {ed deb fin} {
	foreach tag {DELIM DQUOTE QATOM ATOM FLOAT INT VAR LCOMMENT} {
		$ed.t tag remove $tag $deb $fin
		}
}

#tests
proc xxxed:cleartag {ed deb fin} {}


# -----------------------------------------------------------------------------
#
# -----------------------------------------------------------------------------
proc edit:syntax_coloring {ed} {
	global BINP4 prolog:syntaxp4
	$ed config -cursor watch
	update
	ed:setworkstring $ed "Syntax Coloring..."

global tcl_platform
if {$tcl_platform(platform) != "macintosh"} {
	set f [open [set tmp [fs:tmpfile c$ed-[pid]]] w]
		set b 1.0
		set e end
	puts $f [$ed.t get $b $e]
	close $f

	catch {exec $BINP4/newparse -H 50000 -w 5 -r ${prolog:syntaxp4} -colr < $tmp 2> $tmp.err > $tmp.out} res
	errputs $res

	global the_ed
	set the_ed $ed
	foreach tag {DELIM DQUOTE QATOM ATOM FLOAT INT VAR LCOMMENT \
					RULE1 RULE2 RULE3 RULE4} {
		$ed.t tag remove $tag 1.0 end
		}
	source $tmp.out
	$ed config -cursor {}
#3	catch "exec /bin/rm $tmp $tmp.err $tmp.out"
	fs:delete-f $tmp $tmp.err $tmp.out
	}
	ed:setworkstring $ed
	update
}

proc ed:init_tag {ed} {
	if {[winfo depth .] == 1} {
		#
		# Noir et Blanc
		#
		$ed.t tag config DELIM -foreg black
		$ed.t tag config DQUOTE -foreg black
		$ed.t tag config QATOM -foreg black
		$ed.t tag config ATOM -foreg black
		$ed.t tag config FLOAT -foreg black
		$ed.t tag config INT -foreg black
		$ed.t tag config VAR -foreg black
		$ed.t tag config LCOMMENT -foreg black -backg white \
				-borderw 1 -relief raised \
				-font [font: {Times 14 italic}]
#				-font *-Times-Medium-I-Normal-*-140-*
		$ed.t tag config RULE1 -backg white
		$ed.t tag config RULE2 -backg white
		$ed.t tag config RULE3 -backg white
		$ed.t tag config RULE4 -backg white
		} \
	elseif {[winfo depth .] < 8} {
		#
		# 16 couleurs (minable)
		#
		$ed.t tag config DELIM -foreg blue
		$ed.t tag config DQUOTE -foreg purple
		$ed.t tag config QATOM -foreg darkgreen
		$ed.t tag config ATOM -foreg darkgreen
		$ed.t tag config FLOAT -foreg brown
		$ed.t tag config INT -foreg black
		$ed.t tag config VAR -foreg red
		$ed.t tag config LCOMMENT -foreg darkgreen -backg aquamarine \
				-borderw 1 -relief raised \
				-font [font: {Times 14 italic}]
#				-font *-Times-Medium-I-Normal-*-140-*
		$ed.t tag config RULE1 -backg "white"
		$ed.t tag config RULE2 -backg "white"
		$ed.t tag config RULE3 -backg "white"
		$ed.t tag config RULE4 -backg "white"
		} \
	else {
		#
		# 256 Couleurs
		#
		$ed.t tag config DELIM -foreg blue
		$ed.t tag config DQUOTE -foreg purple
		$ed.t tag config QATOM -foreg darkgreen
		$ed.t tag config ATOM -foreg darkgreen
		$ed.t tag config FLOAT -foreg brown
		$ed.t tag config INT -foreg black
		$ed.t tag config VAR -foreg red
		$ed.t tag config LCOMMENT -foreg darkgreen -backg aquamarine \
				-borderw 1 -relief raised \
				-font [font: {Times 14 italic}]
#				-font *-Times-Medium-I-Normal-*-140-*
		$ed.t tag config RULE1 -backg "#ffd0ff"
		$ed.t tag config RULE2 -backg "#ffe0ff"
		$ed.t tag config RULE3 -backg "#a0ffff"
		$ed.t tag config RULE4 -backg "#c0ffff"
		}
		
	$ed.t tag raise sel

	$ed.t tag lower RULE1
	$ed.t tag lower RULE2
	$ed.t tag lower RULE3
	$ed.t tag lower RULE4

	if 0 {
	bind $ed.t <Control-e> "ed:removetagrule $ed.t;\
			$ed.t tag add RULE1 sel.first sel.last"
	bind $ed.t <Control-f> "ed:removetagrule $ed.t;\
			$ed.t tag add RULE2 sel.first sel.last"
	bind $ed.t <Control-E> "ed:removetagrule $ed.t;\
			$ed.t tag add RULE3 sel.first sel.last"
	bind $ed.t <Control-F> "ed:removetagrule $ed.t;\
			$ed.t tag add RULE4 sel.first sel.last"
	}
}

proc ed:removetagrule edt {
	foreach tag {RULE1 RULE2 RULE3 RULE4} {
		$edt tag remove $tag sel.first sel.last
		}
}

set rule 0
#
#offset RELATIFS (ligne.off)
#
proc tcl_rRULE {d f} {
global the_ed rule
	$the_ed.t tag add RULE[expr ($rule % 2) ? 3 : 4] $d $f
	incr rule
update
}
proc tcl_rSPACE {d f} {
global the_ed
	ed:cleartag $the_ed $d $f
}
proc tcl_rDELIM {d f} {
global the_ed
	ed:cleartag $the_ed $d $f
	$the_ed.t tag add DELIM $d $f
}
proc tcl_rGRAPHIC {d f} {
global the_ed
	ed:cleartag $the_ed $d $f
	$the_ed.t tag add ATOM $d $f
}
proc tcl_rCUT {d f} {
global the_ed
	ed:cleartag $the_ed $d $f
	$the_ed.t tag add ATOM $d $f
}
proc tcl_rPVIRG {d f} {
global the_ed
	ed:cleartag $the_ed $d $f
}
proc tcl_rATOM {d f} {
global the_ed
	ed:cleartag $the_ed $d $f
	$the_ed.t tag add ATOM $d $f
}
proc tcl_rINT {d f} {
global the_ed
	ed:cleartag $the_ed $d $f
	$the_ed.t tag add INT $d $f
}
proc tcl_rFLOAT {d f} {
global the_ed
	ed:cleartag $the_ed $d $f
	$the_ed.t tag add FLOAT $d $f
}
proc tcl_rVAR {d f} {
global the_ed
	ed:cleartag $the_ed $d $f
	$the_ed.t tag add VAR $d $f
}
proc tcl_rDQUOTE {d f} {
global the_ed
	ed:cleartag $the_ed $d $f
	$the_ed.t tag add DQUOTE $d $f
}
proc tcl_rBACKQATOM {d f} {
global the_ed
	ed:cleartag $the_ed $d $f
	$the_ed.t tag add DQUOTE $d $f
}
proc tcl_rQATOM {d f} {
global the_ed
	ed:cleartag $the_ed $d $f
	$the_ed.t tag add QATOM $d $f
}
proc tcl_rLCOMMENT {d f} {
global the_ed
	ed:cleartag $the_ed $d $f
	$the_ed.t tag add LCOMMENT $d $f
}

# UNUSED
if 0 {
	#
	# offset ABSOLUS (a partir du debut)
	#
	proc tcl_ATOM {d f} {
	global the_ed
		$the_ed.t tag add ATOM "1.0+$d c" "1.0+$f c"
	}
	proc tcl_INT {d f} {
	global the_ed
		$the_ed.t tag add INT "1.0+$d c" "1.0+$f c"
	}
	proc tcl_FLOAT {d f} {
	global the_ed
		$the_ed.t tag add FLOAT "1.0+$d c" "1.0+$f c"
	}
	proc tcl_VAR {d f} {
	global the_ed
		$the_ed.t tag add VAR "1.0+$d c" "1.0+$f c"
	}
	proc tcl_DQUOTE {d f} {
	global the_ed
		$the_ed.t tag add DQUOTE "1.0+$d c" "1.0+$f c"
	}
	proc tcl_QATOM {d f} {
	global the_ed
		$the_ed.t tag add QATOM "1.0+$d c" "1.0+$f c"
	}
	proc tcl_LCOMMENT {d f} {
	global the_ed
		$the_ed.t tag add LCOMMENT "1.0+$d c" "1.0+$f c"
	}
}

#
# Le dialogue donnant le nombre d'erreur de syntaxe et de warnings
#
proc edit:build_dial_syntaxerror {nbe nbw} {
	upvar #0 font_for_menus fnt
	set W .derr
	toplevel $W
	wm title $W "Errors"
	wm geo $W +400+300

	set fmess $W.fm
	frame $fmess

	#
	# Erreurs
	#
	if {$nbe == 1} { set mtxte "There is a syntax error !"; set cte red } \
	elseif {$nbe}  { set mtxte "There are $nbe syntax errors !!"; set cte red } \
	else           { set mtxte "There is no syntax error"; set cte black }
	#
	# Warnings
	#
	if {$nbw == 1} { set mtxtw "There is a warning !"; set ctw red } \
	elseif {$nbw}  { set mtxtw "There are $nbw warnings !!"; set ctw red } \
	else           { set mtxtw "There is no warning"; set ctw black }

	set n [expr $nbe + $nbw]
	if {$n == 0} {
		set bmp info
		} \
	else {
		set bmp error
		}

	label $fmess.lico -bitmap $bmp
	message $fmess.messe -text $mtxte -font $fnt -aspect 700 -fg $cte
	message $fmess.messw -text $mtxtw -font $fnt -aspect 700 -fg $ctw
	pack $fmess.lico -side left -padx 1m -pady 1m
	pack $fmess.messe $fmess.messw -side top -padx 1m -pady 1m

	set fbut $W.fbut
	frame $fbut -relief groove -bd 2
	pack $fmess $fbut -side top -pady 1m -padx 1m -fill x
	
	# Le bouton 'Ok', par defaut dans son frame 'default'
	frame $fbut.fd -relief sunken -bd 1
	button $fbut.fd.bok -text "Ok" -com "destroy $W" -font $fnt
	pack $fbut.fd.bok -padx 1m -pady 1m -ipadx 2m -ipady 1m
	pack $fbut.fd -side left -pady 3m -padx 1c

	bind $W <Return> "$fbut.fd.bok invoke"
	focus $fbut
	grab $W
	after 5000 catch "{destroy $W}"
	tkwait window $W
}

# -----------------------------------------------------------------------------
# Retourne le nombre d'erreurs de syntaxe
# interactif: 0=pas de dialogue; 1=seulement si erreur; 2=rapport dans tous les cas
# -----------------------------------------------------------------------------
proc edit:check_syntax {ed {sel all} {interactif 0}} {
	global BINP4 prolog:syntaxp4
	edit:clear_all_errors $ed
	$ed.t config -cursor watch
	update

	ed:setworkstring $ed "Checking Syntax..."
	set tmp [fs:tmpfile s${ed}-[pid]]
	set f [open $tmp w]
	if {$sel == "all"} {
		set b 1.0
		set e end
		} \
	else {
		set b sel.first
		set e sel.last
		}
	puts -nonewline $f [$ed.t get $b $e]
	close $f
errputs "exec $BINP4/newparse -H 50000 -w 5 -r ${prolog:syntaxp4} -do -chk -claudops -Wall < $tmp 2> $tmp.err"
	catch {exec $BINP4/newparse -H 50000 -w 5 -r ${prolog:syntaxp4} -do -chk -claudops -Wall < $tmp 2> $tmp.err} res
	set nbe 0
	set nbw 0
	for_file L $tmp.err {
		if {[scan $L "syntax_error on line %d:" i]==1  ||
				[scan $L "warning on line %d:" i]==1  ||
				[scan $L "directive_error on line %d:" i]==1 } {
			incr i -1
#TMP: correction rapide hors newparse (erreur d'une ligne pour les warnings):
			if {[string index $L 0] == "w"} {incr i -1}
			set deb "$b+$i l linestart"
#TMP:		set fin "$b+$i l lineend"
			set fin "$b+$i l lineend +1c"
			$ed.t tag add syntaxerror-$L $deb $fin

			if {[string index $L 0] == "w"} {
				$ed.t tag add warning $deb $fin
				incr nbw
				} \
			else {
				$ed.t tag add erreur $deb $fin
				incr nbe
				}
			}
		}
	$ed.t config -cursor {}
	update

	if {$interactif == 2  ||  $interactif == 1 && $nbe} {
		edit:build_dial_syntaxerror $nbe $nbw
		}

#3	catch "exec /bin/rm $tmp.err $tmp"
	fs:delete-f $tmp.err $tmp
	ed:setworkstring $ed
	return $nbe
}

# -----------------------------------------------------------------------------
# Retourne la premiere position (a partir de pos) erreur ou warning du texte ed.t
# -----------------------------------------------------------------------------
proc ed:next_warning_or_error {ed pos} {
	set u [$ed.t tag nextrange erreur $pos]
	set v [$ed.t tag nextrange warning $pos]
	set u [lindex $u 0]
	set v [lindex $v 0]

	if {$u == ""} {
		return $v
		} \
	elseif {$v == ""} {
		return $u
		} \
	elseif {[$ed.t compare $u < $v]} {
		return $u
		} \
	else {
		return $v
		}
}

# -----------------------------------------------------------------------------
# Il wrappe au debut du fichier
# -----------------------------------------------------------------------------
proc edit:goto_next_error ed {
	set pos [$ed.t index @0,0+[lindex [$ed.t config -hei] 4]l]
	set u [ed:next_warning_or_error $ed $pos]
	if {$u == ""} {
		bip
		set u [ed:next_warning_or_error $ed 1.0]
		if {$u == ""} {
			bip; return
			}
		}
	$ed.t yview -pickplace $u
}

# -----------------------------------------------------------------------------
# Efface toutes les erreurs ou warnings
# -----------------------------------------------------------------------------
proc edit:clear_all_errors ed {
	$ed.mbulle unpost
	$ed.t tag remove erreur 1.0 end
	$ed.t tag remove warning 1.0 end
	set L [lmatch-glob [$ed.t tag names] {syntaxerror-*}]
	foreach i $L {
		$ed.t tag remove $i 1.0 end
		$ed.t tag delete $i
		}
}

# -----------------------------------------------------------------------------
# Efface tous les warnings (pas les erreurs)
# -----------------------------------------------------------------------------
proc edit:clear_warnings ed {
	$ed.mbulle unpost
	$ed.t tag remove warning 1.0 end
	set L [lmatch-glob [$ed.t tag names] {syntaxerror-warning*}]
	foreach i $L {
		$ed.t tag remove $i 1.0 end
		}
}

# -----------------------------------------------------------------------------
# Creation d'un dialogue pour demander "sauver un fichier?"
# retourne ce dialogue
# -----------------------------------------------------------------------------
proc edit:build_dial_asksave {file} {
	upvar #0 font_for_menus fnt

	set W .ds
	toplevel $W
	wm title $W "Save"
	wm geo $W +300+300

	set fmess $W.fm
	frame $fmess
	label $fmess.lico -bitmap questhead
	message $fmess.mess -text "Save Changes for file \"$file\" ?" \
			-font $fnt -wid 7c
	pack $fmess.lico $fmess.mess -side left -padx 5m -pady 1m

	set fbut $W.fb
	frame $fbut -relief groove -bd 2
	pack $fmess $fbut -side top -pady 1m -padx 1m

	# Le bouton 'Sauver', par defaut dans son frame 'default'
	frame $fbut.fd -relief sunken -bd 1
	button $fbut.fd.bok -text "Save" -com "set edit_save YES; destroy $W"
	pack $fbut.fd.bok -padx 1m -pady 1m -ipadx 2m -ipady 1m

	# Le bouton 'Jeter'
	button $fbut.bno -text "Discard" -com "set edit_save NO; destroy $W"
	# Le bouton 'Annuler'
	button $fbut.bcancel -text "Cancel" -com "set edit_save CANCEL; destroy $W"

	pack $fbut.fd -side left -pady 3m -padx 1c
	pack $fbut.bno $fbut.bcancel -side left -pady 3m -padx 1c -ipadx 2m -ipady 1m 
	bind $W <Return> "$fbut.fd.bok invoke"
	bind $W <Escape> "$fbut.bcancel invoke"
	focus $fbut

	return $W
}

# -----------------------------------------------------------------------------
# Retourne YES, NO ou CANCEL
# Sauve le fichier si YES
# Rend CANCEL si sauver se passe mal
# -----------------------------------------------------------------------------
proc edit:demander_sauver ed {
	global edit_save edit_eds
	set edit_save CANCEL

	set d [edit:build_dial_asksave $edit_eds($ed)]
	# Modal
	update
	grab -global $d
	tkwait window $d
	
	if {$edit_save == "CANCEL"} {
		return CANCEL
		} \
	elseif {$edit_save == "YES"} {
		if { ! [edit:Save $ed] } {
			return CANCEL
			}
		return YES
		}
	return NO
}

# -----------------------------------------------------------------------------
# Creation d'un dialogue pour demander "Ya des editeurs a sauver "
# <Review Unsaved> <Save all> <Discard All> <Cancel>
# retourne ce dialogue
# -----------------------------------------------------------------------------
proc edit:build_dial_askreview {} {
	upvar #0 font_for_menus fnt

	set W .ds
	toplevel $W
	wm title $W "Exiting Editors"
	wm geo $W +300+300

	set fmess $W.fm
	frame $fmess
	label $fmess.lico -bitmap questhead
	message $fmess.mess -text "Some Editors need to be saved" \
			-font $fnt -wid 7c
	pack $fmess.lico $fmess.mess -side left -padx 5m -pady 1m

	set fbut $W.fbut
	frame $fbut -relief groove -bd 2
	pack $fmess $fbut -side top -pady 1m -padx 1m
	
	# Le bouton 'Tout Revoir', par defaut dans son frame 'default'
	frame $fbut.fd -relief sunken -bd 1
	button $fbut.fd.bru -text "Review Unsaved" \
			-com "set edit_save REVIEW; destroy $W"
	pack $fbut.fd.bru -padx 1m -pady 1m -ipadx 2m -ipady 1m

	# Le bouton 'Tout sauver'
	button $fbut.bsa -text "Save All" \
			-com "set edit_save SAVEALL; destroy $W"
	# Le bouton 'Quitter'
	button $fbut.bda -text "Quit Anyway" \
			-com "set edit_save DISCARDALL; destroy $W"
	# Le bouton 'Annuler'
	button $fbut.bcancel -text "Cancel" \
			-com "set edit_save CANCEL; destroy $W"

	pack $fbut.fd -side left -pady 3m -padx 5m 
	pack $fbut.bsa $fbut.bda $fbut.bcancel \
			-side left -pady 3m -padx 5m -ipadx 2m -ipady 1m

	bind $W <Return> "$fbut.fd.bru invoke"
	bind $W <Escape> "$fbut.bcancel invoke"
	focus $fbut.fd.bru
	return $W
}

# -----------------------------------------------------------------------------
# Retourne le titre long d'une fenetre
# 	toto.px - /home/p3/dircourant
# 	toto.px - ~p3/dircourant
# -----------------------------------------------------------------------------
proc edit:longtitle {path} {
	global env
	set t [file tail $path]
	set p [file dirname $path]
	if {$p == "."} {return $t}
	set h $env(HOME)
	if {[string first $h $p] == 0} {
		set p ~[string range $p [string len $h] end]
		}
	return [format "%s - %s" $t $p]
}


# -----------------------------------------------------------------------------
# Retourne juste le nom du fichier, sans le chemin pour y arriver
# -----------------------------------------------------------------------------
proc edit:shorttitle {path} {
	return [file tail $path]
}

# -----------------------------------------------------------------------------
# Met la chaine de travail
# -----------------------------------------------------------------------------
proc ed:setworkstring {ed {str ""}} {
	$ed.fs.lwork config -text $str
}

# -----------------------------------------------------------------------------
# Met le message mess dans la ligne de status en bas de l'editeur ed
# -----------------------------------------------------------------------------
proc edit:SetStatus {ed {mess {}}} {
	upvar #0 $ed tab
	set tab(message) $mess
}

# =========================================================
#	L E S   E D I T E U R S
# =========================================================

@com {
Les menus de chaque fenetre editeur:
-----------------------------------------------------------
File  Edit  Find  Syntax  Prolog  ?-Options  ?-Help
-----------------------------------------------------------
}

# -----------------------------------------------------------------------------
# Cree les menus d'un editeur
# -----------------------------------------------------------------------------
proc ed:create_menus {ed} {
	set mf [ed:create_file_menu $ed]
	set me [ed:create_edit_menu $ed]
	set mr [ed:create_find_menu $ed]
	set ms [ed:create_syntax_menu $ed]
	set mp [ed:create_prolog_menu $ed]
	pack $mf $me $mr $ms $mp -side left -padx 2m
#3	tk_menuBar $ed.fm $mf $me $mr $ms $mp
}


@com {
File:
	============
	New Window		^n	# cree une autre fenetre
	Empty				# vide cette fenetre (demande confirm)
	Load File ...		# ouvre un fichier (.px) dans cette fenetre
	Paste File ...		# insere un fichier au pt d'insertion
	Revert to File		# ouvre un fichier (.px) dans cette fenetre
	-----------
	Save 			^s	# Sauve le contenu
	Save As ...		^S	# Sauve le contenu sous un nom (et renomme)
	Save To ...			# Sauve le contenu sous un nom
	-----------
	Close			^w	# Tue la fenetre
	============
}

# -----------------------------------------------------------------------------
# Cree le menu 'File' de l'editeur
# -----------------------------------------------------------------------------
proc ed:create_file_menu {ed} {
	upvar #0 font_for_menus f
	set mb $ed.fm.mbfile
	set m $mb.mfile
	menubutton $mb -menu $m -text [lang: TED.M-file] \
			-underline [lang:u TED.M-file] -font $f

	menu $m
	menu:add $m $ed.t ed New
	menu:add $m $ed.t ed Clear
	menu:add $m $ed.t ed LoadFile
	menu:add $m $ed.t ed PasteFile -state disabled
	menu:add $m $ed.t ed RevertFile -state disabled
	$m add sepa
	menu:add $m $ed.t ed Save
	menu:add $m $ed.t ed SaveAs
	menu:add $m $ed.t ed SaveTo
	$m add sepa
	menu:add $m $ed.t ed Close

	return $mb
}

@com {
Le menu Edit est ce qu'il y a de plus classique pour le mac, a quelques items pres
Edit:
	============
	Undo					#
	-----------
	Cut				^x		#
	Copy			^c		#
	Paste			^v		#
	Paste Selection	^V		# colle la selection courante X
	Clear			^d		#
	-----------
	Shift Left		^[		#
	Shift Right		^]		#
	Balance			F6		#
	Misc.    	>>			# recursif {AutoIdent dynacolor}
	-----------
	Select All		^a		#
	-----------
	Fonts & Size			# recursif pour l'instant
	============
}

# -----------------------------------------------------------------------------
# Cree le menu 'Edit' de l'editeur
# -----------------------------------------------------------------------------
proc ed:create_edit_menu {ed} {
	upvar #0 font_for_menus f
	set mb $ed.fm.mbedit
	set m $mb.medit
	menubutton $mb -menu $m -text [lang: TED.M-edit] \
		-underline [lang:u TED.M-edit] -font $f

	menu $m
	set mfonts [ed:menufont $m $ed]
	set mtext  [ed:menutext $m $ed]

	menu:add $m $ed.t edc undo -state disabled
	$m add sepa
	menu:add $m $ed.t edc cut
	menu:add $m $ed.t edc copy
	menu:add $m $ed.t edc paste
	menu:add $m $ed.t edc paste-selection
	menu:add $m $ed.t edc clear
	$m add sepa
	menu:add $m $ed.t edc shift-left
	menu:add $m $ed.t edc shift-right
	menu:add $m $ed.t edc balance
	$m add cascade -font $f -label [lang: misc-submenu] -menu $mtext
	$m add sepa
	menu:add $m $ed.t edc select-all
	$m add sepa
	$m add cascade -font $f -label [lang: font-submenu] -menu $mfonts

	return $mb
}

#
# Un petit menu, contenant pour l'instant autoindent et dynacolor
#
proc ed:menutext {root ed} {
 	upvar #0 font_for_menus f
   set m [menu $root.mtext]
    $m add checkbutton -font $f -label [lang: auto-indent] \
			-variable $ed\(autoindent\) -selectcolor green
    $m add checkbutton -font $f -label [lang: dynamic-coloring] \
			-variable $ed\(dynacolor\) -selectcolor green
	return $m
}

@com {
Le menu Find est ce qu'il y a de plus classique pour le mac, a quelques items pres.
Il n'y a pas les recherches/remplacements en arriere.
Find:
	============
	Find ...    		^f 			# appel le dialogue
	Find Next			^g			# cherche et selectionne le prochain
	Find <selection>    ^G			# cherche le contenu de la selection
	------------
	Replace				^r
	Replace & Find		^R
	Replace All
	------------
	Find All						# tous les colorier
	Next Found						# le prochain 'found'
	Unhighlight All Found			# vire tous les 'found'
	------------
	Selection -> Find-field			# c'est clair, non ?
	Selection -> Replace-field		# et ca aussi, non ?
	============
}

# -----------------------------------------------------------------------------
# Cree le menu 'Find' de l'editeur
# -----------------------------------------------------------------------------
proc ed:create_find_menu {ed} {
	upvar #0 font_for_menus f
	set mb $ed.fm.mbfind
	set m $mb.mfind
	menubutton $mb -menu $m -text [lang: TED.M-find] \
			-underline [lang:u TED.M-find] -font $f

	menu $m
	menu:add $m $ed.t ed dial-find
	menu:add $m $ed.t ed find-next
	menu:add $m $ed.t ed find-selection
	$m add sepa
	menu:add $m $ed.t ed replace
	menu:add $m $ed.t ed replace-then-find
	menu:add $m $ed.t ed replace-all
	$m add sepa
	menu:add $m $ed.t ed find-all
	menu:add $m $ed.t ed next-found
	menu:add $m $ed.t ed unhilite-all-found
	$m add sepa
	menu:add $m $ed.t ed selection-to-find-field
	menu:add $m $ed.t ed selection-to-replace-field

	return $mb
}

# -----------------------------------------------------------------------------
# Des procedure relais en rapport avec le menu Find.
# -----------------------------------------------------------------------------
proc ed:dial-find {t} { return [edfind:ShowDial [winfo parent $t]] }
proc ed:find-next {t} { return [edfind:FindNext [winfo parent $t]] }
proc ed:find-selection {t} { return [edfind:FindSelection [winfo parent $t]] }
proc ed:replace {t} { return [edfind:Replace [winfo parent $t]] }
proc ed:replace-then-find {t} { return [edfind:ReplaceAndFindNext [winfo parent $t]] }
proc ed:replace-all {t} { return [edfind:ReplaceAll [winfo parent $t]] }
proc ed:find-all {t} { return [edfind:FindAll [winfo parent $t]] }
proc ed:next-found {t} { return [edfind:ShowNextFound [winfo parent $t]] }
proc ed:unhilite-all-found {t} { return [edfind:UnhiliteAllFound [winfo parent $t]] }
proc ed:selection-to-find-field {t} { return [edfind:EnterFind [winfo parent $t]] }
proc ed:selection-to-replace-field {t} { return [edfind:EnterReplace [winfo parent $t]] }


#
# cree le menu des fontes
#
proc ed:menufont {root ed {setstyle 1}} {
 	upvar #0 font_for_menus f
	global fonts
	set m [menu $root.mfonts]
	foreach fn [lsort [array names fonts]] {
		if $setstyle {
			catch {$m add command -label $fn \
			-command [list ed:setfont $fn $ed.t $m] \
			-font [font: $fn]} u
			} \
		else {
			catch {$m add command -label $fn \
			-command [list ed:setfont $fn $ed.t $m] \
			-font $f} u
			}
		}
	return $m
}

#
# Pour les preferences
#
proc ed:makelangmenu {root} {
	set m [menu $root.mlang]
	foreach l {Text Prolog C Tcl} {
		$m add command -label $l \
				-command "set edit:tmp(lang) \"$l\""
		}
	return $m
}

proc ed:setfont {fn txt menu} {
	$txt config -font [font: $fn]
}

@com {
Le contenu de ce menu, dedie au mode Prolog, est temporaire.
Syntax:
	============
	Check Syntax				#
	Check Syntax (Selection)	#
	-----------
	Goto Next Error				#
	Clear All Error Marks		#
	Clear Warning Marks			#
	-----------
	Syntax Coloring				#
	============
}

# -----------------------------------------------------------------------------
# Cree le menu 'Syntax' de l'editeur
# -----------------------------------------------------------------------------
proc ed:create_syntax_menu {ed} {
	upvar #0 font_for_menus f
	set mb $ed.fm.mbsyn
	set m $mb.msyn
	menubutton $mb -menu $m -text [lang: TED.M-syntax] \
			-underline [lang:u TED.M-syntax] -font $f

	menu $m
	menu:add $m $ed.t ed check-syntax
	menu:add $m $ed.t ed check-syntax-of-selection
	$m add sepa
	menu:add $m $ed.t ed goto-next-error
	menu:add $m $ed.t ed detag-errors
	menu:add $m $ed.t ed detag-warnings
	$m add sepa
	menu:add $m $ed.t ed syntax-coloring

	return $mb
}

# -----------------------------------------------------------------------------
# Relais pour le menu Syntaxe de l'editeur
# -----------------------------------------------------------------------------
proc ed:check-syntax {t} {
	return [edit:check_syntax [winfo parent $t] all 2] }
proc ed:check-syntax-of-selection {t} {
	return [edit:check_syntax [winfo parent $t] sel 2] }
proc ed:goto-next-error {t} { return [edit:goto_next_error [winfo parent $t]] }
proc ed:detag-errors {t} { return [edit:clear_all_errors [winfo parent $t]] }
proc ed:detag-warnings {t} { return [edit:clear_warnings [winfo parent $t]] }
proc ed:syntax-coloring {t} { return [edit:syntax_coloring [winfo parent $t]] }



@com {
Le contenu de ce menu, egalement dedie au mode Prolog, est temporaire.
Prolog:
	============
	[_] Check-syntax before # C
	[_] Debug Mode			# D
	[_] Prolog IV Syntax	# P
	-----------
	Compile Window			# o
	Compile Selection		# m
	-----------
	Recompile Window		# W
	Recompile Selection		# S
	-----------
	Consulting	 >>			#
	============
}

# -----------------------------------------------------------------------------
# Cree le menu 'Prolog' de l'editeur
# -----------------------------------------------------------------------------
proc ed:create_prolog_menu {ed} {
	upvar #0 font_for_menus f
	set mb $ed.fm.mbpro
	set m $mb.mpro
	menubutton $mb -menu $m -text [lang: TED.M-prolog] \
			-underline [lang:u TED.M-prolog] -font $f

	menu $m
	set mcons   [ed:create_consult_menu $m $ed]
	set massert [ed:create_assertions_menu $m $ed]

	menu:addcheck $m $ed toggle-check-syntax-before-compile prolog:checksyntax
	menu:addcheck $m $ed toggle-debug-mode prolog:debugmode
	menu:addcheck $m $ed toggle-syntax-p4 prolog:syntaxp4 -offvalue 1 -onvalue 2
	$m add cascade -font $f -label [lang: assertions-submenu] -menu $massert
	$m add sepa
	menu:add $m $ed.t ed:prolog compile-window 
	menu:add $m $ed.t ed:prolog compile-selection 
	$m add sepa
	menu:add $m $ed.t ed:prolog recompile-window 
	menu:add $m $ed.t ed:prolog recompile-selection 
	$m add sepa
	$m add cascade -font $f -label [lang: consulting-submenu] -menu $mcons

	return $mb
}

# -----------------------------------------------------------------------------
# Relais pour le menu Prolog de l'editeur
# -----------------------------------------------------------------------------
proc ed:prolog:compile-window {t} {
	return [edit:prolog:compilewindow [winfo parent $t]] }
proc ed:prolog:compile-selection {t} {
	return [edit:prolog:compileselection [winfo parent $t]] }
proc ed:prolog:recompile-window {t} {
	return [edit:prolog:recompilewindow [winfo parent $t]] }
proc ed:prolog:recompile-selection {t} {
	return [edit:prolog:recompileselection [winfo parent $t]] }

#
# Le sous-menu consulting
#
proc ed:create_consult_menu {root ed} {
	upvar #0 font_for_menus f
	set m [menu $root.mcons]
	menu:add $m $ed edit:prolog consult-window
	menu:add $m $ed edit:prolog consult-selection
	$m add sepa
	menu:add $m $ed edit:prolog reconsult-window
	menu:add $m $ed edit:prolog reconsult-selection

	return $m
}

# -----------------------------------------------------------------------------
# Relais pour le menu Prolog de l'editeur
# -----------------------------------------------------------------------------
proc ed:prolog:consult-window {t} {
	return [edit:prolog:consult-window [winfo parent $t]] }
proc ed:prolog:consult-selection {t} {
	return [edit:prolog:consult-selection [winfo parent $t]] }
proc ed:prolog:reconsult-window {t} {
	return [edit:prolog:reconsult-window [winfo parent $t]] }
proc ed:prolog:reconsult-selection {t} {
	return [edit:prolog:reconsult-selection [winfo parent $t]] }


#
# Le sous-menu assertions
#
proc ed:create_assertions_menu {root ed} {
	upvar #0 font_for_menus f
	set m [menu $root.massert]
	menu:addradio $m $ed assertions-none prolog:assertion_mode -value 0
	menu:addradio $m $ed assertions-static prolog:assertion_mode -value 1
	menu:addradio $m $ed assertions-static-dynamic prolog:assertion_mode -value 2
	return $m
}




@com {
?:
	============
	============
	
Options:
	============
	============

-----------------------------------------------------------
}

proc ed:color_errors {t} {
	if {[winfo depth .] == 1} {
		#
		# Noir et Blanc
		#
		$t tag config warning -backg black -foreg white -border 1 -relief raised
		$t tag config erreur  -backg black -foreg white -border 1 -relief raised
		} \
	else {
		$t tag config warning -backg darkorange -foreg white -border 1 -relief raised
		$t tag config erreur  -backg red -foreg white -border 1 -relief raised
		}
}

# -----------------------------------------------------------------------------
# Cree un Editeur
# W est de la forme .ed<n> (comme .ed7)
# on y construit un frame .ed<n>.frEdit, puis, dessous, les widgets:
#	.fm (menubar)
#	.fs	(status ligne)
#	.sv	(scrollbar vertical)
# -----------------------------------------------------------------------------
proc ed:new {W} {
	toplevel $W
	wm minsize $W [winfo pix $W 5c] [winfo pix $W 4c]

	set ed [ed:newfrEdit $W]
	menu $ed.mbulle -bd 1 -bg yellow -tearoff 0
	pack $ed -side top -fill both -expand yes

	wm protocol $W WM_DELETE_WINDOW "edit:Close $ed"
	return $ed
}

# -----------------------------------------------------------------------------
# C'est ce qui construit le frame Editeur
# -----------------------------------------------------------------------------
proc ed:newfrEdit {root {menu 1} {status 1} {scrollbar 1}} {
	set ed $root.frEdit
	upvar #0 edit:pref prefs
	global fonts
	global $ed

	#
	# Il n'est pas encore nomme
	#
	set ${ed}(named) 0

	frame $ed

	if $menu {
		#
		# Menubar
		#
		frame $ed.fm -relief raised -bd 1
		pack $ed.fm -side top -fill x
	
# Fait plus loin :
#		ed:create_menus $ed
		}

	if $status {
		#
		# La ligne de status de l'editeur
		#
		ed:create_status $ed
		}

	#
	# Une ligne de status en bas de l'editeur
	#
	set ${ed}(message) ""
	entry $ed.en -relief sunken -bd 1 -textvariable ${ed}(message)
	pack $ed.en -side bottom -fill x

	if $scrollbar {
		#
		# scrollbar vertical de l'editeur
		#
		scrollbar $ed.sv -command "$ed.t yview" -relief sunken
#				-bg #e300f300f500 -activeback #00002e2e0000
		pack $ed.sv -side right -fill y
		}
	#
	# L'objet Texte de l'editeur
	#
	text $ed.t -relief sunken -bd 2 \
			-wid 80 -hei 24			\
			-fg $prefs(fg)	\
			-bg $prefs(bg)	\
			-font [font: $prefs(font)]

	pack $ed.t -side left -fill both -expand yes

	#
	# liaison scrollbar, texte, et numlignes
	#
	$ed.t config -yscrollcom "ed:txt_scrl_lines $ed.sv $ed.t"

	#
	# Divers, touches et tags
	#
#TMP
#3	ed:bind $ed.t

	global misc:pref
	if [set misc:pref(emacslike)] {
		bindtags $ed.t [list $ed.t ted_common ted_emacs moved $ed all]
		} \
	else {
		bindtags $ed.t [list $ed.t ted_common ted_pc moved $ed all]
		}
	ed:init_tag $ed

	if $menu {
		ed:create_menus $ed
		}

	#
	# juste pour souligner (debug)
	# TMP:
	$ed.t tag config UNDER -underline 1

	# Pour les recherches (find all)
	$ed.t tag config found  -backg blue -foreg white -border 1 -relief raised

	# Pour les erreurs
	ed:color_errors $ed.t

	$ed.t tag bind erreur <Enter> "ed:synterr_bulle $ed %X %Y"
	$ed.t tag bind erreur <Leave> "$ed.mbulle unpost"
	$ed.t tag bind warning <Enter> "ed:synterr_bulle $ed %X %Y"
	$ed.t tag bind warning <Leave> "$ed.mbulle unpost"

	$ed.t tag raise sel

	return $ed
}

#
# Appele pour montrer le libelle d'une erreur de syntaxe
#
proc ed:synterr_bulle {ed X Y} {
	set L [lmatch-glob [$ed.t tag names current] {syntaxerror-*}]
	$ed.mbulle delete 0 last
	foreach i $L {
		$ed.mbulle add command -label [string range $i [string first - $i] end]
		}
	$ed.mbulle post [expr $X +5] [expr $Y +20]
}

# -----------------------------------------------------------------------------
# Ligne de statut de l'editeur (divers gadgets)
# Edited T Prolog Lines:[120-142] Pos: 127.3			travail
# -----------------------------------------------------------------------------
proc ed:create_status {ed} {
	global edit:pref

	set fs $ed.fs
	#
	# Ligne de statut de l'editeur (pas tout fait)
	#
	frame $fs -relief raised -bd 1 -hei 20
	pack $fs -side top -fill x

	global $ed
	set $ed\(dirty\) ""
	# TMP:
	set $ed\(autoindent\) 1
	set $ed\(dynacolor\) 1
	set $ed\(dirtyname\) ""
	trace variable $ed\(dirty\) w "ed:setdirtyname $ed"
	
	#
	# Le statut dirty (Edited, ReadOnly ou pas)
	# on peut l'enlever avec Ctrl-Shift-B1
	#
	label $fs.ld -textvariable $ed\(dirtyname\) -width 8
	pack $fs.ld -side left -anc c
	bind $fs.ld <Control-Shift-Button-1> "ed:rolldirty $ed.t"
	bind $fs.ld <Button-1> "focus $ed.t"
	
	#
	# Un menu, contenant le mode (langage)
	#
	set v ${ed}(lang)
	set $v [set edit:pref(lang)]
	menubutton $fs.mbm -menu $fs.mbm.m -relief raised -bd 1 -textvariable $v \
			-indicatoron 1 -padx 1 -pady 1
	menu $fs.mbm.m -tearoff 0
	$fs.mbm.m add radiobutton -label Text		-variable $v -indicator 0
	$fs.mbm.m add radiobutton -label Prolog		-variable $v -indicator 0
	$fs.mbm.m add radiobutton -label C  		-variable $v -indicator 0
	$fs.mbm.m add radiobutton -label Tcl  		-variable $v -indicator 0
	pack $fs.mbm -side left

	#
	# Positions dans le texte
	#
	label $fs.ll -text [lang: "TED.Lines:"]
	pack $fs.ll -side left
	
	label $fs.lines -text "\[120-142\]" -width 11
	pack $fs.lines -side left

	label $fs.lp -text [lang: "TED.Pos:"]
	pack $fs.lp -side left
	
	label $fs.pos -text "1.0" -width 8
	pack $fs.pos -side left

	label $fs.lwork -text "" -relief sunken -bd 1 -anchor e \
			-font [font: {helvetica 12 italic}]
#			-font -*-Helvetica-Medium-O-Normal--*-120-*
	pack $fs.lwork -side right -expand yes -fill x
}

#
#
#
proc ed:setdirtyname {ed n1 n2 op} {
	global $ed
	set $ed\(dirtyname\) [lang: [set $ed\(dirty\)]]
}

# -----------------------------------------------------------------------------
# L'association text/scrollbar/numlignes
# ws est le scrollbar
# wt est le texte
# TMP:
# -----------------------------------------------------------------------------
proc ed:txt_scrl_lines {ws wt a b} {
	$ws set $a $b
	
#	scan [$wt index end] "%f" pos
#	set b [min $b [expr int($pos)]]

	set bot [winfo height $wt]
	set v \[[$wt index @0,0]-[$wt index @0,$bot]\]
# TMP:
	[winfo parent $wt].fs.lines config -text $v -wid [string len $v]
}


# -----------------------------------------------------------------------------
# On detruit la fenetre complete et les variables associees a l'editeur
# -----------------------------------------------------------------------------
proc ed:close_window {ed} {
	global $ed
#3	catch "exec /bin/rm [fs:tmp]/b$ed-[pid]"
	fs:delete-f [fs:tmpfile b$ed-[pid]]
	destroy [winfo toplevel $ed]
	unset $ed
}

# -----------------------------------------------------------------------------
# Charge un Fichier dans l'editeur au pt d'insertion
# retourne 1 si ok, 0 sinon
# -----------------------------------------------------------------------------
proc ed:load_file {ed file} {
	if {[catch {open $file r} f]} {
		return 0
		}
	$ed.t insert insert [read $f]
	close $f
	return 1
}

# -----------------------------------------------------------------------------
# Sauve un editeur dans un fichier donne
# retourne 1 si ok, 0 sinon
# -----------------------------------------------------------------------------
proc ed:save_file {ed file} {
	if {[catch {open $file w} f]} {
		return 0
		}
	puts -nonewline $f [$ed.t get 1.0 end-1c]
	close $f
	return 1
}

# -----------------------------------------------------------------------------
# Vide un editeur
# -----------------------------------------------------------------------------
proc ed:empty_window {ed} {
	$ed.t delete 1.0 end
}

# -----------------------------------------------------------------------------
# Statut d'edition d'une fenetre ("" xou Edited xou ReadOnly)
# -----------------------------------------------------------------------------
proc ed:setdirty {t {val "Edited"}} {
	set ed [winfo parent $t]
	global $ed
	if {"[set $ed\(dirty\)]" != "ReadOnly"} {
		set $ed\(dirty\) $val
		}
}

# -----------------------------------------------------------------------------
# isdirty ou pas ? (1 ou 0)
# -----------------------------------------------------------------------------
proc ed:isdirty {t} {
	set ed [winfo parent $t]
	global $ed
	return [expr ! [string compare [set $ed\(dirty\)] "Edited"]]
}
#
proc edit:isdirty {ed} {
	global $ed
	return [expr ! [string compare [set $ed\(dirty\)] "Edited"]]
}


# -----------------------------------------------------------------------------
# ReadOnly ou pas ? (1 ou 0)
# -----------------------------------------------------------------------------
proc ed:isreadonly {t} {
	set ed [winfo parent $t]
	global $ed
	set v [expr ![string compare [set $ed\(dirty\)] "ReadOnly"]]
	if $v bip
	return $v
}


# -----------------------------------------------------------------------------
# Permutation circulaire entre "", Edited et ReadOnly
# -----------------------------------------------------------------------------
proc ed:rolldirty {t} {
	set ed [winfo parent $t]
	global $ed
	switch [set $ed\(dirty\)] {
		""
			{set v ReadOnly ; $t config -state disabled}
		"Edited"
			{set v "" ; $t config -state normal}
		"ReadOnly"
			{set v "Edited" ; $t config -state normal}
		}
	set $ed\(dirty\) $v
}


#
# Delai en ms entre deux backups des editeurs de textes
# (5 minutes)
global ed:bkdelayms
set ed:bkdelayms [expr 5 * 60 * 1000]

# -----------------------------------------------------------------------------
# AUTO-BACKUP toutes les x millisecondes
# after $ed:bkdelayms "ed:backuptexts"
# -----------------------------------------------------------------------------
proc ed:backuptexts {} {
	global ed:bkdelayms
	global edit_eds
	foreach ed [array names edit_eds] {
		upvar #0 $ed tab
		if {[edit:isdirty $ed]} {
			set file "[fs:tmpfile b$ed-[pid]]"
			ed:save_file $ed $file
			}
		}
	after ${ed:bkdelayms} ed:backuptexts
}

#-- EOF edit.tcl --------------------------------------------------------------
