# -- edprolog.tcl
# La gestion des requetes Prolog sous l'Editeur
#
# Auteur:		Pascal Bouvier
# Creation:		12/06/95
# Modifications:	
#				PBO 06/02/96 variable pour parse prolog:syntaxp4
#				PBO 11/04/97 corr. consultwindow -> consult-window, etc...
#				PBO 07/09/98 ajout var. prolog:assertion_mode
#
# (c) PrologIA 1995,1998
# -----------------------------------------------------------------------------


# =============================================================================
# Globales
# =============================================================================

# Le num courant de requete:
set prolog:reqid 0

# Le tableau de requetes
global prolog:lreq

# Le booleen check-syntax (pour avant de compiler...)
global prolog:checksyntax
set prolog:checksyntax 1

# Le booleen debug-mode (pour compiler...)
global prolog:debugmode

# Le booleen syntax-p4 (pour iso+(1) prolog4(2))
global prolog:syntaxp4
set prolog:syntaxp4 2

# Le trileen assertion_mode (0:sans ; 1:statique ; 2:statique & dynamique)
global prolog:assertion_mode
set prolog:assertion_mode 0

# -----------------------------------------------------------------------------
# Identification unique des requetes au sein d'une session Environnement.
# -----------------------------------------------------------------------------
proc prolog:newid {} {
	global prolog:reqid
	return [incr prolog:reqid]
}

# -----------------------------------------------------------------------------
# fabrique la chaine de reponse de prolog a tcl
# -----------------------------------------------------------------------------
proc prolog:buildEndRequest {id} {
	global forprolog4
	if $forprolog4 {
		return "sysc__wsend(\['prolog:endrequest ', $id\]).\n"
		} \
	else {
		# PAS A JOUR !
		#global quotedfor_tcl
		#return "outm(\"$quotedfor_tcl prolog:endrequest $id\\n\").\n"
		}
}

# -----------------------------------------------------------------------------
# On construit la structure logique d'une requete et on l'enregistre
# -----------------------------------------------------------------------------
proc prolog:startrequest {id titre action ed} {
	upvar #0 prolog:lreq lreq
	set lreq($id) [list $titre $action $ed]
}

# -----------------------------------------------------------------------------
# Est-ce qu'un but prolog peut etre lance d'ici.
# -----------------------------------------------------------------------------
proc prolog:isavailable {} {
	global f1

	set replyid [conn:pro_fun "isdbgwaiting" "" $f1]
	if {$replyid == 0} {
		errputs "too busy for request"
		return 0
		}
	set answer [conn:getanswer $replyid]
	if {$answer == 1} {
		# il faut deconnecter le debugger
		set rep [dial:oui_non "A debugging session is currently active, \
				you have first to stop it before launching a goal:" \
			"Menu actions warning" \
			"Stop current debugging session" \
			"Cancel action and continue debugging"]
		if {$rep == 1} {
			conn:envoie "no_debug\nkill\n" $f1
			} \
		else {
			return 0
			}
		}

	set replyid [conn:pro_fun "isdbg" "" $f1]
	if {$replyid == 0} {
		errputs "too busy for request"
		return 0
		}
	set answer [conn:getanswer $replyid]
	if {$answer == 1} {
		# il faut deconnecter le debugger
		errputs "il faut deconnecter temporairement le debugger"
		if {[conn:pro_call "setdbg" "0" $f1] == 0} {
			errputs "too busy for request"
			return 0
			}
		}

	return 1
}


# -----------------------------------------------------------------------------
# On est notifie qu'une requete prolog vient de se terminer
# -----------------------------------------------------------------------------
proc prolog:endrequest {id} {
	upvar #0 prolog:lreq lreq
	set r $lreq($id)
	set titre [lindex $r 0]
	set action [lindex $r 1]
	set ed [lindex $r 2]
	eval $action
	unset lreq($id)
	errputs "<<< La requete '$titre' ($id) est terminee."
	ed:setworkstring $ed
}

proc edit:rm {file} {
	#errputs "--- Le fichier '$file' est efface."
	#fs:delete-f $file"
}

# -----------------------------------------------------------------------------
# Cree, enregistre et envoie la requete a prolog
# -----------------------------------------------------------------------------
proc prolog:sendrequest {id titre but completion ed} {
	global f1
	if {! [prolog:isavailable]} {bip; return}

	prolog:startrequest $id $titre $completion $ed
	conn:envoie "$but.\n [prolog:buildEndRequest $id]" $f1
	errputs ">>> La requete '$titre' ($id) est envoyee."
	ed:setworkstring $ed "$titre..."
}

#
#
#
proc edit:prolog:set_assertion_mode {} {
	global prolog:assertion_mode
	return "record(discipl__debug_level, ${prolog:assertion_mode})"
}


# -----------------------------------------------------------------------------
# Des buts prolog lances par menus (Prolog IV)
# -----------------------------------------------------------------------------

#==============================================================================
# C O M P I L E
#==============================================================================
proc edit:prolog:compilewindow {ed} {
	global prolog:checksyntax prolog:debugmode BINP4 prolog:tmpficed
	if ${prolog:checksyntax} {
		if [edit:check_syntax $ed all 1] return
		}
	set id [prolog:newid]
	set tmpfic [fs:tmpfile f$id-[pid]]
	if ${prolog:debugmode} {
		ed:prolog:preprocesse $tmpfic $ed 1.0 end
		set fic $tmpfic.p4g
		} \
	else {
#3		exec cat > $tmpfic.p4 << [$ed.t get 1.0 end]
		fs:strtofile [$ed.t get 1.0 end] $tmpfic.p4
		set fic $tmpfic.p4
		}
	set prolog:tmpficed [list $tmpfic $ed]

	prolog:sendrequest $id \
		"Compiling Window" \
		"[edit:prolog:set_assertion_mode], compile(\"$fic\")" \
		"edit:rm \"$fic\"" \
		$ed
}
#
proc edit:prolog:compileselection {ed} {
	global prolog:checksyntax prolog:debugmode BINP4 prolog:tmpficed
	if { ! [catch "$ed.t index sel.first"] } {
		if ${prolog:checksyntax} {
			if [edit:check_syntax $ed sel 1] return
			}
		set id [prolog:newid]
		set tmpfic [fs:tmpfile f$id-[pid]]
		if ${prolog:debugmode} {
			ed:prolog:preprocesse $tmpfic $ed sel.first sel.last
			set fic $tmpfic.p4g
			} \
		else {
#3			exec cat > $tmpfic.p4 << [$ed.t get sel.first sel.last]
			fs:strtofile [$ed.t get sel.first sel.last] $tmpfic.p4
			set fic $tmpfic.p4
			}
		set prolog:tmpficed [list $tmpfic $ed]

		prolog:sendrequest $id \
			"Compiling Selection" \
			"[edit:prolog:set_assertion_mode], compile(\"$fic\")" \
			"edit:rm \"$fic\"" \
			$ed
		}
	}
#
proc edit:prolog:recompilewindow {ed} {
	global prolog:checksyntax prolog:debugmode BINP4 prolog:tmpficed
	if ${prolog:checksyntax} {
		if [edit:check_syntax $ed all 1] return
		}
	set id [prolog:newid]
	set tmpfic [fs:tmpfile f$id-[pid]]
	if ${prolog:debugmode} {
		ed:prolog:preprocesse $tmpfic $ed 1.0 end
		set fic $tmpfic.p4g
		} \
	else {
#3		exec cat > $tmpfic.p4 << [$ed.t get 1.0 end]
		fs:strtofile [$ed.t get 1.0 end] $tmpfic.p4
		set fic $tmpfic.p4
		}
	set prolog:tmpficed [list $tmpfic $ed]

	prolog:sendrequest $id \
		"Recompiling Window" \
		"[edit:prolog:set_assertion_mode], recompile(\"$fic\")" \
		"edit:rm \"$fic\"" \
		$ed
}
#
proc edit:prolog:recompileselection {ed} {
	global prolog:checksyntax prolog:debugmode BINP4 prolog:tmpficed
	if { ! [catch "$ed.t index sel.first"] } {
		if ${prolog:checksyntax} {
			if [edit:check_syntax $ed sel 1] return
			}
		set id [prolog:newid]
		set tmpfic [fs:tmpfile f$id-[pid]]
		if ${prolog:debugmode} {
			ed:prolog:preprocesse $tmpfic $ed sel.first sel.last
			set fic $tmpfic.p4g
			} \
		else {
#3			exec cat > $tmpfic.p4 << [$ed.t get sel.first sel.last]
			fs:strtofile [$ed.t get sel.first sel.last] $tmpfic.p4
			set fic $tmpfic.p4
			}
		set prolog:tmpficed [list $tmpfic $ed]


		prolog:sendrequest $id \
			"Recompiling Selection" \
			"[edit:prolog:set_assertion_mode], recompile(\"$fic\")" \
			"edit:rm \"$fic\"" \
			$ed
	}
}

# -----------------------------------------------------------------------------
# L'appel standard au preprocesseur
# -----------------------------------------------------------------------------
proc ed:prolog:preprocesse {tmpfic ed deb fin} {
	global BINP4 prolog:syntaxp4
#3	exec echo ":-unit('$tmpfic.p4').\n:-debug." > $tmpfic.p4
	fs:strtofile ":-unit('$tmpfic.p4').\n:-debug.\n" $tmpfic.p4 
#3	exec cat >> $tmpfic.p4 << [$ed.t get $deb $fin]
	fs:strtofile-append [$ed.t get $deb $fin] $tmpfic.p4

	catch {exec $BINP4/newparse -dl -df -H 50000 -r ${prolog:syntaxp4} \
		-g0 -gx -do -claudops $tmpfic.p4 -s $tmpfic.sym -o $tmpfic.p4g}
}

# VIEUX
#proc ed:prolog:preprocesse {tmpfic ed deb fin} {
#	global BINP4
#	exec echo ":-debug." > $tmpfic.p4g
#	exec echo "'$tmpfic.p4'." >> $tmpfic.p4g
#	exec cat > $tmpfic.p4 << [$ed.t get $deb $fin]
#	catch "exec $BINP4/newparse -H 50000 -r 2 -g0 $tmpfic.p4 -s $tmpfic.sym >> $tmpfic.p4g"
#	}

#==============================================================================
# C O N S U L T
#==============================================================================
proc edit:prolog:consult-window {ed} {
	global prolog:checksyntax
	if ${prolog:checksyntax} {
		if [edit:check_syntax $ed all 1] return
		}
	set id [prolog:newid]
	set fic [fs:tmpfile f$id.[pid].p4]
#3	exec cat > $fic << [$ed.t get 1.0 end]
	fs:strtofile [$ed.t get 1.0 end] $fic
	
	prolog:sendrequest $id \
			"Consulting Window" \
			"consult(\"$fic\")" \
			"edit:rm \"$fic\"" \
			$ed
}
#
proc edit:prolog:consult-selection {ed} {
	global prolog:checksyntax
	if { ! [catch "$ed.t index sel.first"] } {
		if ${prolog:checksyntax} {
			if [edit:check_syntax $ed sel 1] return
			}
		set id [prolog:newid]
		set fic [fs:tmpfile f$id.[pid].p4]
#3		exec cat > $fic << [$ed.t get sel.first sel.last]
		fs:strtofile [$ed.t get sel.first sel.last] $fic

		prolog:sendrequest $id \
				"Consulting Selection" \
				"consult(\"$fic\")" \
				"edit:rm \"$fic\"" \
				$ed
		}
	}
#
proc edit:prolog:reconsult-window {ed} {
	global prolog:checksyntax
	if ${prolog:checksyntax} {
		if [edit:check_syntax $ed all 1] return
		}
	set id [prolog:newid]
	set fic [fs:tmpfile f$id.[pid].p4]
#3	exec cat > $fic << [$ed.t get 1.0 end]
	fs:strtofile [$ed.t get 1.0 end] $fic
	
	prolog:sendrequest $id \
			"Reconsulting Window" \
			"reconsult(\"$fic\")" \
			"edit:rm \"$fic\"" \
			$ed
}
#
proc edit:prolog:reconsult-selection {ed} {
	global prolog:checksyntax
	if { ! [catch "$ed.t index sel.first"] } {
		if ${prolog:checksyntax} {
			if [edit:check_syntax $ed sel 1] return
			}
		set id [prolog:newid]
		set fic [fs:tmpfile f$id.[pid].p4]
#3		exec cat > $fic << [$ed.t get sel.first sel.last]
		fs:strtofile [$ed.t get sel.first sel.last] $fic

		prolog:sendrequest $id \
				"Reconsulting Selection" \
				"reconsult(\"$fic\")" \
				"edit:rm \"$fic\"" \
				$ed
		}
}

# INUTILE
#if 0 {
#global forprolog4
#
#if $forprolog4 {
#	proc edit:prolog:compileandload {ed} {
#		global edit_eds
#		global BINP4
#		if [edit:check_syntax $ed all 1] return
#		set src $edit_eds($ed)
#		set so [file root $src]
#		exec $BINP4/bst.exe << "cpx('$src'). halt. "
#		exec make $so
#		sendprolog "load_module('$so'). \n"
#		}
#	} \
#else {
#	proc edit:prolog:compileandload {ed} {
#		errputs "compile-and-load: non-implantee en Prolog III"
#		}
#	}
#}

#-- EOF edprolog.tcl ----------------
