#
# edtag.tcl
#

@com {
A faire:
	{@items
{@it	+ Detruire les variables globales quand on detruit une fenetre config}
{@it	+ Faire {@b configurer} (config:configurer)}
{@it	- Refaire {@b misajour}}
	}
}

#-------------------------------------------------------------------------
proc et:newid {} {
	upvar #0 et:confid i
	return [incr i]
}

#-------------------------------------------------------------------------
#
#
proc et:init {} {
	toplevel .et -class browse
	wm title .et "Browse Widget"
	wm protocol .et WM_DELETE_WINDOW "win:destroy .et"
	wm minsize .et 1 1

	label .et.lb -relief groove -text .
	listbox .et.ls -relief sunken
#	tk_listboxSingleSelect .et.ls
	bind .et.ls <Double-Button-1> {aller_fils %y}

	button .et.bmis  -text "Mise a jour" -comm et:misajour
	button .et.bconf -text "Configurer..." -comm "et:action config:new"
	button .et.bshow -text "Montrer" -comm "et:action et:showwidget"
	button .et.bkill -text "Detruire" -comm "et:action win:destroy"

	pack .et.lb -fill x
	pack .et.ls -side top -expand yes -fill both
	pack .et.bmis .et.bconf .et.bshow .et.bkill -side top -fill x

	global et:confid
	set et:confid 0
}

#-------------------------------------------------------------------------
# Une form est un doublet label-entry (champ editable a etiquette)
#-------------------------------------------------------------------------
proc et:form:new {w n optionname} {
	frame $w.f$n
	label $w.l$n -text $optionname -anc e
	entry $w.e$n -wid 20 -textv ${w}($optionname) -relief sunken
	pack $w.l$n -in $w.f$n -side left -padx 1m -fill x -expand 1
	pack $w.e$n -in $w.f$n -side left -padx 1m
	#
	# Pour faire configurer quand on fait return
	#
#	bind $w.e$n <Return> "$w.fb.bc invoke"
#	bind $w.e$n <Escape> "$w.fb.bd invoke"
	bind $w <Return> "$w.fb.bc invoke"
	bind $w <Escape> "$w.fb.bd invoke"
	return $w.f$n
}

#-------------------------------------------------------------------------
#
# Une fenetre de configuration d'un widget donne
#
proc config:new {w} {
	set conf [toplevel .conf[et:newid] -class config]
	wm title $conf "Configure \"$w\""
	wm protocol $conf WM_DELETE_WINDOW "win:destroy $conf"

	global $conf

	label $conf.l0 -text "Class: [winfo class $w]"
	pack $conf.l0 -side top
	set i 1
	set ${conf}(nom) $w
	foreach c [$w config] {
		if {[llength $c]==5} {
			set opt [lindex $c 0]
			set form [et:form:new $conf $i $opt]
			set ${conf}($opt) [lindex $c 4]
			pack $form -side top -fill x
			incr i
			}
		}
	frame $conf.fb
	pack $conf.fb -side top -fill x -padx 1m -pady 2m
	button $conf.fb.bc -text Config -comm "config:configurer $w $conf"
	button $conf.fb.br -text Revert -comm "config:restaurer $w $conf"
	button $conf.fb.bd -text Dismiss -comm "win:destroy $conf"
	pack $conf.fb.bc $conf.fb.br $conf.fb.bd -side left -fill x -expand yes
}

#-------------------------------------------------------------------------
#
# Une fenetre de configuration d'un Tag donne (pseudo-widget)
#
proc config:new-pseudo {w} {
	set conf [toplevel .conf[et:newid] -class config]
	wm title $conf "Configure \"$w\""
	wm protocol $conf WM_DELETE_WINDOW "win:destroy $conf"

	global $conf

	set parent [et:winfo-parent $w]
	set name [et:winfo-name $w]
	label $conf.l0 -text "Class: pseudo"
	pack $conf.l0 -side top
	set i 1
	set ${conf}(nom) $w
	foreach c [$parent tag config $name] {
		if {[llength $c]==5} {
			set opt [lindex $c 0]
			set form [et:form:new $conf $i $opt]
			set ${conf}($opt) [lindex $c 4]
			pack $form -side top -fill x
			incr i
			}
		}
	frame $conf.fb
	pack $conf.fb -side top -fill x -padx 1m -pady 2m
	button $conf.fb.bc -text Config -comm "config:configurer-pseudo $parent $name $conf"
	button $conf.fb.br -text Revert -comm "config:restaurer-pseudo $parent $name $conf"
	button $conf.fb.bd -text Dismiss -comm "win:destroy $conf"
	pack $conf.fb.bc $conf.fb.br $conf.fb.bd -side left -fill x -expand yes
}

#-------------------------------------------------------------------------
#
# C'est ce qui est appele apres avoir detruit une fenetre 'config'
# Il faut liberer toutes les globales associees aux entries
# On detruit donc le tableau du genre '.conf34'
#
proc config:did_destroy {w} {
if 1 {
	upvar #0 $w tab
	unset tab
	} \
elseif 0 {
	global $w
	unset $w
	} \
elseif 0 {
	uplevel #0 "unset $w"
	}
}

#-------------------------------------------------------------------------
#
# Configure le widget w a partir des donnees dans le tableau
#
proc config:configurer {w tableau} {
	upvar #0 $tableau tab
	set L {}
	foreach opt [array names tab] {
		if {[string index $opt 0] == "-" \
			&& [string compare $opt "-class"] != 0 \
			&& [string compare $opt "-container"] != 0 } {
			lappend L $opt $tab($opt)
			}
		}
	if {[info comm $w] != ""} {
		eval $w config $L
		} \
	else {
		win:destroy $tableau
		}
}

#-------------------------------------------------------------------------
#
# Configure le widget parent/name partir des donnees dans le tableau
#
proc config:configurer-pseudo {parent name tableau} {
	upvar #0 $tableau tab
	set L {}
	foreach opt [array names tab] {
		if {[string index $opt 0] == "-"} {
			lappend L $opt $tab($opt)
			}
		}
	if {[info comm $parent] != ""} {
		eval $parent tag config $name $L
		} \
	else {
		win:destroy $tableau
		}
}

#-------------------------------------------------------------------------
#
# Restaure le tableau a partir du widget w
#
proc config:restaurer {w tableau} {
	nyi
}
proc config:restaurer-pseudo {parent name tableau} {
	nyi
}

#-------------------------------------------------------------------------
#
#
#
proc et:showwidget {w} {
	set t [winfo toplevel $w]
	if {[wm state $t] == "iconic"} {
		wm deiconify $t
		}
	raise $t
}

#-------------------------------------------------------------------------
#
# les tags pour les Texts pour l'instant
#
proc et:winfo-parent {w} {
	if {[winfo exists $w]} {
		return [winfo parent $w]
		} \
	else {
		set L [split $w .]
		set pc [lindex $L end]
		if {[string index $pc 0] == "&"} {	# c'est un tag
			set path [lrange $L 0 [expr [llength $L]-2]]
			set path [join $path .]
			return $path
			}
		}
	return .
}

#-------------------------------------------------------------------------
#
#
#
proc et:winfo-name {w} {
	if {[winfo exists $w]} {
		return [winfo name $w]
		} \
	else {
		set l [split $w .]
		set pc [lindex $l end]
		if {[string index $pc 0] == "&"} {	# c'est un tag
			return [string range $pc 1 end]
			}
		}
	return ""
}

proc et:winfo-child {w} {
	if {[winfo exists $w]} {
		return [winfo child $w]
		} \
	else {
		return {}
		}
}

#-------------------------------------------------------------------------
#
# les tags pour les Texts pour l'instant
#
proc et:pseudo-child {w} {
	if {[winfo exists $w]} {
		if {[winfo class $w] == "Text"} {
			return [$w tag names]
			} \
		} \
	else {
		set pc [lindex [split $w .] end]
		if {[string index $pc 0] == "&"} {	# c'est un tag
			return {}
			}
		# sinon c'est une erreur
		}
	return {}
}

#-------------------------------------------------------------------------
#
# 
#
proc et:is-pseudo {w} {
	set pc [lindex [split $w .] end]
	if {[string index $pc 0] == "&"} {	# c'est un tag
		return 1
		} \
	else {
		return 0
		}
}

#-------------------------------------------------------------------------
#
# Insere les fils dans la listbox (avec un <up> eventuel)
#
proc et:misajour {} {
	.et.ls delete 0 end
	set root [.et.lb cget -text]
	if { $root != "" } {
		.et.ls insert end "<up>"
		set Pfils [et:pseudo-child $root]
		set Lfils [et:winfo-child $root]
		} \
	else {
		set Pfils {}
		set Lfils "."
		}
	foreach elt $Pfils {
		.et.ls insert end $root.&$elt
		}
	foreach elt $Lfils {
		.et.ls insert end $elt
		}
}


#-------------------------------------------------------------------------
#
# Down
#
proc aller_fils {y} {
	set elt [.et.ls get [.et.ls nearest $y]]
	if {$elt == "<up>"} {
		set elt [et:winfo-parent [.et.lb cget -text]]
		if {$elt == ""} {set elt .}
		}
	.et.lb config -text $elt
	et:misajour
}

#-------------------------------------------------------------------------
#
#
#
proc et:action {command} {
	if {[set ind [.et.ls curselection]] == ""} {
		set elt [.et.lb cget -text]
		} \
	else {
		set elt [.et.ls get $ind]
		}

	if {$elt == "<up>"} {
		set elt [.et.lb cget -text]
		if [winfo exists $elt] {
			set elt [et:winfo-parent $elt]
			if {$elt == ""} {set elt .}
			$command $elt
			et:misajour
			return
			}
		} \
	elseif [winfo exists $elt] {
		$command $elt
		et:misajour
		return
		} \
	elseif [et:is-pseudo $elt] {
		${command}-pseudo $elt
		et:misajour
		return
		}

# n'existe pas: on remonte jusqu'a "."

	.et.lb config -text .
	et:misajour
}

#-------------------------------------------------------------------------
#-------------------------------------------------------------------------
#
# La classe win gere les fenetres (particulierement leur creation/destruction)
# il faut utiliser win:destroy au lieu de destroy
#
# de plus, a la creation, il faut faire:
# wm protocol .mywin WM_DELETE_WINDOW "win:destroy .mywin"
#
# Par convention, les classes de prolog4 (non-tk) sont en minuscules
# Son handler est alors appele.
#
proc win:destroy {args} {
	foreach w $args {
#
# notification A definir !!
#		win:notify_destroy $w

		set class [winfo class $w]
		set userclass [string match {[a-z]} [string index $class 0]]
		if {$userclass  &&  "" != [info proc $class:will_destroy]} {
			$class:will_destroy $w
			}

		destroy $w

		if {$userclass  &&  "" != [info proc $class:did_destroy]} {
			$class:did_destroy $w
			}
		}
}


#-------------------------------
# EOF edtag.tcl
#-------------------------------
