# -- find.tcl
# Le recherches/remplacement dans les editeurs
#
# Auteur:		Pascal Bouvier
# Creation:		16/08/95
# Modifications:	
#				PB 16/08/95 1er chantier
#				PB 04/10/95 fini de peaufiner
#				PB 16/10/95 init. de edfind:find et edfind:replace dans init
#				PB 05/02/96 bouton close
#				PB 02/04/96 setdirty quand replaceall a fait des remplacements
#				PB 28/04/96 mis readonly
#				PBO 21/10/98 La hauteur actuelle dans ShowNextFound.
#
# (c) PrologIA 1998
# -----------------------------------------------------------------------------

@com {
Pour la recherche de chaines.
-----------------------------
on sait traiter les types de recherche suivants:
litteral regexp (pattern PAS FAIT)

litteral:
	x
	case
	(fait avec [string first patt str])

regexp:
	x | . * + ? [^x-x] ^ $ ( ) \x
	case/nocase
	(fait avec [regexp -indices -- patt str])

#pattern: (PAS FAIT)
#	x * ? [x-x] \x
#	case
#	(fait avec [string match patt str])

NOTES:
nocase peut etre realise en faisant d'abord [string tolower patt str]
sur ce qu'on cherche.

Pour le remplacement de chaines.
--------------------------------
}


# -----------------------------------------------------------------------------
# Appelee pour creer le dialogue Find
# -----------------------------------------------------------------------------
proc edfind:initdial {} {
	set T .dialfind
	toplevel $T
	wm title $T [lang: T-find]
	wm iconify $T

	#
	# frame find 
	#
	frame $T.ff
	label $T.ff.lf -text [lang: FND.find]
	entry $T.ff.ef -textvar edfind:find -relief sunken
	bind $T.ff.ef  <Return> "$T.fball.bfa invoke"

	pack $T.ff.lf -side left -fill x -expand no
	pack $T.ff.ef -side left -fill x -expand yes
	pack $T.ff -side top -fill x -pady 1m

	#
	# frame replace 
	#
	frame $T.fr
	label $T.fr.lf -text [lang: FND.replace]
	entry $T.fr.ef -textvar edfind:replace -relief sunken

	pack $T.fr.lf -side left -fill x -expand no
	pack $T.fr.ef -side left -fill x -expand yes
	pack $T.fr -side top -fill x -pady 1m

	#
	# frame options (case, regexp) 
	#
	frame $T.fo
	label $T.fo.l -text [lang: FND.options]
	checkbutton $T.fo.cbc -text [lang: FND.ignore-case] -variable edfind:ignorecase
	checkbutton $T.fo.cbr -text [lang: FND.regular-expression] -variable edfind:regexp

	pack $T.fo.l $T.fo.cbc $T.fo.cbr -side left -fill x -expand no
	pack $T.fo -side top -fill x -pady 1m

	#
	# frame targets avec le menu des cibles possibles
	#
	frame $T.ft
	label $T.ft.lf -text [lang: FND.targets]
	set mbm $T.ft.mb
	menubutton $mbm -menu $mbm.m -relief raised -text [lang: FND.multi-file-find]

	menu $mbm.m
	pack $T.ft.lf $mbm -side left
	pack $T.ft -side top -fill x -pady 1m

	#
	# Met a jour le menu des fenetres ouvertes
	#
	edfind:updwin

	#
	# frame des divers boutons
	#

	#
	# buttons FindAll UnfindAll ReplaceAll
	#
	set f $T.fball
	frame $f
	button $f.bfa -text [lang: FND.find-all] \
			-command {edfind:doall_in_targets findall}
	button $f.bua -text [lang: FND.unfind-all] \
			-command {edfind:doall_in_targets unfindall}
	button $f.bra -text [lang: FND.replace-all] \
			-command {edfind:doall_in_targets replaceall}
	button $f.bcl -text [lang: FND.close] \
			-command edfind:close
	pack $f.bfa $f.bua $f.bra $f.bcl -side left -fill x -expand yes
	pack $f -side top -fill x -expand yes -pady 1m

	#
	# Les compteurs d'occurences et la gauge
	#
	label $T.lcount -textvariable edfind:count -width 5
	gauge:init $T.g
	label $T.lgcount -textvariable edfind:gcount -width 6
	pack $T.lcount $T.g $T.lgcount -side left -pady 1m
}

# -----------------------------------------------------------------------------
# Appelee qu'une fois pour initialiser le dialogue Find
# -----------------------------------------------------------------------------
proc edfind:init {} {
	#
	# Le dialogue (pas necessaire de l'initialiser ici)
	#
	edfind:initdial

	global edfind:find edfind:replace
	set edfind:find ""
	set edfind:replace ""
	#
	# ignorecase in {0,1}
	#
	global edfind:ignorecase
	set edfind:ignorecase 1
	#
	# regexp in {0,1}
	#
	global edfind:regexp
	set edfind:regexp 0
	#
	# le tableau des cibles (0/1 si cible ou pas)
	#
	global edfind:targets
	set edfind:targets(0) 0; unset edfind:targets(0)

	#
	# Enregistrement de la callback lors de la creation/destruction d'editeur
	#
	edit:recordcallback edfind:notify
}

# -----------------------------------------------------------------------------
# Pour fermer
# -----------------------------------------------------------------------------
proc edfind:close {} {
	edit:unrecordcallback edfind:notify
	catch "destroy .dialfind"
}

# -----------------------------------------------------------------------------
# "Find ..."
#
# Cree le dialogue find s'il n'existe pas deja.
# Place d'office $ed comme la seule cible
# MENU
# -----------------------------------------------------------------------------
proc edfind:ShowDial {ed} {
	if {[info command .dialfind] == ""} {
		edfind:initdial
		} \
	else {
		# le couple iconify/deiconify est plus rapide que le 
		# lent raise .dialfind (au moins en tk3.6)
		wm iconify .dialfind
		}
	edfind:untargetall
	global edfind:targets
	set edfind:targets($ed) 1
	edfind:singletarget
	wm deiconify .dialfind
	focus .dialfind.ff.ef
}

#
proc edfind:untargetall {} {
	global edfind:targets
	foreach name [array names edfind:targets] {
		set edfind:targets($name) 0
		}
}

# -----------------------------------------------------------------------------
# doall_in_targets effectue les operations suivantes selon findcom:
# "Find All"
# "UnFind All"
# "Replace All"
# -----------------------------------------------------------------------------
proc edfind:doall_in_targets {findcom} {
	upvar #0 edfind:targets targets
	global edfind:count edfind:gcount
	set edfind:count 0
	set edfind:gcount 0
	.dialfind.lcount config -foreg red
	.dialfind.lgcount config -foreg red
	foreach ed [array names targets] {
		gauge:set .dialfind.g 0
		if {$targets($ed) && [info command $ed] != ""} {
			catch "edfind:$findcom $ed"
			if ${edfind:count} {
				global edit_eds
				puts "$edit_eds($ed) : ${edfind:count} occurrence(s)."
				}
			incr edfind:gcount ${edfind:count}
			}
		}
	.dialfind.lcount config -foreg black
	.dialfind.lgcount config -foreg black
	gauge:set .dialfind.g 0
}


# -----------------------------------------------------------------------------
# Un simple relai.
# Enleve de ed tous les patterns enlumines qui matchent.
# -----------------------------------------------------------------------------
proc edfind:unfindall {ed} {
	return [edfind:findall $ed remove]
}

# -----------------------------------------------------------------------------
# La grosse machinerie de recherche de texte dans un editeur donne
# Fait les expression-regulieres/chaine-litterale (case/nocase)
# Recherche toutes les occurences
# Installe/enleve tous les tags found dans ed
# cmd vaut add ou remove
# Si la chaine a rechercher est vide, sort immediatement
# -----------------------------------------------------------------------------
proc edfind:findall {ed {cmd add}} {
	global edfind:find edfind:ignorecase edfind:regexp
	global edfind:count
	set edfind:count 0
	set t $ed.t
	if {"${edfind:find}" == ""} {
		bip; return
		}
	#
	# partie entiere (avoir la derniere ligne)
	#
	scan [$t index end] "%d" maxl

	if ${edfind:regexp} {
		#
		# astuce pour 'regexp'
		#
		if ${edfind:ignorecase} {
			set re_case -nocase
			} \
		else {
			set re_case -indices
			}

		set line 1
		set off 0
		for {} {$line <= $maxl} {incr line; set off 0} {
			set texte [$t get $line.$off $line.end]

			while {1} {
				if {0==[regexp $re_case -indices -- ${edfind:find} $texte pos]} {
					break
					} \
				else {
					set p1 [lindex $pos 0]
					set p2 [lindex $pos 1]; incr p2
					if {$p1 == $p2} {
						incr p2
						set texte [string range $texte $p2 end]
						incr off $p2
						if {$texte == ""} break
						continue
						}
					set texte [string range $texte $p2 end]

					incr p1 $off
					incr p2 $off
					$t tag $cmd found $line.$p1 $line.$p2
					set off $p2
					incr edfind:count
					if {$texte == ""} break
					}
				}
			if {$line % 50 == 1} {
				gauge:set .dialfind.g [expr ($line*100)/$maxl]
				update
				}
			}
		} \
	else {
		#
		# si ignorecase, on passe tout en minuscule
		#
		if ${edfind:ignorecase} {
			set find [string tolower ${edfind:find}]
			} \
		else {
			set find ${edfind:find}
			}
		set len [string len ${edfind:find}]

		set line 1
		set off 0
		for {} {$line <= $maxl} {incr line; set off 0} {
			set texte [$t get $line.$off $line.end]
			if ${edfind:ignorecase} {
				set texte [string tolower $texte]
				}
			while {1} {
				if {-1 == [set p1 [string first $find $texte]]} {
					break
					} \
				else {
					set p2 [expr $p1 + $len]
					set texte [string range $texte $p2 end]

					incr p1 $off
					incr p2 $off
					$t tag $cmd found $line.$p1 $line.$p2
					set off $p2
					incr edfind:count
					}
				}
			if {$line % 50 == 1} {
				gauge:set .dialfind.g [expr ($line*100)/$maxl]
				update
				}
			}
		}

	gauge:set .dialfind.g 100
	update
	edfind:ShowNextFound $ed first
}

# -----------------------------------------------------------------------------
# MENU
# -----------------------------------------------------------------------------
proc edfind:FindAll {ed} {
	global edfind:count edfind:gcount
	set edfind:count 0
	set edfind:gcount 0
	.dialfind.lcount config -foreg red
	.dialfind.lgcount config -foreg red
	gauge:set .dialfind.g 0

	edfind:findall $ed

	set edfind:gcount ${edfind:count}
	.dialfind.lcount config -foreg black
	.dialfind.lgcount config -foreg black
	gauge:set .dialfind.g 0
}


# -----------------------------------------------------------------------------
# Recherche le prochain tag 'found' (dans la page suivante)
# MENU
# On ne se sert plus du scrollbar pour obtenir les positions des lignes du texte
# -----------------------------------------------------------------------------
proc edfind:ShowNextFound {ed {f ""}} {
    set t $ed.t
#	set u [$t index @0,0+[$t cget -hei]l] ; # BUG: pas la bonne hauteur
    set u [$t index @0,[winfo hei $t]] ; # PBO 21/10/98 La hauteur actuelle.

    if {"[set r [$t tag nextrange found "$u linestart"]]" != ""} {
        set u [lindex $r 0]
        } \
	else {
		bip; return
		}
	$t yview -pickplace $u
}


# -----------------------------------------------------------------------------
# Enleve les tags 'found' d'un editeur donne
# MENU
# -----------------------------------------------------------------------------
proc edfind:UnhiliteAllFound {ed} {
	$ed.t tag remove found 1.0 end
}


# -----------------------------------------------------------------------------
# La grosse machinerie de recherche du prochain texte dans un editeur donne
# C'est quasiment une copie de edfind:findall 
# La difference est l'arret a la premiere trouvaille, et la selection de celle-ci.
# Si la chaine a rechercher est vide, sort immediatement
# Pas de gauge ni de compteur
# retourne 1 si trouve, 0 si pas trouve -1 si erreur
# MENU
# -----------------------------------------------------------------------------
proc edfind:FindNext {ed} {
	global edfind:find edfind:ignorecase edfind:regexp
	global edfind:count
	set t $ed.t
	if {"${edfind:find}" == ""} {
		bip; return -1
		}
	#
	# partie entiere (avoir la derniere ligne)
	#
	scan [$t index end] "%d" maxl

	#
	# position de la selection ou du caret
	#
	scan [$t index insert] "%d.%d" line off	

	if ${edfind:regexp} {
		#
		# astuce pour 'regexp'
		#
		if ${edfind:ignorecase} {
			set re_case -nocase
			} \
		else {
			set re_case -indices
			}

		for {} {$line <= $maxl} {incr line; set off 0} {
			set texte [$t get $line.$off $line.end]

			while {1} {
				if {0==[regexp $re_case -indices -- ${edfind:find} $texte pos]} {
					break
					} \
				else {
					set p1 [lindex $pos 0]
					set p2 [lindex $pos 1]; incr p2
					if {$p1 == $p2} {
						incr p2
						set texte [string range $texte $p2 end]
						incr off $p2
						if {$texte == ""} break
						continue
						}
					incr p1 $off
					incr p2 $off
					$t tag remove sel 1.0 end
					$t tag add sel $line.$p1 $line.$p2
					$t mark set insert $line.$p2
					eda:displayinsert $t
					return 1
					}
				}
			}
		} \
	else {
		#
		# si ignorecase, on passe tout en minuscule
		#
		if ${edfind:ignorecase} {
			set find [string tolower ${edfind:find}]
			} \
		else {
			set find ${edfind:find}
			}
		set len [string len ${edfind:find}]

		for {} {$line <= $maxl} {incr line; set off 0} {
			set texte [$t get $line.$off $line.end]
			if ${edfind:ignorecase} {
				set texte [string tolower $texte]
				}
			while {1} {
				if {-1 == [set p1 [string first $find $texte]]} {
					break
					} \
				else {
					set p2 [expr $p1 + $len]

					incr p1 $off
					incr p2 $off
					$t tag remove sel 1.0 end
					$t tag add sel $line.$p1 $line.$p2
					$t mark set insert $line.$p2
					eda:displayinsert $t
					return 1
					}
				}
			}
		}
	bip; return 0
}

# -----------------------------------------------------------------------------
# Met la selection dans le find-field puis cherche la prochaine occurence
# Si pas de selection, prend ce qu'y avait dans le find-field
# MENU
# -----------------------------------------------------------------------------
proc edfind:FindSelection {ed} {
	edfind:EnterFind $ed
	catch "$ed.t mark set insert sel.last"
	edfind:FindNext $ed
}

@com {
Si on utilise les expressions regulieres, le remplacement se fait avec regsub :
	regsub ?switches? -- exp string subSpec varName
avec switches in {-all, -nocase}
Dans ce cas, l'expression reguliere doit pouvoir se trouver dans la selection.
Sans les expressions regulieres, le remplacement se fait par une chaine constante, quoi qu'il y ait dans la selection.
}
# -----------------------------------------------------------------------------
# Remplace la selection par la chaine (ou regexp) de remplacement
# Sort de suite si pas de selection
# Retourne 1 si OK (remplacement), 0 sinon
# MENU
# -----------------------------------------------------------------------------
proc edfind:Replace {ed} {
	global edfind:find edfind:replace edfind:ignorecase edfind:regexp
	set t $ed.t
    if [ed:isreadonly $t] {return 0}

	if [catch "$t index sel.first"] {bip; return 0}

	set txt [$t get sel.first sel.last]
	
	if ${edfind:regexp} {
		if ${edfind:ignorecase} {
			if {0==[regsub -nocase -- ${edfind:find} $txt ${edfind:replace} S]} {
				bip; return 0
				}
			} \
		else {
			if {0==[regsub -- ${edfind:find} $txt ${edfind:replace} S]} {
				bip; return 0
				}
			}
		} \
	else {
		set S ${edfind:replace}
		}
	$t delete sel.first sel.last
	$t insert insert $S
	ed:setdirty $t
	return 1
}

# -----------------------------------------------------------------------------
# Remplace la selection par la chaine (ou regexp) de remplacement
# Puis cherche le suivant
# MENU
# -----------------------------------------------------------------------------
proc edfind:ReplaceAndFindNext {ed} {
	edfind:Replace $ed
	edfind:FindNext $ed
}


# -----------------------------------------------------------------------------
# La grosse machinerie de remplacement de texte dans un editeur donne
# Fait les expression-regulieres/chaine-litterale (case/nocase)
# Recherche et remplace toutes les occurences
# Si la chaine a rechercher est vide, sort immediatement
# NOTE:
# en tcl7.3 le compte de regsub est le nombre de lignes changees
# en tcl7.4 ce sera le nombre de remplacements. (rien a faire)
# -----------------------------------------------------------------------------
proc edfind:replaceall {ed} {
	global edfind:find edfind:replace edfind:ignorecase edfind:regexp
	global edfind:count
	set edfind:count 0
	set t $ed.t
    if [ed:isreadonly $t] {return 0}

	if {"${edfind:find}" == ""} {
		bip; return
		}
	#
	# partie entiere (avoir la derniere ligne)
	#
	scan [$t index end] "%d" maxl

	if ${edfind:regexp} {
		#
		# astuce pour 'regsub'
		#
		if ${edfind:ignorecase} {
			set re_case -nocase
			} \
		else {
			set re_case -all
			}

		set line 1
		for {} {$line <= $maxl} {incr line} {
			set texte [$t get $line.0 $line.end]

			if {0 == [set nb [regsub $re_case -all -- \
						${edfind:find} \
						[$t get $line.0 $line.end] \
						${edfind:replace} \
						Subs]]} {
				} \
			else {
				$t delete $line.0 $line.end
				$t insert $line.0 $Subs
				incr edfind:count $nb
				}
			
			if {$line % 50 == 1} {
				gauge:set .dialfind.g [expr ($line*100)/$maxl]
				update
				}
			}
		} \
	else {
		#
		# si ignorecase, on passe tout en minuscule
		#
		if ${edfind:ignorecase} {
			set find [string tolower ${edfind:find}]
			} \
		else {
			set find ${edfind:find}
			}
		set len [string len ${edfind:find}]

		set line 1
		set off 0
		for {} {$line <= $maxl} {incr line; set off 0} {
			set orgtexte [$t get $line.$off $line.end]
			set orgt $orgtexte
			if ${edfind:ignorecase} {
				set texte [string tolower $orgtexte]
				} \
			else {
				set texte $orgtexte
				}
			set Subs ""
			while {1} {
				if {-1 == [set p1 [string first $find $texte]]} {
					append Subs $orgt
					break
					} \
				else {
					append Subs [string range $orgt 0 [expr $p1 -1]]
					append Subs ${edfind:replace}
					set p2 [expr $p1 + $len]
					set texte [string range $texte $p2 end]
					set orgt [string range $orgt $p2 end]
					incr edfind:count
					}
				}
			if {[string compare $Subs $orgtexte]} {
				$t delete $line.0 $line.end
				$t insert $line.0 $Subs
				}
			if {$line % 50 == 1} {
				gauge:set .dialfind.g [expr ($line*100)/$maxl]
				update
				}
			}
		}

	gauge:set .dialfind.g 100
	if ${edfind:count} {
		ed:setdirty $t
		}
	update
}

# -----------------------------------------------------------------------------
# MENU
# -----------------------------------------------------------------------------
proc edfind:ReplaceAll {ed} {
	global edfind:count edfind:gcount
	set edfind:count 0
	set edfind:gcount 0
	.dialfind.lcount config -foreg red
	.dialfind.lgcount config -foreg red
	gauge:set .dialfind.g 0

	edfind:replaceall $ed

	set edfind:gcount ${edfind:count}
	.dialfind.lcount config -foreg black
	.dialfind.lgcount config -foreg black
	gauge:set .dialfind.g 0
}

# -----------------------------------------------------------------------------
# Met a jour le menu en fonction des editeurs existants
# # ajoute a la fin 'console' et 'all'
# c'est utilise dans le binding <Button-1/2>
# on met a jour parallelement edfind:targets
# -----------------------------------------------------------------------------
proc edfind:updwin {} {
	set m [lindex [.dialfind.ft.mb config -menu] 4]
	$m delete 0 last
	global edit_eds
	global edfind:targets

	#
	# on ajoute aux cibles des nouveaux editeurs
	#
	foreach i [array names edit_eds] {
		set ti [wm title [winfo toplevel $i]]
		$m add checkbutton -label $ti -command edfind:singletarget \
				-variable edfind:targets($i) -selectcolor green
		}
#	$m add checkbutton -label console
#	$m add command -label all
}

# -----------------------------------------------------------------------------
# Donne la selection au champ Find du dialogue
# retourne 1 si ok, 0 sinon
# MENU
# -----------------------------------------------------------------------------
proc edfind:EnterFind {ed} {
	set t $ed.t
	if [catch "$t index sel.first"] {
		bip; return 0
		}
	global edfind:find
	set edfind:find [$t get sel.first sel.last]
	return 1
}

# -----------------------------------------------------------------------------
# Donne la selection au champ Replace du dialogue
# MENU
# -----------------------------------------------------------------------------
proc edfind:EnterReplace {ed} {
	set t $ed.t
	if [catch "$t index sel.first"] {
		bip; return
		}
	global edfind:replace
	set edfind:replace [$t get sel.first sel.last]
}

# -----------------------------------------------------------------------------
# Les notifieurs appeles depuis l'editeur
# Il faut les avoir enregistres
# -----------------------------------------------------------------------------
proc edfind:notify_record {ed file} {
	global edfind:targets
	set edfind:targets($ed) 0
	set m [lindex [.dialfind.ft.mb config -menu] 4]
	set ti [wm title [winfo toplevel $ed]]
	$m add checkbutton -label $ti -command edfind:singletarget \
			-variable edfind:targets($ed) -selectcolor green
	edfind:singletarget
}

#
proc edfind:notify_unrecord {ed} {
	global edfind:targets edit_eds
	unset edfind:targets($ed)

	set m [lindex [.dialfind.ft.mb config -menu] 4]
	set ti [edit:longtitle $edit_eds($ed)]
	$m delete $ti
	edfind:singletarget
}

#
proc edfind:singletarget {} {
	global edit_eds
	upvar #0 edfind:targets targets

	set sum 0
	foreach ed [array names targets] {
		incr sum $targets($ed)
		if $targets($ed) {
			set theed $ed
			}
		}
	switch -exact $sum {
		0
			{ set title [lang: FND.none] }
		1
			{ set title [edit:longtitle $edit_eds($theed)]}
		default
			{ set title [lang: FND.multi-file-find] }
		}
	.dialfind.ft.mb config -text $title
}

#-- EOF find.tcl --------------------------------------------------------------
