#-- graf.tcl
# Quelques primitives graphiques pour Prolog IV
# Pascal Bouvier
# (c) PrologIA 1995-98
#
# ------------------------------------------------------------------------------

# Liste des clicks (liste de doublets {x y})
global list_clicks
set list_clicks {}

# ------------------------------------------------------------------------------
# A appeler au debut
# ------------------------------------------------------------------------------
proc tcl_inigraphic {} {
	global list_clicks
	set list_clicks {}

	catch "destroy .g"
	toplevel .g
	wm title .g "graphic"
	wm minsize .g  1 1
	canvas .g.c -bg #c0ffc0
	global .g.l.v
	set .g.l.v "0:0"
	label .g.l -width 20 -textvar .g.l.v -text "0:0"
	frame .g.fbot
	pack .g.fbot -side bottom -fill x
	frame .g.grow -width 18 -relief raised
	scrollbar .g.sv -orient vert  -relief sunken
	scrollbar .g.sh -orient horiz -relief sunken
	pack .g.l -in .g.fbot -side left 
	pack .g.sh  -in .g.fbot -fill x -side left -expand 1
	pack .g.grow -in .g.fbot -side left
	pack .g.c -side left -fill both -expand 1
	pack .g.sv -side left -fill both

	.g.c config -xscrollcom {.g.sh set}
	.g.c config -yscrollcom {.g.sv set}
	.g.c config -scrollreg {0 0 10000 10000}
	.g.c config -selectbor 2 -selectba pink -selectfor blue
	.g.sv config -command {.g.c yview}
	.g.sh config -command {.g.c xview}

	bind .g.c <B2-Motion> {.g.c scan dragto %x %y;  }
	bind .g.c <Button-2> {.g.c scan mark %x %y;  }
	bind .g.c <Motion> {set .g.l.v [format "%%d : %%d" %x %y]}
	# pour les clics
	bind .g.c <Button-1> {lappend list_clicks [list %x %y]}

	global cur_graphic
	set cur_graphic .g.c
	global cur_fcolor
	set cur_fcolor black
	global cur_x cur_y
	set cur_x 0
	set cur_y 0
	global cur_font
	set cur_font [font: "Helvetica 12 bold"]
	global cur_tag
	set cur_tag {}
}

if 0 {
set untexte [.g.c creat text 20 20 -text {ceci est une ligne de texte}  -an nw]
.g.c focus $untexte
.g.c select from $untexte 0
.g.c select to $untexte 10
}

# ------------------------------------------------------------------------------
# Installe un gris comme couleur des prochains objets a creer
# ------------------------------------------------------------------------------
proc tcl_gr_fggray {c} {
	if {$c == 1.0} {
		set col white
		} \
	elseif {$c == 0.0} {
		set col black
		} \
	elseif {$c == 0.5} {
		set col gray
		}
	global cur_fcolor
	set cur_fcolor $col
}

# ------------------------------------------------------------------------------
# Installe une couleur sous la forme r,g,b, des prochains objets a creer
# ------------------------------------------------------------------------------
proc tcl_gr_fgcolor3 {r g b} {
	global cur_fcolor
	set cur_fcolor [format "#%02x%02x%02x" $r $g $b]
}

# ------------------------------------------------------------------------------
# Installe une couleur des prochains objets a creer
# ------------------------------------------------------------------------------
proc tcl_gr_fgcolor {col} {
	global cur_fcolor
	set cur_fcolor $col
}

# ------------------------------------------------------------------------------
# Cree le texte S en position x,y
# ------------------------------------------------------------------------------
proc tcl_gr_drawstring_at {s x y} {
	global cur_graphic cur_fcolor
	global cur_font cur_tag
	set x0 [expr $x]
	set y0 [expr $y]

	set t [$cur_graphic creat text $x0 $y0 -text $s \
		-tags $cur_tag \
		-font $cur_font \
		-fill $cur_fcolor]
	return $t
}

# ------------------------------------------------------------------------------
# Cree le texte S en position courante du crayon (en bas a droite = nw).
# ------------------------------------------------------------------------------
proc tcl_gr_drawstring {s} {
	global cur_graphic cur_fcolor
	global cur_font cur_tag cur_x cur_y
	set x0 $cur_x
	set y0 $cur_y

	set t [$cur_graphic creat text $x0 $y0 -anchor nw -text $s \
		-tags $cur_tag \
		-font $cur_font \
		-fill $cur_fcolor]
	return $t
}


# ------------------------------------------------------------------------------
# Cree le texte S dans le cadre x,y x1,y1
# ------------------------------------------------------------------------------
proc tcl_gr_drawstring_in {s x y x1 y1} {
	global cur_graphic cur_fcolor
	global cur_font cur_tag
	set x0 [expr $x]
	set y0 [expr $y]
	set x10 [expr $x1]
	set y10 [expr $y1]
#	$cur_graphic creat rect $x0 $y0 $x10 $y10 \
#		-tags $cur_tag \
#		-outline red
	set mx [expr ($x1-$x)/2 + $x]
	set my [expr ($y1-$y)/2 + $y]

	set t [$cur_graphic creat text $mx $my -text $s \
		-tags $cur_tag \
		-font $cur_font \
		-an c -fill $cur_fcolor \
		]
#	$cur_graphic select from $t 0
#	$cur_graphic select to $t end
}

# ------------------------------------------------------------------------------
# Cree une fleche de x,y vers x1,y1
# ------------------------------------------------------------------------------
proc tcl_gr_arrow {x y x1 y1} {
	global cur_graphic cur_fcolor cur_tag
	set x0 [expr $x]
	set y0 [expr $y]
	set x10 [expr $x1]
	set y10 [expr $y1]
	$cur_graphic create line $x0 $y0 $x10 $y10 \
		-tags $cur_tag \
		-fill $cur_fcolor \
		-wid 1 -arrow last
}

# ------------------------------------------------------------------------------
# Deplace le crayon en x,y
# ------------------------------------------------------------------------------
proc tcl_gr_moveto {x y} {
	global cur_x cur_y
	set cur_x [expr $x]
	set cur_y [expr $y]
}

# ------------------------------------------------------------------------------
# Deplace le crayon de dx,dy
# ------------------------------------------------------------------------------
proc tcl_gr_move {dx dy} {
	global cur_x cur_y
	set cur_x [expr $cur_x + $dx]
	set cur_y [expr $cur_y + $dy]
}


# ------------------------------------------------------------------------------
# Recupere les coordonnees du crayon en x,y
# ------------------------------------------------------------------------------
proc tcl_gr_penloc {} {
	global cur_x cur_y f1
	return "\[$cur_x, $cur_y\]. "
#old	conn:repondre "\[$cur_x, $cur_y\]. " $f1
}

# ------------------------------------------------------------------------------
# Dessine une ligne de l'endroit courant du crayon vers x,y
# Deplace le crayon en x,y
# ------------------------------------------------------------------------------
proc tcl_gr_lineto {x y} {
	global cur_x cur_y cur_fcolor
	global cur_graphic cur_tag
	set x0 [expr $x]
	set y0 [expr $y]
	$cur_graphic creat line $cur_x $cur_y $x0 $y0 \
		-tags $cur_tag \
		-fill $cur_fcolor
	set cur_x $x0
	set cur_y $y0
}

# ------------------------------------------------------------------------------
# Dessine une ligne brisee (polygone)
# ------------------------------------------------------------------------------
proc tcl_gr_polyline {args} {
	global cur_x cur_y cur_fcolor
	global cur_graphic cur_tag
	set L {}
	foreach c $args {
		set c0 [expr $c]
		lappend L $c0
		}
	eval $cur_graphic creat line $L \
		-tags \{$cur_tag\} \
		-fill $cur_fcolor
}



# ------------------------------------------------------------------------------
# Dessine une ligne de l'endroit courant du crayon d'une quantite dx,dy
# Deplace le crayon en fin de trait
# ------------------------------------------------------------------------------
proc tcl_gr_line {dx dy} {
	global cur_x cur_y cur_fcolor
	global cur_graphic cur_tag
	set x0 [expr $cur_x + $dx]
	set y0 [expr $cur_y + $dy]
	$cur_graphic creat line $cur_x $cur_y $x0 $y0 \
		-tags $cur_tag \
		-fill $cur_fcolor
	set cur_x $x0
	set cur_y $y0
}

# ------------------------------------------------------------------------------
# Dessine un rectangle (contour ou remplissage)
# ------------------------------------------------------------------------------
proc tcl_gr_rect {o x y x1 y1} {
	global cur_graphic cur_fcolor cur_tag
	set X [expr $x]
	set Y [expr $y]
	set X1 [expr $x1]
	set Y1 [expr $y1]
	switch $o {
		frame -
		frameRect
			{$cur_graphic creat rect $X $Y $X1 $Y1 \
				-tags $cur_tag \
				-outline $cur_fcolor}
		
		paint -
		paintRect
			{$cur_graphic creat rect $X $Y $X1 $Y1 \
				-tags $cur_tag \
				-outline $cur_fcolor \
				-fill $cur_fcolor}
		frameOval
			{$cur_graphic creat oval $X $Y $X1 $Y1 \
				-tags $cur_tag \
				-outline $cur_fcolor}
		
		paintOval
			{$cur_graphic creat oval $X $Y $X1 $Y1 \
				-tags $cur_tag \
				-outline $cur_fcolor \
				-fill $cur_fcolor}
		erase
			{
			set L [$cur_graphic find enclosed $X $Y $X1 $Y1]
			eval $cur_graphic delete $L
			}
		eraseoverlap
			{
			set L [$cur_graphic find overlapping $X $Y $X1 $Y1]
			eval $cur_graphic delete $L
			}
		}
}

# ------------------------------------------------------------------------------
# Met une fonte donnee pour creation ulterieure de texte
# ------------------------------------------------------------------------------
proc tcl_gr_setxfont {f} {
	global cur_font
	set cur_font $f
}

# ------------------------------------------------------------------------------
# Met une fonte donnee pour creation ulterieure de texte
# ------------------------------------------------------------------------------
proc tcl_gr_setfont {fn {s 12} {b Bold}} {
	global cur_font tcl_platform
	if {"$tcl_platform(platform)" == "windows"} {
		set cur_font [list $fn $s $b]
		} \
	else {
		set cur_font [format "-Adobe-%s-%s-R-Normal--*-%d0-*" $fn $b $s]
		}
}

# ------------------------------------------------------------------------------
proc zoom {{zx 2} {zy 2}} {
	global cur_graphic
	foreach i [$cur_graphic find all] {
		$cur_graphic scale $i 0 0 $zx $zy
		}
}

proc hidetext {{x 10000} {y 10000}} {
	global cur_graphic
	foreach i [$cur_graphic find all] {
		if {[$cur_graphic type $i] == "text"} {
			$cur_graphic move $i $x $y
			}
		}
}

proc showtext {{x -10000} {y -10000}} {
	global cur_graphic
	foreach i [$cur_graphic find all] {
		if {[$cur_graphic type $i] == "text"} {
			$cur_graphic move $i $x $y
			}
		}
}

proc recolortext {{col black}} {
	global cur_graphic
	foreach i [$cur_graphic find all] {
		if {[$cur_graphic type $i] == "text"} {
			$cur_graphic itemconf $i -fill $col
			}
		}
}



# ------------------------------------------------------------------------------
# Efface tous les objets tagges T (ou tous si non-specifies)
# ------------------------------------------------------------------------------
proc tcl_gr_erase {{T all}} {
	global cur_graphic cur_fcolor
	if {"$T" == "all"} {
		eval $cur_graphic del [$cur_graphic find all]
		} \
	else {
		eval $cur_graphic del [$cur_graphic find withtag $T]
		}
}

#
proc tcl_gr_objmove {T dx dy} {
	global cur_graphic
	foreach i [$cur_graphic find $T] {
		$cur_graphic move $i [expr $dx] [expr $dy]
		}
}

# ------------------------------------------------------------------------------
# Installe le tag T pour les prochains objets crees
# ------------------------------------------------------------------------------
proc tcl_gr_settag {T} {
	global cur_tag
	set cur_tag $T
}

# ------------------------------------------------------------------------------
# Vire tous les clicks tapes d'avance.
# ------------------------------------------------------------------------------
proc tcl_gr_flush_clicks {} {
	global list_clicks
	set list_clicks {}
}

# ------------------------------------------------------------------------------
# Recupere les coordonnes d'un clic dans la fenetre (X,Y).
# Fait echouer prolog (en ne rendant pas une liste) si pas de click.
# ------------------------------------------------------------------------------
proc tcl_gr_click_nowait {} {
	global list_clicks
	if {[string length $list_clicks] != 0} {
		set clic [lindex $list_clicks 0]
		set list_clicks [lrange $list_clicks 1 end]
		return "\[[lindex $clic 0], [lindex $clic 1]\]. "
		} \
	else {
		return NOCLICK
		}
}

# ------------------------------------------------------------------------------
# Recupere les coordonnes d'un clic dans la fenetre (X,Y).
# Attend s'il n'y en a pas d'avance.
# ------------------------------------------------------------------------------
proc tcl_gr_click_wait {} {
	global list_clicks
	while {[string length $list_clicks] == 0} {
		after 200
		update
		}
	
	set clic [lindex $list_clicks 0]
	set list_clicks [lrange $list_clicks 1 end]
	return "\[[lindex $clic 0], [lindex $clic 1]\]. "
}


# ------------------------------------------------------------------------------
# Illumination de lignes de texte d'une fenetre dont on donne le fichier.
# Le message a y associer est une suite de mots dans args.
# - Si c'est le fichier temporaire servant a la compilation, on deduit
# la fenetre d'origine.
# - sinon, on ouvre le fichier (si pas deja ouvert) dans une fenetre.
# Appele par prolog (discipl) [predicat gr_illuminer(File, Lig1, Lig2, LTerms)]
# ------------------------------------------------------------------------------
proc tcl_open_file_and_highlight_lines {F l1 l2 args} {
	puts "[info level 0]"
	global prolog:tmpficed
	set tmpfic [lindex ${prolog:tmpficed} 0]
	set ed [lindex ${prolog:tmpficed} 1]
	if {[string compare $tmpfic.p4 $F] == 0} {
		# Parce que le fichier tmp.p4 contient deux lignes en plus au debut.
		# (:-unit et :-debug)
		incr l1 -2
		incr l2 -2
		} \
	else { # ce n'est pas le fichier tmp.
		if {[string len [set ed [edit:deja_ouvert [pwd]/$F]]]} {
			#rien } \
		elseif {[string len [set ed [edit:deja_ouvert $F]]]} {
			#rien } \
		else { # on l'ouvre dans une nouvelle fenetre
			set ed [edit:open_window $F]
			}
		}
	set top [winfo toplevel $ed]
	raise $top
	wm deiconify $top

	# Parce qu'on est deja sur la premiere ligne
	incr l1 -1
	incr l2 -1

	set deb "1.0+$l1 l linestart"
	set fin "1.0+$l2 l lineend +1c"
	if {[$ed.t compare $deb == end]} {
		# Si on est a la fin, on selectionne la derniere ligne.
		set deb "end-1l"
		set fin "end"
		}
	
	$ed.t tag add erreur $deb $fin
	$ed.t tag add "syntaxerror-assertion $args" $deb $fin
	$ed.t see $deb
}


#--- EOF graf.tcl ---------------------------------------------------#
