# -- newcons.tcl
# La console pour Prolog IV (ou autre executable Unix)
#
# Auteur:		Pascal Bouvier
# Creation:		../01/95
# Modifications:	
#				PBO 23/10/95 3eme chantier
#				PBO 05/02/96 autolaunch de l'exe
#				PBO 09/04/96 P4_EXECUTABLE
#				PBO 25/04/96 refait la proc obtient et fait le flag usestcl
#				PBO ../11/96 refait la proc obtient (tk4)
#				PBO 21/10/98 changement dans clockclicksms (pour les unixes)
#
# (c) PrologIA 1998
# -----------------------------------------------------------------------------


# ==== G L O B A L E S =======================================

global cons:echo
set cons:echo 1

#
# Les differents parametrages modifiables par l'utilisateur
# indexes par: exe argv userargv autolaunch
#
global cons:pref env BINP4
if [catch "set env(P4_EXECUTABLE)" cons:pref(exe)] {
	set cons:pref(exe) $BINP4/prolog4.exe
	}

set cons:pref(argv) ""
set cons:pref(userargv) ""
set cons:pref(autolaunch) 0
set cons:pref(usestcl) 0

global cons:tmp

# Tableau des buts
global cons:goals
# Liste des nums de buts
global cons:listgoals
# Position dans la liste de but
global cons:pos
# Numero du prochain libre
global cons:1free

# Pour les non-portables
global tcl_platform

# ou vont les sorties
global conn:current_wout
set conn:current_wout ""

# ==== P R O C S =======================================


# -----------------------------------------------------------------------------
# Construit la commande a lancer, avec tous les arguments
# tires des preferences.
# -----------------------------------------------------------------------------
proc cons:com {} {
	upvar #0 cons:pref pref
	if $pref(usestcl) {set tcl -tcl} else {set tcl ""}

	lappend L $pref(exe)
	if $pref(usestcl) {lappend L -tcl}
	if {"$pref(argv)" != ""} {
		set L [concat $L $pref(argv)]
		}
	if {"$pref(userargv)" != ""} {
		lappend L "--"
		set L [concat $L $pref(userargv)]
		}
	return $L

#	if {"$pref(userargv)" != ""} {
#		return "$pref(exe) $tcl $pref(argv) -- $pref(userargv)"
#		} \
#	else {
#		return "$pref(exe) $tcl $pref(argv)"
#		}
}

# -----------------------------------------------------------------------------
# (re)demarrer Prolog: le double pipe est dans la globale f1
# -----------------------------------------------------------------------------
proc cons:restart {} {
	global f1 conn:current_wout
	catch "cons:killprolog"
#3 NONBLOCK non portable sur PC
	set f1 [open |[cons:com] RDWR]
	process:record [pid $f1]
	fconfigure $f1 -blocking 0 -buffering none
	set conn:current_wout .cons.frtty.t

	global indomessage
	set indomessage 0

global tcl_platform
if {$tcl_platform(platform) == "windows"} {
	set_p4_nbtclqueries 0
	set_prolog_event 0
	# pour que l'environnement reagisse vite a l'utilisateur (Hight)
	winrenice H
	}
}

# -----------------------------------------------------------------------------
# Tuer Prolog
# -----------------------------------------------------------------------------
proc cons:killprolog {} {
	global f1 semsend ack
	set semsend 0
	set ack 2
	after cancel "cons:listener"
	process:kill [pid $f1]
	catch {close $f1}

global tcl_platform
if {$tcl_platform(platform) == "windows"} {
	# on diminue la priorite de l'environnement graphique (Normal)
	winrenice N
	}
}

# -----------------------------------------------------------------------------
# Interrompre Prolog (user-interrupt)
# -----------------------------------------------------------------------------
proc conn:ctrl-c {} {
	global f1 tcl_platform
	if {$tcl_platform(platform) == "unix"} {
		kill SIGINT [pid $f1]
		} \
	else {
		t2p_send_ctrlc_event
		}
}

# -----------------------------------------------------------------------------
# Faire passer Prolog en debug mode
# -----------------------------------------------------------------------------
proc conn:debug-on {} {
	global f1
	conn:pro_call "setdbg" 1 $f1
}

# =============================================================================
# Communication par chaines
# =============================================================================

# -----------------------------------------------------------------------------
# Envoi d'une chaine dans le fichier de communication
# -----------------------------------------------------------------------------

global nbsigusr1 semsend ack
set nbsigusr1 0
set semsend 0

#
# Signal de synchronisation: un message a ete envoye
#
proc conn:signal-SIGUSR1 {pid} {
	global tcl_platform
	if {"$tcl_platform(platform)" == "windows"} {
		t2p_incr_query_nb
		} \
	else {
		kill SIGUSR1 $pid
		}
}

global conn:messid
set conn:messid 0

# -----------------------------------------------------------------------------
# Nouvelle identification de message
# -----------------------------------------------------------------------------
proc conn:newid {} {
	global conn:messid
	return [incr conn:messid]
}

#
# On envoie des caracteres de s dans f
# Dans le cas 'usetcl', ce sont des messages qui sont envoyes,
# par paquets de 900 chars au plus.
# 
# Retourne messid si ok, 0 sinon
# (dans le cas !usetcl, il n'y a pas de message)
#
proc conn:envoie {s f} {
	upvar #0 cons:pref(usestcl) usestcl

	if { ! $usestcl } {
		puts $f $s; flush $f
		return 1
		}

	global semsend nbsigusr1 ack
	if $semsend {return 0}
	set semsend 1
	errputs "ine $nbsigusr1"

	while 1 {
		set ack 0
		incr nbsigusr1
		conn:signal-SIGUSR1 [pid $f]
		set mess [string range $s 0 900]
		set s [string range $s 901 end]
		set messid [conn:newid]
		set s1 "\x1bpro_get $messid [translit \n \x01 $mess]"
		puts $f $s1; flush $f
		while {! $ack} {conn:poll}
		if {$ack == 2} {
			errputs "Aborting 'conn:envoie'"
			return 0
			}
		if {"$s" == ""} break
		}

	errputs "oute $nbsigusr1"
	set semsend 0
	return $messid
}

#
# version publique de conn:envoie
#
proc conn:send {s} {
	global f1
	return [conn:envoie $s $f1]
}

#
# Repond au message replyid
# Retourne messid si ok, 0 sinon
#
proc conn:repondre {s replyid f} {
	global semsend nbsigusr1 ack
	if $semsend {return 0}
	errputs "inr $nbsigusr1"
	set semsend 1
	set ack 0
	incr nbsigusr1
	conn:signal-SIGUSR1 [pid $f]
	set messid [conn:newid]
	set s1 "\x1bpro_answ $messid $replyid [translit \n \x01 $s]"
	errputs "\[mess=$s1=ssem\]"
	puts $f $s1; flush $f
	while {! $ack} {conn:poll}
	if {$ack == 2} {
		errputs "Aborting 'conn:envoie'"
		return 0
		}

	errputs "outr $nbsigusr1"
	set semsend 0
	return $messid
}

#
# Retourne l'id de la requete RECUE, 0 sinon.
#
proc conn:pro_fun {fun s f} {
	global semsend nbsigusr1 ack tab_answer conn:current_wout
	if $semsend {return 0}
	errputs "infun $nbsigusr1"
	set semsend 1
	set ack 0
	incr nbsigusr1
	conn:signal-SIGUSR1 [pid $f]
	set messid [conn:newid]
	set s1 "\x1bpro_$fun $messid [translit \n \x01 $s]"
	puts $f $s1; flush $f
	while {! $ack} {conn:poll}
	if {$ack == 2} {
		errputs "Aborting 'conn:envoie'"
		return 0
		}
#3	while {[obtient ${conn:current_wout} $f] != "ANSWER"} {true}
#4	while {[cons:domessage] != "ANSWER"} {after 200 ; true}
	#
	# Nouveau
	#
	conn:wait_answer $messid

	errputs "outfun $nbsigusr1 ($tab_answer($messid))"
	set semsend 0
	return $messid
}

#
proc conn:pro_call {proc s f} {
	global semsend nbsigusr1 ack
	if $semsend {return 0}
	errputs "incall $nbsigusr1"
	set semsend 1
	set ack 0
	incr nbsigusr1
	conn:signal-SIGUSR1 [pid $f]
	set messid [conn:newid]
	set s1 "\x1bpro_$proc $messid [translit \n \x01 $s]"
	puts $f $s1; flush $f
	while {! $ack} {conn:poll}
	if {$ack == 2} {
		errputs "Aborting 'conn:envoie'"
		return 0
		}

	errputs "outcall $nbsigusr1"
	set semsend 0
	return $messid
}

#
#
#
proc conn:wait_answer {messid} {
	global conn:waitmess
	set conn:waitmess $messid
	while {[set id [cons:domessage 0]] != "$messid"} {
		after 200
		}
	set conn:waitmess 0
}

#
# 
#
if {$tcl_platform(platform) == "windows"} {
	proc conn:poll {} {
		after 100
		global ack
		set ack 1	
	}
} else {
	proc conn:poll {} {
		after 100
		global ack
		p4loadsigvars _ ack _
		if $ack {errputs Ack}
		}
}


proc true {} {}

# TMP :
# pour compatibilite seulement avec des appels restants
#
proc envoie {s} {
	global f1
	puts "******** APPEL OBSOLETE A envoie ********"
	puts -nonewline $f1 $s; flush $f1;
}

# -----------------------------------------------------------------------------
# Lecture dans le fichier de communication
#3
# -----------------------------------------------------------------------------
#proc conn:rece {f} {
#	set r [gets $f]
#		errputs <<$r>>
#	return [translit \x1 \n $r]
#}

global inrece
set inrece 0
global theline
set theline {}
# -----------------------------------------------------------------------------
# Recoit un paquet entier (commande entiere) (une ligne donc)
# ou rien, si rien n'est disponible.
# -----------------------------------------------------------------------------
proc conn:rece {f} {
global theline
global inrece

set inrece 1
	if {"$theline" == ""} {
set inrece 2
		if {[catch "read $f" theline]} {
set inrece 3
			global errorInfo
			errputs $errorInfo
set inrece 0
			return -code error
			}
		if {"$theline" == ""} {
set inrece 0
			return ""
			}
		}

set inrece 4
	while 1 {
set inrece 5
		if {[set i [string first \n $theline]] != -1} break
set inrece 6
		if {[catch "read $f" line]} {
set inrece 7
			global errorInfo
			errputs $errorInfo
set inrece 0
			return -code error
			}
		errputs "{{$line}}"
set inrece 8
		append theline $line
set inrece 9
		}
	set res [string range $theline 1 [expr $i-1]]
set inrece 10
	set theline [string range $theline [expr $i +1] end]
set inrece 11
#		errputs <<$res>>
set inrece 0
	return [translit \x1 \n $res]
	}

# -----------------------------------------------------------------------------
# Recoit des octets du coprocessus
# ou rien, si rien n'est disponible.
# -----------------------------------------------------------------------------
proc conn:rece_notcl {f} {
global theline

	if {[catch "read $f" theline]} {
		global errorInfo
		errputs $errorInfo
		return -code error
		}
	return $theline
	}


global cons:fonthistory
global tcl_platform
if {$tcl_platform(platform) == "unix"} {
	set cons:fonthistory "*-Courier-Medium-R-Normal-*-140-*"
	} \
else {
	set cons:fonthistory {"courier new" 14 bold}
	}

# -----------------------------------------------------------------------------
# Envoie le texte du panneau de but dans la console
# -----------------------------------------------------------------------------
proc cons:envoiehistory {} {
	global cons:echo conn:current_wout
	if ${cons:echo} {
	#	tty:inserer_text ${conn:current_wout} [.cons.fh.thist get 1.0 end-1c]\n
		tty:text_any_key ${conn:current_wout} [.cons.fh.thist get 1.0 end-1c]\n
		} \
	else {
		global f1
		conn:envoie [.cons.fh.thist get 1.0 end-1c]\n $f1
		}
}


# -----------------------------------------------------------------------------
# Efface le panel de but
# -----------------------------------------------------------------------------
proc cons:newhistory {} {
	.cons.fh.thist delete 1.0 end
}

# -----------------------------------------------------------------------------
# Initialisation de l'history (les buts)
# -----------------------------------------------------------------------------
proc cons:initgoals {} {
	global cons:goals cons:listgoals cons:pos cons:1free
	upvar #0 cons:goals goals
	set goals(0) ""
	set cons:listgoals {0}
	set cons:pos 0
	set cons:1free 1
}

# -----------------------------------------------------------------------------
# On sauve du pad courant dans le tableau
# -----------------------------------------------------------------------------
proc cons:savegoal {} {
	global cons:goals cons:listgoals cons:pos cons:1free
	upvar #0 cons:goals goals
	#
	# On sauve le courant dans le tableau
	#
	set cur [lindex ${cons:listgoals} ${cons:pos}]
	set goals($cur) [.cons.fh.thist get 1.0 end-1c]
}

# -----------------------------------------------------------------------------
# Creation d'un nouveau pad de but
# -----------------------------------------------------------------------------
proc cons:newgoal {{com {}}} {
	global cons:goals cons:listgoals cons:pos cons:1free
	upvar #0 cons:goals goals
	#
	# On sauve le courant dans le tableau
	#
	set cur [lindex ${cons:listgoals} ${cons:pos}]
	set goals($cur) [.cons.fh.thist get 1.0 end-1c]

	#
	# On en cree un nouveau
	#
	incr cons:pos
	if { $com == "dup" } {
		set goals(${cons:1free}) $goals($cur)
		} \
	else {
		set goals(${cons:1free}) ""
		}
	set cons:listgoals [linsert ${cons:listgoals} ${cons:pos} ${cons:1free}]
	.cons.fh.thist delete 1.0 end
	.cons.fh.thist insert insert $goals(${cons:1free})
	incr cons:1free
}

# -----------------------------------------------------------------------------
# Destruction du pad de but
# -----------------------------------------------------------------------------
proc cons:killgoal {} {
	global cons:goals cons:listgoals cons:pos cons:1free
	upvar #0 cons:goals goals

	if {[llength ${cons:listgoals}] == 1} {
		bip
		return
		}

	#
	# On detruit celui en cons:pos
	#
	set cur [lindex ${cons:listgoals} ${cons:pos}]
	unset goals($cur)
	.cons.fh.thist delete 1.0 end

	set cons:listgoals [lreplace ${cons:listgoals} ${cons:pos} ${cons:pos}]
	if {${cons:pos} >= [llength ${cons:listgoals}]} {
		incr cons:pos -1
		}
	set cur [lindex ${cons:listgoals} ${cons:pos}]
	.cons.fh.thist insert insert $goals($cur)
}

# -----------------------------------------------------------------------------
# Le prochain pad de but
# -----------------------------------------------------------------------------
proc cons:nextgoal {} {
	global cons:listgoals cons:pos

	if {${cons:pos} >= [llength ${cons:listgoals}]-1} {
		bip
		return
		}
	cons:savegoal
	incr cons:pos
	cons:setgoal ${cons:pos}
}

# -----------------------------------------------------------------------------
# Le precedent pad de but
# -----------------------------------------------------------------------------
proc cons:prevgoal {} {
	global cons:pos

	if {${cons:pos} == 0} {
		bip
		return
		}
	cons:savegoal
	incr cons:pos -1
	cons:setgoal ${cons:pos}
}

# -----------------------------------------------------------------------------
# Affiche le pad de but courant
# -----------------------------------------------------------------------------
proc cons:setgoal {pos} {
	global cons:goals cons:listgoals
	upvar #0 cons:goals goals
	set cur [lindex ${cons:listgoals} $pos]
	.cons.fh.thist delete 1.0 end
	.cons.fh.thist insert insert $goals($cur)
}

# -----------------------------------------------------------------------------
# Met a jour un des menus 'Get...' ou 'Exec....'
# -----------------------------------------------------------------------------
proc cons:menugoal {W com} {
	global cons:listgoals cons:pos
	upvar #0 cons:goals goals
	cons:savegoal
	$W delete 0 last
	set i 0
	foreach cur ${cons:listgoals} {
		set txt $goals($cur)
		if {[string length $txt] > 70} {
			set txt "[string range $txt 0 69]..."
			}
		$W add command -label $txt  -command "cons:${com}thegoal $i"
		incr i
		}
#	$W entryconfig ${cons:pos} -backg green -acc 0
	$W entryconfig ${cons:pos} -under 0
}

#
proc cons:getthegoal {n} {
	global cons:pos
	if {${cons:pos} != $n} {
			set cons:pos $n
			cons:setgoal $n
			}
}

#
proc cons:execthegoal {n} {
	global cons:pos cons:echo cons:listgoals conn:current_wout
	upvar #0 cons:goals goals

	set cur [lindex ${cons:listgoals} $n]
	set txt $goals($cur)\n
	if ${cons:echo} {
		tty:text_any_key ${conn:current_wout} $txt
		} \
	else {
		global f1
		conn:envoie $txt $f1
		}
}

# -----------------------------------------------------------------------------
# Sauve les buts dans un fichier
# -----------------------------------------------------------------------------
proc cons:qsave {{file .p4queries}} {
    if [catch "open \"$file\" w" f] return
	pref:printarray $f cons:goals
	puts $f ""
	global cons:pos cons:listgoals cons:1free
	puts $f "set cons:pos ${cons:pos}"
	puts $f "set cons:listgoals [list ${cons:listgoals}]"
	puts $f "set cons:1free ${cons:1free}"
    close $f
}

# -----------------------------------------------------------------------------
# Recupere les buts sauves dans un fichier
# -----------------------------------------------------------------------------
proc cons:qload {{file .p4queries}} {
	global cons:pos
    catch "uplevel #0 source $file"
	cons:setgoal ${cons:pos}
}


# -----------------------------------------------------------------------------
# Initialisation de la console
# -----------------------------------------------------------------------------
proc cons:init {} {
	global cons:fonthistory
	set W .cons
	toplevel $W
	#
	# On empeche l'utilisateur de fermer la console
	#
	wm protocol $W WM_DELETE_WINDOW "# rien"

	wm minsize $W 1 1

	wm title $W [lang: T-pconsole]
#old	wm geo $W +430+27
	wm geo $W -0+0

	#
	# History, en bas
	#
	set fh [frame $W.fh]
		set fb [frame $fh.fb -relief sunken -bd 1]
			button $fb.bn -text [lang: PCON.new-goal] -command {cons:newgoal}
			button $fb.bk -text [lang: PCON.kill-goal] -command {cons:killgoal}
			menubutton $fb.mbgg -text [lang: PCON.mget-goal] -m $fb.mbgg.m \
					-relief raised -pady 5 ; #le defaut (4) n'est pas comme un bouton ord.
				menu $fb.mbgg.m -postcommand "cons:menugoal $fb.mbgg.m get" -tearoff 0
			menubutton $fb.mbxg -text [lang: PCON.mexec-goal] -m $fb.mbxg.m \
					-relief raised -pady 5 ; #le defaut (4) n'est pas comme un bouton ord.
				menu $fb.mbxg.m -postcommand "cons:menugoal $fb.mbxg.m exec" -tearoff 0 
			button $fb.be -text [lang: PCON.exec-goal] -command {cons:envoiehistory}
			checkbutton $fb.cbe -text [lang: PCON.toggle-echo] -variable cons:echo
			button $fb.bpr -text [lang: PCON.prev-goal] -command {cons:prevgoal} -padx 1
			button $fb.bne -text [lang: PCON.next-goal] -command {cons:nextgoal} -padx 1
			pack $fb.bn -side left -pady 0m
			pack $fb.bk -side left -pady 0m
			pack $fb.mbgg -side left -pady 0m
			pack $fb.mbxg -side left -pady 0m
			pack $fb.be -side left -fill x -expand y
			pack $fb.cbe -side left -pady 0m
			pack $fb.bpr -side left -pady 0m
			pack $fb.bne -side left -pady 0m
		text $fh.thist -yscroll "$fh.sb set" -hei 4 -wid 20 \
				-relief sunken -bd 1 \
				-font ${cons:fonthistory}
		scrollbar $fh.sb -orient verti -command "$fh.thist yview" \
				-relief sunken -bd 1 
#				-bg #e3f3f5 -activebackground #002e00
		pack $fh.fb -side top -fill x -expand y
		pack $fh.thist -side left -fill both -expand y
		pack $fh.sb -side left -fill y
	pack $fh -side bottom -fill both

#3	text:Standard_Keys $W.fh.thist
	bind $W.fh.thist <KP_Enter> "cons:envoiehistory ; break"
	bind $W.fh.thist <Control-Return> "cons:envoiehistory ; break"
	bind $W.fh.thist <Control-Key-N> "cons:newgoal"
	bind $W.fh.thist <Control-Key-D> "cons:newgoal dup"
	bind $W.fh.thist <Control-Key-K> "cons:killgoal"
	bind $W.fh.thist <Control-Key-p> "cons:prevgoal"
	bind $W.fh.thist <Control-Key-n> "cons:nextgoal"

	bind $W.fh.thist <Control-Key-x> [bind Text <<Cut>>]
	bind $W.fh.thist <Control-Key-c> [bind Text <<Copy>>]
	bind $W.fh.thist <Control-Key-v> [bind Text <<Paste>>]
	cons:initgoals

	#
	# La zone "tty"
	#
	set ftty [tty:newframe $W]
	pack $ftty -side left -expand yes -fill both

	#
	# Le frame menu (et des boutons du haut)
	#
	set fr $W.fr
	frame $fr -relief raised -bd 1
	pack $fr -before $ftty -side top -fill x

	#
	# TMP:
	# Le bouton Connection et le bouton stop
	#
	global conn:current_wout
	radiobutton $fr.cb -text [lang: PCON.output-here] -variable conn:current_wout \
		-activeback #eed5b7 -activeforegr Black \
		-fg Black -selectcolor green \
		-value $ftty.t
	button $fr.bdebug -text [lang: PCON.debug] -command {conn:debug-on} \
		-bd 1 -padx 5 -pady 0
	button $fr.bstop -text [lang: PCON.stop] -command {conn:ctrl-c} \
		-bd 1 -padx 5 -pady 0
	pack $fr.bstop $fr.bdebug $fr.cb -padx 2m -side right

	#
	# Pour verifier la presence du processus
	#
	checkbutton $fr.cbex -text "" -variable conn:exists -selectcolor green \
		-highlightthickness 0 -padx 0 -pady 0

	set mp [cons:create_process_menu $fr]
	set mp4 [cons:create_p4_menu $fr]
	pack $fr.cbex $mp $mp4 -side left -padx 2m
#3	tk_menuBar $fr $mp $mp4

	#
	# Dimensions de la fenetre
	#
	wm geo $W 640x480
	wm minsize $W 280 200

	update
	global cons:pref
	#
	# On lance l'exe (s'il y a lieu) dans 1s,
	# (donc apres que les init soient faites)
	#
	if [set cons:pref(autolaunch)] {after 1000 "cons:RunProlog"}

#3	#
#3	# ce signal sert a cadencer les messages.
#3	# (acknowledge de prolog)
#3	#
#3	signal trap SIGUSR1 {set ack 1; errputs Ack}

	global tcl_platform
	if {"$tcl_platform(platform)" == "unix"} {
		#
		#4 associe des variables C aux trois signaux INT USR1 USR2
		#
		p4installsignals
		}

	#
	# surveiller le co-processus prolog
	#
	conn:f1exists
}

@com {
_Process:
	============
	_Run					# L'application est lancee
	-----------
	_Kill					# " " est tuee
	-----------
	_Preferences ...		# Pour changer les parametres du process
	============
Prolog IV:
	============
	_Show Stacks				# Montrer les piles
	============
}


# -----------------------------------------------------------------------------
# Le menu 'Process' de la console
# -----------------------------------------------------------------------------
proc cons:create_process_menu {rootfm} {
	upvar #0 font_for_menus f
	set mb $rootfm.mbproc
	set m $mb.mproc
	menubutton $mb -menu $m -font $f \
			-text [lang: PCON.M-process] -underline [lang:u PCON.M-process]

	menu $m
	$m add command -font $f \
			-label [lang: process-run] -underline [lang:u process-run] \
			-command {cons:RunProlog}
	$m add sepa
	$m add command -font $f \
			-label [lang: process-kill] -underline [lang:u process-kill] \
			-command {cons:killprolog}
	$m add sepa
	$m add command -font $f \
			-label [lang: process-prefs] -underline [lang:u process-prefs] \
			-command {cons:preferences}
	return $mb
}

proc cons:preferences {} {
	pref:setpreference Prolog
}


# -----------------------------------------------------------------------------
# Le menu 'Prolog IV' de la console
# -----------------------------------------------------------------------------
proc cons:create_p4_menu {rootfm} {
	upvar #0 font_for_menus f
	set mb $rootfm.mbp4
	set m $mb.m
	menubutton $mb -menu $m -font $f \
			-text [lang: PCON.M-prologiv] -underline [lang:u PCON.M-prologiv]
	menu $m
	$m add command -font $f \
			-label [lang: process-showstacks] -underline [lang:u process-showstacks] \
			-command {stack:init}
	return $mb
}


proc cons:RunProlog {} {
	cons:restart
	cons:listener
}

# TMP A JETER
#if $forprolog4 {
#	set for_tcl "\x1btcl_"
#	set quotedfor_tcl "\\x1b\\tcl_proc"
#	} \
#else {
#	# PAS A JOUR
#	set for_tcl "\x1btcl_"
#	set quotedfor_tcl "\\x1btcl_proc"
#	}

# OLD:
#proc tcl_proc {messid args} {
##errputs "$args"
#	uplevel #0 eval $args
#}

# LA NOUVELLE !
# Il manque encore le moyen d'indiquer a prolog qu'une erreur
# s'est produite.
proc tcl_proc {messid args} {
#errputs "$args"
	catch {uplevel #0  $args}
}

# OLD
#proc tcl_fun {messid args} {
#	global f1
##errputs <fun-"$args">
#	catch "uplevel #0 eval $args" u
#	conn:repondre $u $messid $f1
#}

global conn:lastfunmessid
set conn:lastfunmessid -1

# LA NOUVELLE !
# elle retourne une erreur a Prolog le cas echeant.
proc tcl_fun {messid args} {
	global f1 conn:lastfunmessid
	set conn:lastfunmessid $messid
#errputs <fun-"$args">
	if {[catch {uplevel #0  $args} u] == 1} {
		set u "tcl_error([conn:to_prolog_atom $u]). "
		}
	conn:repondre $u $messid $f1
}

#
# convertit une chaine en atome prolog (quote)
#
proc conn:to_prolog_atom {a} {
	regsub -all "'" $a  {''} a
	regsub -all {\\} $a {\\\\} a
	regsub -all "\n" $a {\\n} a
	return '$a'
}



#proc oldtcl_fun args {
##errputs <fun-"$args">
#	catch "uplevel #0 eval $args" u
#	envoie $u\n
#}

#proc receive {f} {
#	while 1 {
#		set s [conn:rece $f]
#		if {"$s" != ""} {return $s}
#		}
#}

##
## Ces deux variables servent a cadencer la frequence des appels a update
## dans 'obtient' a des valeurs raisonnables pour l'interactivite.
## Le retour n'est pas utilise
##
#set nextupdate [getclock]
#set delta 1

#
# Quand elle est appelee, du texte est disponible
#
#proc obtient {W f} {
#	global errorCode errorInfo
#	global for_tcl
#	global nextupdate delta
#	global pro_answer
#	set errorCode ""
#	set errorInfo ""
#
#	puts "************** APPEL OBSOLETE A 'obtient' **************"
#
#	if {! [catch "conn:rece $f" ligne] } {
##errputs -nonewline <<<<<$ligne>>>>>
#
#		while {"$ligne" != "" } {
#			set last [string len $ligne]
#			incr last -1
#			if {-1 == [set s [string first $for_tcl $ligne]]} {
#				if {-1 == [set s [string last \033 $ligne]]} {
#					tty:insere_style out $W $ligne ""
#					set ligne ""
#					} \
#				else {
#					if {$s != 0} {
#						tty:insere_style out $W [string range $ligne 0 [expr $s-1]] ""
#						set ligne [string range $ligne $s $last]
#						}
#					if {$s >= [expr $last-3]} {
##						errputs "esc-char seen at end of gets (waiting more)"
#						set u [receive $f]
#						append ligne $u
#						errputs "on ajoute ($ligne)"
#						} \
#					else {
#						tty:insere_style out $W $ligne ""
#						set ligne ""
#						}
#					}
#				} \
#			else {
#				# ya quelque chose pour tcl
#				if {$s != 0} {
#					# ya du texte avant
#					tty:insere_style out $W [string range $ligne 0 [expr $s -1]] ""
#					}
#				# +1 : on saute ESC
#				set ligne [string range $ligne [expr $s +1] end]
#				# on cherche \n
#				if {-1 != [set s2 [string first \n $ligne]]} {
#					incr s2 -1
#					set cmd [string range $ligne 0 $s2]
#					if {[string range $cmd 0 12] == "tcl_proanswer"} {
#						set pro_answer [string range $cmd 14 end] 
#						return ANSWER
#						}
#					if { [catch "$cmd"] } {
#						errputs \{"erreur dans obtient pour ($cmd)"\}
#						errputs "<$errorInfo>"
#						}
#					incr s2 2
#					set ligne [string range $ligne $s2 end]
#					} \
#				else {
#					# il faut agglutiner
#					# TMP
#					errputs \{"commande non terminee par \\n ! (on attend)"\}
#					set cmd $ligne
#					set ligne [receive $f]
##errputs -nonewline <<<<$ligne>>>>
#					while { -1 == [set s2 [string first \n $ligne]] } {
#						append cmd $ligne
#						set ligne [receive $f]
##errputs -nonewline <<<$ligne>>>
#						}
#					incr s2 -1
#					append cmd [string range $ligne 0 $s2]
#					if {[string range $cmd 0 12] == "tcl_proanswer"} {
#						set pro_answer [string range $cmd 14 end] 
#						return ANSWER
#						}
#					if { [catch "$cmd"] } {
#						errputs \{"erreur dans obtient pour ($cmd)"\}
#						errputs "<$errorInfo>"
#						}
#					incr s2 2
#					set ligne [string range $ligne $s2 end]
#					}
#				}
#
#			if {$nextupdate <= [getclock]} {
#				update
#				set nextupdate [expr [getclock]+$delta]
#				}
#			if [eof $f] {
#				close $f
#				return -code error -errorinfo QUITTER 
#				}
#			}
#		}
#
## est-ce que le fichier (process) existe encore ?
#	if {[catch "eof $f"]} {
#		return -code error -errorinfo NOTOPEN 
#		}  \
#	elseif {[eof $f]} {
#		close $f
#		return -code error -errorinfo QUITTER 
#		}
#		
#	return --NOANSWER--
#}


#
# Pour l'affichage:
# 'select' est sense ne pas bloquer plus de 0.1s
# on ne bloque que sur les entrees (sorties de prolog)
#
#global cons:selectval
#set cons:selectval 0.1
#proc cons:waitio {W f} {
#	global cons:selectval
#	set r [select $f {} {} ${cons:selectval}] 
#	if {! [lempty [lindex $r 0]] } {
#		#read: qqch est disponible pour l'affichage
#		obtient $W $f
#		} else {
#		#read: rien n'est disponible
#		}
#	# Pour la saisie
#	if { [lempty [lindex $r 1]] } {
#		#write: tout n'est pas consomme
#		} else {
#		#write: tout est consomme (on peut avancer iomark)
#		}
#}

#
# La boucle est lancee du bouton "Connection" de la console
#3
#proc cons:g {} {
#	global thestop f1
#	while {$thestop} {
#		update
#		if [set r [catch {cons:waitio .cons.frtty.t $f1} res]] {
#			set thestop 0
#			}
#		}
#	if $r {puts $res}
#	puts "La machine Prolog est arretee. ($r)"
#}


#
# Donne un temps systeme en ms (tire des clicks de clock)
# NON PORTABLE :
#     (1000000 par seconde sur unix))
#     (1000 par seconde sur pc))

# PBO 21/10/98 Change "machine" par "platform" et "sun4m" par "unix"
if {$tcl_platform(platform) == "unix"} {
	proc clockclicksms {} {
		set u [clock clicks]
		set l [string len $u]
		return [string range $u 0 [expr $l-4]]
		#return [expr {int([clock clicks]/1000)}]
		}
} else {
	proc clockclicksms {} {
		return [clock clicks]
		}
	}

#
# Ces deux variables servent a cadencer la frequence des appels a update
# dans 'domessage' a des valeurs raisonnables pour l'interactivite.
# Le retour n'est pas utilise
#
set nextupdate [clockclicksms]
set delta 500

global indomessage
set indomessage 0

proc cons:domessage {{update 1}} {
	global f1 tab_answer errorInfo errorCode conn:current_wout
	global nextupdate delta
global indomessage
set indomessage 1
	errputs "--------u($update)--"
	set com [conn:rece $f1]
	errputs <<$com>>

	while 1 {
		if {"$com" == ""} break

		if {"[string range $com 0 5]" == "texte "} {
			tty:insere_style out ${conn:current_wout} [string range $com 6 end] ""
			} \
		elseif {"[string range $com 0 13]" == "tcl_proanswer "} {
			set answer [string range $com 14 end] 
			regexp -- {tcl_proanswer ([0-9]+) ([0-9]+) (.*)$} $com _ messid replyid mess
			set tab_answer($replyid) $mess
set indomessage 0
			return $replyid
			} \
		else {
			if { [catch "$com"] } {
			errputs "\{erreur dans domessage pour ($com)\}"
			errputs "<$errorInfo>"
			}
			}
		if {$nextupdate <= [clockclicksms]} {
			errputs "--update ($update)--"
			if $update update else {update idletask}
			set nextupdate [expr [clockclicksms]+$delta]
			}

		set com [conn:rece $f1]
		errputs <<$com>>
		}

	if {$nextupdate <= [clockclicksms]} {
		errputs "--updatend ($update)--"
		if $update update else {update idletask}
		set nextupdate [expr [clockclicksms]+$delta]
		}
set indomessage 0
	return ""
	}


proc cons:domessage_notcl {} {
	global f1 errorInfo errorCode conn:current_wout
	global nextupdate delta
	set com [conn:rece_notcl $f1]
	errputs <<$com>>

	set buftext {}

	while 1 {
		if {"$com" == ""} break

		append buftext $com
		if {$nextupdate <= [clockclicksms]} {
			tty:insere_style out ${conn:current_wout} $buftext ""
			set buftext {}
			update
			set nextupdate [expr [clockclicksms]+$delta]
			}

		set com [conn:rece_notcl $f1]
		errputs <<$com>>
		}

	if { "$buftext" != "" } {
		tty:insere_style out ${conn:current_wout} $buftext ""
		set nextupdate [expr [clockclicksms]+$delta]
		}
#	update idletask
	return ""
	}

#

proc conn:getanswer {replyid} {
	global tab_answer
    set answer $tab_answer($replyid)
	unset tab_answer($replyid)
	return $answer
	}

global conn:ms
set conn:ms 200
set conn:waitmess 0
#
#
#
proc cons:listener {} {
	global conn:ms
global indomessage
	if {$indomessage == 0} {
		upvar #0 cons:pref(usestcl) usestcl
		if $usestcl cons:domessage else cons:domessage_notcl
		}
	after ${conn:ms} cons:listener
	}

# ----------------------------------------------------------------------------
# Handler pour verifier periodiquement l'existence du processus.
# On se sert ici de eof, qui ne marche pas sur PC !!!
# (faudra trouver autre chose : pid ne detecte pas sous unix)
# ----------------------------------------------------------------------------
proc conn:f1exists {} {
	global f1 conn:exists
	if {[catch {eof $f1}] || [eof $f1]} {
		set conn:exists 0
		} \
	else {
		set conn:exists 1
		}
	after 100 conn:f1exists
	}

# a jeter
#proc cons:g {} {
#	global f1 conn:current_wout
#	set conn:current_wout .cons.frtty.t
#	cons:listener
#	fileevent $f1 r test
#	}

#-- EOF newcons.tcl -----------------------------------------------------------
