# -- perf.tcl --
# montrer les piles de Prolog IV
#
# (c) PrologIA 1997
# ------------------------------------------------------------------------------

# des identifications de piles de Prolog IV
global stack:Ls
global stack:Lsn

#
#
#
proc stack:init {} {
	global stack:Ls stack:Lsn
	upvar #0 font_for_menus f
	set W .stack
	if [winfo exists $W] {
		wm deiconify $W
		return
		}

	toplevel $W
	wm protocol $W WM_DELETE_WINDOW "stack:off ; destroy $W"
	wm title $W [lang: T-stacks]
	set stack:Ls {Heap Env Choice Trail Global}; # (Work) Dict
	set stack:Lsn [list \
		[lang: heap-stack] \
		[lang: env-stack] \
		[lang: choice-stack] \
		[lang: trail-stack] \
		[lang: globalheap-stack] ]; # (Work) Dict

	foreach s ${stack:Ls} sn ${stack:Lsn} {
		set f$s [stack:make_pad $W $s $sn]
		global stack:x$s
		set stack:x$s 1
		global stack:$s:max
		set stack:$s:max 0
		}

	set fm [frame $W.fm -relief raised -bd 1]
		set mb $fm.mb
		set m $mb.m
		menubutton $mb -menu $m -text [lang: STACK.M-stacks] \
			-underline [lang:u STACK.M-stacks] -font $f
		menu $m
		foreach s ${stack:Ls} sn ${stack:Lsn} {
			$m add checkbutton -label $sn -selectcolor green \
				-command [list stack:showhide [set f$s] $s] \
				-variable stack:x$s -font $f
			}
		button $fm.bon -text "On" -command {stack:on} -bd 1 -padx 5 -pady 0
		button $fm.boff -text "Off" -command {stack:off} -bd 1 -padx 5 -pady 0
		set mbs [stack:mkupdatemenu $fm]
		label $fm.ls1 -text [lang: L-Updateevery] -padx 5 -font $f
		label $fm.ls2 -text [lang: L-sec.] -padx 5 -font $f
		pack $mb -side left
		pack $fm.boff $fm.bon $fm.ls2 $mbs $fm.ls1 -side right
	pack $fm -side top -fill x
		
	pack $fHeap $fEnv $fChoice $fTrail $fGlobal -side top -pady 1m
}

#
proc stack:on {} {
	after cancel {stack:update_all .stack}
	stack:after
}
#
proc stack:off {} {
	after cancel {stack:update_all .stack}
}

#
proc stack:showhide {Ws s} {
	global stack:x$s
	if [set stack:x$s] {
		pack $Ws -side top -pady 1m
		} \
	else {
		pack forget $Ws
		}
}

#
#  (menu pour choisir le temps en secondes)
#
proc stack:mkupdatemenu {root} {
	set mb $root.mbsec
	menubutton $mb -menu $mb.m -textvariable stack:delay \
		 -bd 1 -padx 1 -pady 1 -relief raised
	set m [menu $mb.m -tearoff 0]
	foreach s {0.5 1 2 3 4 5 10} {
		$m add radiobutton -label $s -variable stack:delay -indicator 0
		}
	return $mb
}


#
#
#
proc stack:make_pad {root st stn} {
	set fr [frame $root.f_$st]
	canvas $fr.c -backg black -width 300 -heig 101 -highlightthickness 0
	label $fr.lsn -text $stn -width 15
	label $fr.lv -fg red
	label $fr.lm -fg blue
	pack $fr.c -side left
	pack $fr.lsn $fr.lv $fr.lm -side top
	stack:mkgridlog $fr.c	
	return $fr
}

#
#
#
proc stack:mkgrid10 {C} {
	for {set i -10} {$i < 310} {incr i 10} {
		# lignes verticales
		$C create line $i 0 $i 100 -fill darkgreen -tag {grille}
		}
	for {set i 10} {$i < 100} {incr i 10} {
		# lignes horizontales
		$C create line -10 $i 310 $i  -fill darkgreen -tag {grille}
		}
	$C create line -1 100 301 100 -tag {max} -fill blue
}

proc stack:mkgridlog {C} {
	for {set i -10} {$i < 310} {incr i 10} {
		# lignes verticales
		$C create line $i 0 $i 100 -fill darkgreen -tag {grille}
		}
	for {set i 2} {$i < 10} {incr i 2} {
		# lignes horizontales
		set v [stack:formulelog $i]
		set v [expr 100 -$v -1]
		$C create line -10 $v 310 $v  -fill darkgreen -tag {grille}
		}
	for {set i 20} {$i < 100} {incr i 20} {
		# lignes horizontales
		set v [stack:formulelog $i]
		set v [expr 100 -$v -1]
		$C create line -10 $v 310 $v  -fill darkgreen -tag {grille}
		}
	$C create line -1 100 301 100 -tag {max} -fill blue
}



# Mise a jour des afficheurs toutes les stack:delay secondes :
global stack:delay
set stack:delay 1
global stack:phase
set stack:phase 0

#
proc stack:test {W} {
	for {set i 0} {$i < 30} {incr i} {
		after 500
		set n [expr (($i % 10) == 9) * 10 -1]
		$W.c move grille $n 0
		update
		}
}

#
#
#
proc stack:after {{W .stack}} {
	global stack:delay
	after [expr int(1000 * ${stack:delay})] stack:update_all $W
}

#
#
#
proc stack:formulelog10 {Value} {
	if {$Value == 0} {return 0}
	return [expr pow(2, log10($Value)) * 25]
}
#
# loga X = log10 X / log10 a
#
proc stack:formulelog {Value} {
	if {$Value == 0} {return 0}
	return [expr pow(2, log10($Value)/log10(5)) * (100/pow(2, 2/log10(5)))]
}
#
proc stack:formule10 {Value} {
	return $Value
}

#
#
#
proc stack:update_all {root} {
	global stack:phase f1
	if {[winfo exist $root] == 0} return
	set reply [conn:pro_fun getheapsvalues "" $f1]
	if $reply {
		set answer [conn:getanswer $reply]
		global stack:Ls
		foreach s ${stack:Ls} {
			stack:update $root.f_$s $s $answer
			}
#		foreach sl [pack slaves $root] {
#			if {[regexp {f_(.*)} [winfo name $sl] _ stname] == 1} {
#				stack:update $sl $stname $answer
#				}
#			}
		}
	update
	incr stack:phase
	stack:after $root
}

#
# getheapsvalues retourne:
# bH H mH
# bE E ESP mE
# bB B BSP mB
# bT TR mT
# bW W mW
# bG G mG
#
proc stack:update {W stname list} {
	global stack:phase
	upvar #0 stack:$stname:max max
	set can $W.c
	set n [expr ((${stack:phase} % 10) == 9) * 10 -1]
	$can move grille $n 0
	catch {$can move graph -1 0}
	#
	# on enleve ce qui sort a gauche
	#
	eval $can delete [$can find enclosed -100 -2 -20 102]
	set value [stack:get$stname $list]
	set svalue [format "%4.2f %%" $value]
	$W.lv config -text $svalue
	set value [stack:formulelog $value]
	set top [expr 100-$value-1]
	$can create line 297 $top 297 100 -tag {graph} -fill red
	if {$value > $max} {
		set max $value
		$can coord max -1 $top 301 $top
		$W.lm config -text "$svalue (max)"
		}

}

#
# getheapsvalues retourne:
# bH H mH
# bE E ESP mE
# bB B BSP mB
# bT TR mT
# bW W mW
# bG G mG
#

#
proc stack:getHeap {L} {
	set base [lindex $L 0]
	set H [lindex $L 1]
	set max [lindex $L 2]
	return [expr (double($H-$base))/($max - $base)*100]
}
#
proc stack:getEnv {L} {
	set base [lindex $L 3]
	set E [lindex $L 4]
#	set ESP [lindex $L 5]
	set max [lindex $L 6]
	return [expr (double($E-$base))/($max - $base)*100]
}
#
proc stack:getChoice {L} {
	set base [lindex $L 7]
	set B [lindex $L 8]
#	set BSP [lindex $L 9]
	set max [lindex $L 10]
	return [expr (double($B-$base))/($max - $base)*100]
}
#
proc stack:getTrail {L} {
	set base [lindex $L 11]
	set T [lindex $L 12]
	set max [lindex $L 13]
	return [expr (double($T-$base))/($max - $base)*100]
}
#
proc stack:getWork {L} {
	set base [lindex $L 14]
	set W [lindex $L 15]
	set max [lindex $L 16]
	return [expr (double($W-$base))/($max - $base)*100]
}
#
proc stack:getGlobal {L} {
	set base [lindex $L 17]
	set G [lindex $L 18]
	set max [lindex $L 19]
	return [expr (double($G-$base))/($max - $base)*100]
}



# getheapsvalues retourne:
# bH H mH
# bE E ESP mE
# bB B BSP mB
# bT TR mT
# bW W mW
# bG G mG

# -- EOF perf.tcl -------------------------------------------------------------
