# -- pref.tcl
# Les preferences de l'environnement
#
# Auteur:		Pascal Bouvier
# Creation:		09/10/95
# Modifications:	
#				PBO 16/10/95 .p4prefs sauve dans le home-directory
#				PBO 05/02/96 autolaunch de l'exe
#				PBO 01/12/96 - p4prefs.tcl sur pc
#							 - le lit d'abord dans le kit avant HOME puis pwd
#				PBO 10/01/97 - les prefs "Misc" (emacs-like, 2buttons)
#
# (c) PrologIA 1995-2000
# -----------------------------------------------------------------------------


# ==== G L O B A L E S =======================================

global tcl_platform

#
# Le type de preferences
# {Text-Attributes Prolog}
set pref:theme Text-Attributes

#
# le nom du fichier de preferences selon unix ou autre
#
global pref:filename

if {$tcl_platform(platform) == "unix"} {
	set pref:filename ".p4prefs"
} else {
	set pref:filename "p4prefs.tcl"
	}

# ==== P R O C S =======================================

# -----------------------------------------------------------------------------
# Cree le dialogue (on ne le cree qu'une seule fois)
# -----------------------------------------------------------------------------
proc pref:init {} {
	set w .prefs
	if [winfo exists $w] {
		wm deiconify $w
		raise $w
		return
		}
	toplevel $w
	pref:reinit $w
}

#
#
#
proc pref:reinit {w} {
	wm title $w [lang: T-preferences]
#	global pref:theme

	set f [frame $w.ftopic]
		label $f.l -text [lang: PRF.topic]
		set mbm $f.mbm
		set m $mbm.m
		menubutton $mbm -menu $m -relief raised -bd 1 -textvar pref:theme \
				-indicatoron 1
		menu $m
		$m add radiobutton -label Language \
				-variable pref:theme -indicatoron 0 \
				-command "pref:installframein $w.f"
		$m add radiobutton -label Text-Attributes \
				-variable pref:theme -indicatoron 0 \
				-command "pref:installframein $w.f"
		$m add radiobutton -label Prolog \
				-variable pref:theme -indicatoron 0 \
				-command "pref:installframein $w.f"
		$m add radiobutton -label Misc \
				-variable pref:theme -indicatoron 0 \
				-command "pref:installframein $w.f"
		pack $f.l $f.mbm -side left
	pack $f -side top -pady 2m -padx 2m -ipadx 5 -ipady 2
	
	#
	# le frame a inserer
	#
	frame $w.f -relief groove -bd 2
	pref:installframein $w.f
	pack $w.f -side top -pady 2m -padx 1m -expand y -fill both

	#
	# Les boutons
	#
	set fbut [frame $w.fbut]
		set bsetd [button $fbut.bsetd -text [lang: PRF.set-default] \
				-command pref:setdefault]
		set bshowd [button $fbut.bshowd -text [lang: PRF.show-default] \
				-command pref:showdefault]
		set bclose [button $fbut.bclose -text [lang: PRF.close] \
				-command pref:close]
		pack $bsetd $bshowd $bclose -side left -padx 1m -pady 2m -ipadx 1m \
				-fill x -expand 1
					
	pack $fbut -side top -pady 2m -expand y -fill both

}

# -----------------------------------------------------------------------------
# Ferme la fenetre des preferences
# -----------------------------------------------------------------------------
proc pref:close {} {
	catch "destroy .prefs"
}

# -----------------------------------------------------------------------------
# Installe une frame "flottante"
# -----------------------------------------------------------------------------
proc pref:installframein {root} {
	global pref:theme
	set com pref:${pref:theme}
	if [catch "info proc $com"] return
	#
	# Sinon c'est un nouveau
	#
	foreach ch [winfo child $root] {
		destroy $ch
		}
	return [$com $root]
}

# -----------------------------------------------------------------------------
# Installe le dialogue, avec le topic donne
# Le topic doit etre bon (pas de test)
# -----------------------------------------------------------------------------
proc pref:setpreference {topic} {
	global pref:theme
	set w .prefs

	set otheme ${pref:theme}
	set pref:theme $topic

	if [winfo exists $w] {
		if {$otheme != $topic} {
			pref:installframein $w.f
			}
		wm deiconify $w
		raise $w
		} \
	else {
		pref:init
		}
}

# -----------------------------------------------------------------------------
# Affecte definitivement l'attribut
# -----------------------------------------------------------------------------
proc pref:setdefault {} {
	global pref:theme
	set com pref:${pref:theme}:setdefault
	return [$com]
}

# -----------------------------------------------------------------------------
# Montre l'attribut (perd l'ancien essai)
# -----------------------------------------------------------------------------
proc pref:showdefault {} {
	global pref:theme
	set com pref:${pref:theme}:showdefault
	return [$com]
}

# -----------------------------------------------------------------------------
# Sauve les preferences dans le HOME directory
# TMP:
# pas d'erreur si ne peut le creer
# -----------------------------------------------------------------------------
proc pref:save {} {
	global pref:filename
	if [catch "open \"~/${pref:filename}\" w" f] return

	foreach arr [uplevel #0 info vars *:pref] {
		pref:printarray $f $arr
		puts $f ""
		}
	close $f
}

# -----------------------------------------------------------------------------
# Charge les preferences presentes dans le home-dir puis dans le dir courant
# TMP:
# pas d'erreur si le trouve pas
# -----------------------------------------------------------------------------
proc pref:load {} {
	global chemin pref:filename
	uplevel #0 catch [list [list source $chemin/${pref:filename}]]
	uplevel #0 catch [list [list source ~/${pref:filename}]]
	uplevel #0 catch [list [list source [pwd]/${pref:filename}]]
#	catch {uplevel #0 source $chemin/${pref:filename}}
#	catch {uplevel #0 source ~/${pref:filename}}
#	catch {uplevel #0 source [pwd]/${pref:filename}}
}

# -----------------------------------------------------------------------------
# TMP:
# Ecrit un tableau tcl, dans un fichier, pour relecture ulterieure
# -----------------------------------------------------------------------------
proc pref:printarray {f arr} {
	upvar #0 $arr tab
	foreach idx [array names tab] {
		puts $f "set $arr\($idx\) [list $tab($idx)]"
		}
	}

#==============================================================================

# -----------------------------------------------------------------------------
# Le frame pour les preferences Text (editors)
# -----------------------------------------------------------------------------
proc pref:Text-Attributes {rootf} {
	upvar #0 font_for_menus fnt
	arraycopy edit:tmp edit:pref

	#
	# La fonte
	#
	set ffont [frame $rootf.ffont]
		label $ffont.l -text [lang: PRF.font]
		menubutton $ffont.mb -textvar edit:tmp(font) \
				-font $fnt -relief raised -bd 1 -indicatoron 1
		$ffont.mb config -menu [font:makemenu $ffont.mb]
		pack $ffont.l $ffont.mb -side left
	#
	# Le papier (Background)
	#
	set fbgcol [frame $rootf.fbgcol]
		label $fbgcol.l -text [lang: PRF.bkcolor]
		entry $fbgcol.e -relief sunken -bd 2 -textvar edit:tmp(bg)
		pack $fbgcol.l $fbgcol.e -side left
	#
	# L'encre (Foreground)
	#
	set ffgcol [frame $rootf.ffgcol]
		label $ffgcol.l -text [lang: PRF.fgcolor]
		entry $ffgcol.e -relief sunken -bd 2 -textvar edit:tmp(fg)
		pack $ffgcol.l $ffgcol.e -side left
	#
	# Le langage (de programmation)
	#
	set flang [frame $rootf.flang]
		label $flang.l -text [lang: PRF.prog-language]
		menubutton $flang.mb -textvar edit:tmp(lang) \
				-font $fnt -relief raised -bd 1 -indicatoron 1
		$flang.mb config -menu [ed:makelangmenu $flang.mb]
		pack $flang.l $flang.mb -side left

	pack $ffont $fbgcol $ffgcol $flang -side top \
			-pady 2m -padx 2m -ipady 1m -ipadx 1m
}

proc pref:Text-Attributes:setdefault {} {
	arraycopy edit:pref edit:tmp
}

proc pref:Text-Attributes:showdefault {} {
	arraycopy edit:tmp edit:pref
}

#==============================================================================

# -----------------------------------------------------------------------------
# Le frame pour les preferences Prolog IV (console)
# -----------------------------------------------------------------------------
proc pref:Prolog {rootf} {
	upvar #0 font_for_menus fnt
	arraycopy cons:tmp cons:pref

	#
	# Le chemin complet de l'executable
	#
	set fproexe [frame $rootf.fproexe]
		label $fproexe.l -text [lang: PRF.exec-name]
		entry $fproexe.e -relief sunken -bd 2 -textvar cons:tmp(exe)
		pack $fproexe.l -side left
		pack $fproexe.e -side left -fill x -expand 1
	#
	# Les options de Prolog IV
	#
	set fproargv [frame $rootf.fproargv]
		label $fproargv.l -text [lang: PRF.exec-options]
		entry $fproargv.e -relief sunken -bd 2 -textvar cons:tmp(argv)
		pack $fproargv.l -side left
		pack $fproargv.e -side left -fill x -expand 1
	#
	# Les user_options de Prolog IV
	#
	set fprouargv [frame $rootf.fprouargv]
		label $fprouargv.l -text [lang: PRF.user-exec-options]
		entry $fprouargv.e -relief sunken -bd 2 -textvar cons:tmp(userargv)
		pack $fprouargv.l -side left
		pack $fprouargv.e -side left -fill x -expand 1
	#
	# Auto-launch at start-up
	#
	set cbautolaunch [checkbutton $rootf.cbauto \
		-text [lang: PRF.auto-exec] -variable cons:tmp(autolaunch)]

	pack $fproexe $fproargv $fprouargv -fill x -expand 1 \
			-side top -pady 2m -padx 2m -ipady 1m -ipadx 1m
	#
	# Uses Tcl
	#
	set cbusestcl [checkbutton $rootf.cbtcl \
		-text [lang: PRF.pro-tcl-flag] -variable cons:tmp(usestcl)]

	pack $cbautolaunch -side left -pady 2m -padx 2m
	pack $cbusestcl -side right -pady 2m -padx 2m
}

proc pref:Prolog:setdefault {} {
	arraycopy cons:pref cons:tmp
}

proc pref:Prolog:showdefault {} {
	arraycopy cons:tmp cons:pref
}

#==============================================================================

# -----------------------------------------------------------------------------
# Le frame pour les preferences du langage
# -----------------------------------------------------------------------------
proc pref:Language {rootf} {
	upvar #0 font_for_menus fnt
	arraycopy super:tmp super:pref

	#
	# Le langage (ecriture de l'utilisateur)
	#
	set flang [frame $rootf.flanguage]
		label $flang.l -text [lang: PRF.user-language]

		menubutton $flang.mb -textvar super:tmp_lang \
				-font $fnt -relief raised -bd 1 -indicatoron 1
		$flang.mb config -menu [super:makeuserlangmenu $flang.mb]
		pack $flang.l $flang.mb -side left

	pack $flang -side top \
			-pady 2m -padx 2m -ipady 1m -ipadx 1m
}

proc pref:Language:setdefault {} {
	arraycopy super:pref super:tmp
	super:load_language_strings
	set w .prefs
#	pref:installframein $w.f
	foreach sl [pack slaves $w] {
		destroy $sl
		}
	pref:reinit $w
}

proc pref:Language:showdefault {} {
	arraycopy super:tmp super:pref
}

#==============================================================================

# -----------------------------------------------------------------------------
# Le frame pour les preferences misc (emacs-mode, mouse buttons (2/3) )
# -----------------------------------------------------------------------------
proc pref:Misc {rootf} {
	upvar #0 font_for_menus fnt
	arraycopy misc:tmp misc:pref

	#
	# emacs binding
	#
	set cbemacs [checkbutton $rootf.cbemacs \
		-text [lang: PRF.emacs-binding] -variable misc:tmp(emacslike)]
	#
	# Souris a deux ou trois boutons
	#
	set cb2buttons [checkbutton $rootf.cb2buttons -state disabled \
		-text [lang: PRF.mouse-has-two-buttons] -variable misc:tmp(2buttons)]

	pack $cbemacs -side top -pady 2m -padx 2m
	pack $cb2buttons -side top -pady 2m -padx 2m
}

proc pref:Misc:setdefault {} {
	arraycopy misc:pref misc:tmp
}

proc pref:Misc:showdefault {} {
	arraycopy misc:tmp misc:pref
}

#-- EOF pref.tcl --------------------------------------------------------------
