# -- procolor.tcl
# La coloration syntaxique dynamique pour Prolog IV
#
# Auteur:		Pascal Bouvier
# Creation:		26/09/95
# Modifications:	
#				PB Reorganisation generale
#
# (c) PrologIA 1995
# -----------------------------------------------------------------------------


# ==== G L O B A L E S =======================================

#
# Une variable pour debugger (les sorties vont dans stdout)
#
global ed:dyncolpro:coldebug
set ed:dyncolpro:coldebug 0

#
# Le fichier pipe de transmission des donnees
#
global ed:dyncolpro:f2

# ==== P R O C S =======================================

# -----------------------------------------------------------------------------
# Lance le co-process de generation d'ordre syntaxique (parse)
# - proc ed:dyncol:Prolog:init {}
# -----------------------------------------------------------------------------
# (Re)colorie ce qu'il y a autour d'insert dans le texte t
# C'est ce qui est lance par les actions
# - proc ed:dyncol:Prolog:colorie {t}
# -----------------------------------------------------------------------------

# ==== I M P L T =======================================

# -----------------------------------------------------------------------------
# Lance le co-process de generation d'ordre syntaxique (parse)
# -----------------------------------------------------------------------------
proc ed:dyncol:Prolog:init {} {
	global BINP4 tcl_platform
	if {$tcl_platform(platform) == "macintosh"} return
	upvar #0 ed:dyncolpro:f2 f2
	catch {process:kill [pid $f2]}
	set f2 [open |[list $BINP4/newparse -coltok -r 2 -ctrlc] {RDWR}]
	process:record [pid $f2]
	# TMP:
	# on peut sans doute bufferiser par ligne (ou par block si on flush)
	fconfigure $f2 -blocking 0 -buffering none
#3	fcntl $f2 NONBLOCK 1
}

# -----------------------------------------------------------------------------
# Retourne une ligne de texte et avance la marque locale coltok
# En fin, se met sur end et rend "\1\n"
# -----------------------------------------------------------------------------
proc ed:dyncolpro:autreligne {w} {
	upvar #0 ed:dyncolpro:coldebug coldebug
	if {[$w compare coltok < end-1c]} {
		set ligne "[$w get coltok {coltok lineend}]\n"
		set cur [$w index coltok]
		$w mark set coltok [$w index {coltok +1l linestart}]
		if {$cur == [$w index coltok]} {
			$w mark set coltok end-1c
			}
		if $coldebug {puts -nonewline "\"$ligne\""}
		return $ligne
		} \
	else {
		if $coldebug {puts -nonewline "\"EOF\n\""}
		return "\1\n"
		}
}


# -----------------------------------------------------------------------------
# Rend la position AVANT le debut du token, afin que nextrange
# puisse fonctionner.
# On trouve forcement le debut du token
# A VOIR le cas du debut de fichier, position 1.0
# -----------------------------------------------------------------------------
proc ed:dyncolpro:start {w i tag} {
	set R [$w tag prevrange $tag $i]
#	puts "## $R t: $tag  i:$i"
	if {"$R" == ""} { return $i }
	set fin [lindex $R 1]
	if {[$w compare $fin < $i]} {
		return $i
		} \
	else {
		return [lindex $R 0]
		}

#3
#3 vieux:
#3
	if {$tag == "LCOMMENT"} {
		set j [$w index "$i-20l linestart"]
		while 1 {
			set R [$w tag nextrange $tag $j]
			if {$R == ""} {
				set j [$w index "$j-20l linestart"]
				continue
				}
			set deb [lindex $R 0]
			set fin [lindex $R 1]
			if {[$w compare $i < $deb]} {
				set j [$w index "$j-20l linestart"]
				} \
			elseif {[$w compare $fin < $i]} {
				set j $fin
				} \
			else {
				return $deb
				}
			}
		puts stderr "% EXCEPTION IN proc start"
		}
	#
	# les autres tokens: on recule d'un car a la fois
	#
	while {[$w compare 1.0 < $i]} {
		if {[lsearch [$w tag names $i] $tag] == -1} break
		set i [$w index $i-1c]
		}
	return $i
}

# -----------------------------------------------------------------------------
# Retourne la prochaine region 'tag' apres l'endroit 'i'
# Au passage, ca souligne les tokens voisins de l'endroit altere
# Utilisee par getrange
# -----------------------------------------------------------------------------
proc ed:dyncolpro:selfin {w i tag} {
	upvar #0 ed:dyncolpro:coldebug coldebug
	set R [$w tag nextrange $tag $i]
	if $coldebug {puts ($R)}
#	dou $w $R
	return $R
}

#
# Pour (de)souligner le texte w
#
proc dou {w R} { eval $w tag add UNDER $R }
proc u {w} { $w tag remove UNDER 1.0 end-1c }


proc eda:toktag {L} {
	set R {}
	set n [llength $L]
	for {set i 0} {$i < $n} {incr i} { 
		set t [lindex $L $i]
		switch -exact -- $t {
			DELIM -
			ATOM -
			INT -
			QATOM -
			LCOMMENT -
			DQUOTE -
			FLOAT -
			VAR {lappend R $t}
			}
		}
	return $R
}

# -----------------------------------------------------------------------------
# Rend une region de tokens entiers autour d'inipos, dans le texte w
# -----------------------------------------------------------------------------
proc ed:dyncolpro:getrange {w inipos} {
	set Lt [$w tag names $inipos]
#	puts "%-% $Lt"
#3	set thetag [lmatch -regexp $Lt "DELIM|ATOM|INT|QATOM|LCOMMENT|DQUOTE|FLOAT|VAR"]
	set thetag [eda:toktag $Lt]
#	puts "%-% $thetag"
	if {$thetag != {}} {
		set R [ed:dyncolpro:selfin $w [ed:dyncolpro:start $w $inipos $thetag] $thetag]
		} \
	else {
		# TMP:
		#set R [list [$w index $inipos-1c] [$w index $inipos]]
		set R [list [$w index $inipos] [$w index $inipos+1c]]
		}
	return $R
}

# -----------------------------------------------------------------------------
# (Re)colorie ce qu'il y a autour d'insert dans le texte t
# C'est ce qui est lance par les actions
# (avant {insert-3c}
# -----------------------------------------------------------------------------
proc ed:dyncol:Prolog:colorie {t {avant insert-4c} {apres insert+1c}} {
	#
	# TMP:
	#
	global the_ed
	set the_ed [winfo parent $t]

#	u $t
	set R [ed:dyncolpro:getrange $t $avant]
	set deb [lindex $R 0]

	set R [ed:dyncolpro:getrange $t $apres]
	set fin [lindex $R 1]

	$t mark set coltok $deb
	ed:dyncolpro:do $t $deb $fin
}

# -----------------------------------------------------------------------------
# Utilitaire servant a se faire montrer les tags associes aux caracteres
# entre deb et fin
# -----------------------------------------------------------------------------
proc viewtag {w deb fin} {
	set txt [$w get $deb $fin]
	set l [string len $txt]
	for {set i 0} {$i < $l} {incr i} {
		puts "$i : ([string index $txt $i]) \
			[$w tag names [$w index $deb+${i}c]]"
		}
}

if 0 {
	#
	# faineant
	#
	proc ed:dyncolpro:color {deb fin} {
		global w
		$w mark set coltok $deb
		ed:dyncolpro:do $w $deb $fin
	}
	#
	# faineant
	#
	proc ed:dyncolpro:col {} {
		global w
		ed:dyncolpro:color [$w index sel.first] [$w index sel.last]
		}
	}

#
# do 12.3 12.7
#
proc ed:dyncolpro:do {w deb fin} {
	global tcl_platform
	upvar #0 ed:dyncolpro:f2 f2
	upvar #0 ed:dyncolpro:coldebug coldebug

	set isunix [expr \"$tcl_platform(platform)\" == \"unix\"]

	#
	# On signale a parse un nouveau calcul
	#
#	if $isunix {
#3		while { [read $f2 1] != "" } {}
#TMP enlevee
#3		kill SIGUSR1 [pid $f2]
#		}
	
	set prems 1
	
	while 1 {
		if $isunix {
			#while { [gets $f2 res] == -1 } {}
			while { [gets $f2 res] == "" } {}
			} \
		else {
			after 10
			while { [gets $f2 res] == -1 } {after 30}
			}
		if $coldebug {puts <<$res>>}
		if {"$res" == "tcl_moretext"} {
			if $prems {
				#
				# On envoie les positions l1.c1 l2.c2
				#
				scan $deb "%d.%d" l1 c1
				scan $fin "%d.%d" l2 c2
				if $coldebug {puts  "($l1 $c1 $l2 $c2)"}
				puts -nonewline $f2 "\x1b\x1$l1 $c1 $l2 $c2\n "
				set prems 0
				} \
			else {
				puts -nonewline $f2 [ed:dyncolpro:autreligne $w]
				}
			flush $f2
			} \
		elseif {"$res" == "tcl_finish"} {
			break
			} \
		else {
			if $coldebug {puts "eval $res"}
			eval $res
			}	
	}
}

# -- EOF procolor.tcl ---------------------------------------------------------
