# -- tty.tcl
# Une sorte de console pour l'environnement Prolog IV
#
# Auteur:		Pascal Bouvier
# Creation:		12/06/95
# Modifications:	
#				none
#
# (c) PrologIA 1995
# -----------------------------------------------------------------------------


global tty:fontout tty:fonterror tty:fontuser
global tcl_platform
if {$tcl_platform(platform) == "unix"} {
	set tty:fontout   *-Courier-Medium-R-Normal-*-140-*
	set tty:fonterror *-Courier-Bold-R-Normal-*-140-*
	set tty:fontuser  *-Courier-Bold-R-Normal-*-140-*
	} \
else {
	set tty:fontout   {"courier new" 14 ""}
	set tty:fonterror {"courier new" 14 "bold"} 
	set tty:fontuser  {"courier new" 14 "bold"} 
	}


# -----------------------------------------------------------------------------
# Cree un panel "tty"
# -----------------------------------------------------------------------------
proc tty:newframe {root} {
	set fr $root.frtty
	
	#
	# Frame, text et scrollbar
	#
	frame $fr
	text $fr.t -relief sunken -bd 1 \
			-yscrollcommand "$fr.sv set" -wid 20
	scrollbar $fr.sv -orient vertical -command "$fr.t yview" \
			-relief sunken -bd 1
#			-bg #e3f3f5 -activebackg #002e00

	pack $fr.t -side left -fill both -expand yes
	pack $fr.sv -side left -fill y

	#
	# tags et marks de la console
	#
	$fr.t mark set iomark insert
	
	global tty:fontout tty:fonterror tty:fontuser
	$fr.t tag config out   -font ${tty:fontout}
	$fr.t tag config error -font ${tty:fonterror} -foreg red
	$fr.t tag config user  -font ${tty:fontuser}

	#
	#
	tty:Standard_Keys $fr.t

	return $fr
}

#
# Bindings: on prend les bindings par defaut, et on en ecrase certains
# par de nouveaux.
# A cause de Key/Any-Key il faut en enlever apres coups.
#
proc tty:Standard_Keys {t} {
#3	text:Standard_Keys $t

#3	bind $t <Return> {}
	bind $t <Any-Key> {
		tty:text_any_key %W %A
		break
		}
	foreach event {Key-Left Key-Right Key-Up Key-Down Key-BackSpace} {
		bind $t <$event> {#none }
		}
	bind $t <Control-Key-v> {
		if ![catch {
			selection get -display %W -selection CLIPBOARD} _res] {
			tty:text_any_key %W $_res
			}
		}
	bind $t <Control-Key-c> [bind Text <<Copy>>]
	bind $t <Control-Key-x> [bind Text <<Cut>>]
}

# -----------------------------------------------------------------------------
# Le traitement d'un caractere (ou d'un texte)
# dans une console
# -----------------------------------------------------------------------------
proc tty:text_any_key {W A} {
    if {"$A" != "" } {
		if [$W compare iomark <= insert] {
	#            errputs ok
			} \
		else {
			$W mark set insert end
			}
		tty:inserer_text $W $A
		}
}

# -----------------------------------------------------------------------------
# (suite) Le traitement d'un caractere (ou d'un texte)
# dans une console
# -----------------------------------------------------------------------------
proc tty:inserer_text {W A} {
	set m -1
	if  [$W compare iomark == insert] {
		set m [$W index iomark]
		}
	set A [translit "\r" "\n" $A]
	$W insert insert $A

	set len [string len $A]
	tty:set_style_user $W "insert-${len}c" insert

	if {$m != -1} {$W mark set iomark $m}

	$W yview -pickplace insert

	if {"[string index $A [expr $len-1]]" == "\n"} {
		global f1
		set ee "[$W get iomark insert]"
		# NEW SIGUSR1
		conn:envoie $ee $f1
		#TMP
		$W mark set iomark insert
		}
}

# -----------------------------------------------------------------------------
# Les deux styles dans tty :
#	- user (ce que frappe le programmeur)
#	- out (les sorties)
#	- error (Une sortie enluminee)
# -----------------------------------------------------------------------------
proc tty:set_style_out {W args} {
	eval $W tag remove user $args
	eval $W tag remove error $args
	eval $W tag add out $args
}
proc tty:set_style_user {W args} {
	eval $W tag remove out $args
	eval $W tag remove error $args
	eval $W tag add user $args
}
proc tty:set_style_error {W args} {
	eval $W tag remove out $args
	eval $W tag remove user $args
	eval $W tag add error $args
}
# -----------------------------------------------------------------------------
# Placement de texte s "a la fin" du tty T
# On met un newline a la fin par defaut (sinon le suffixe donne, comme "")
# -----------------------------------------------------------------------------
proc tty:insere_style {style T s {nl \n}} {
	set m [$T index iomark]
	tty:set_style_$style $T iomark

	$T insert iomark $s$nl

	$T yview -pickplace end
	tty:set_style_$style $T $m iomark
}

#-- EOF tty.tcl ---------------------------------------------------------------
