// Paquetage utilitaires
import java.util.Vector;
// Paquetage prolog
import prolog.*;
import java.sql.*;

public class exemples
{

//-----------------------------------------------------------------------------
//----- permute/2 -------------------------------------------------------------
//-----------------------------------------------------------------------------

  /**
   * BUT PROLOG : permute/2
   *
   * ARGUMENTS :
   *   1 -- quelconque
   *   2 -- variable
   *
   * Place le premier argument du but Prolog dans le second.
   *
   * EXEMPLES CORRECTS :
   *   > permute(34,a);
   *   {a=34}
   */

  public static PrologGoal permute (PrologGoal but)
  {
    try
    {
      /*
       * On recupere les deux termes du but.
       */
      PrologTerm term0 = but.getTerm(0);
  
      /*
       * On echange les places des deux termes du but.
       */
      but.setTerm(1, term0);
  
      but.setError(0);

      return but;
    }
    catch (Exception e)
    {
	  /*
	   * Si on a une exception, on positionne le champs "errno" de la classe
	   * PrologGoal avec le numro d'erreur qui convient. Pour que Prolog II+
	   * retourne un message cohrent, il ne faut pas oublier d'ajouter ce
	   * numro dans les fichiers err.txt et fr_err.txt de Prolog II+.
	   */
      System.out.println("exemples.permute (Exception) : " + e);
      int my_error = 1001;
      but.setError(my_error);

      return but;
    }
  }

//-----------------------------------------------------------------------------
//----- peigner/2 -------------------------------------------------------------
//-----------------------------------------------------------------------------

  /**
   * BUT PROLOG : peigner/2
   *
   * ARGUMENTS :
   *   1 -- une liste
   *   2 -- une variable
   *
   * Peigne la liste donnee en premier argument et place le resultat
   * dans le second argument.
   *
   * EXEMPLE CORRECT :
   *   >> peigner([[2,3],4,[5,[6,7]]],A).
   *   A = [2,3,4,5,6,7].
   */

  public static PrologGoal peigner (PrologGoal but)
  {
    try
    {
      /*
       * On peigne la liste donnee en premier argument puis
       * on place le resultat dans le vecteur result (qui ne
       * contient pas l'element nil). On rajoute ensuite
       * l'atome nil au vecteur "result".
       */
      Vector result = new Vector();
      parcourir((PrologDot)(but.getTerm(0)), result);
      result.addElement(new PrologNil());

      /*
       * On construit une liste a partir du vecteur "result".
       */
      PrologDot resultat = new PrologDot(result);

      /*
       * On place une variable comme premier element du but pour accelerer
       * l'ecriture sur la socket puis on place la liste peignee comme second
       * argument.
       */
      //but.setTerm(0, new PrologVar()); // prolog IV
      but.setTerm(1, resultat);

      return but;
    }
    catch (PrologException e)
    {
      System.out.println("exemples.peigner (PrologException) : " + e);
      int my_error = 1002;
      but.setError(my_error);

      return but;
    }
    catch (Exception e)
    {
      System.out.println("exemples.peigner (Exception) : " + e);
      int my_error = 1003;
      but.setError(my_error);

      return but;
    }
  }

  public static void parcourir (PrologDot arbre, Vector result) throws Exception
  {
    try
    {
      Object tmp;
      for (int i = 0 ; i < arbre.getSize()-1 ; i++)
      {
        tmp = arbre.getElementAt(i);
        if (tmp instanceof PrologDot)
        {
          if (((PrologDot)tmp).isList())
          {
            parcourir((PrologDot)tmp, result);
          }
          else
          {
            result.addElement(tmp);
          }
        }
        else
        {
          result.addElement(tmp);
        }
      }
    }
    catch (Exception e)
    {
      System.out.println("exemples.parcourir (Exception) : " + e);
      throw e;
    }
  }

//-----------------------------------------------------------------------------
//----- inverse/2 -------------------------------------------------------------
//-----------------------------------------------------------------------------

  /**
   * BUT PROLOG : inverse/2
   *
   * ARGUMENTS :
   *   1 -- une liste
   *   2 -- une variable
   *
   * Inverse la liste donnee en premier argument et place le
   * resultat dans le second argument.
   *
   * EXEMPLE CORRECT :
   *   >> inverse([4,5,6,7],A).
   *   A = [7,6,5,4].
   */

  public static PrologGoal inverse (PrologGoal but)
  {
    try
    {
      /*
       * On recupere la liste qui se trouve en premier argument
       * du but.
       */
      PrologDot liste = (PrologDot)(but.getTerm(0));

      /*
       * On ne peut inverser qu'une liste et pas une paire pointee.
       */
      if (liste.isList() == false)
      {
        throw new PrologException("Le premier argument doit etre une liste !");
      }

      /*
       * On place les elements de la liste "liste" en sens inverse
       * dans le vecteur "inverse" puis on construit la liste "inv"
       * a partir de ce vecteur.
       */
      Vector inverse = new Vector();
      for (int i = liste.getSize()-2 ; i >= 0 ; i--)
      {
        inverse.addElement(liste.getElementAt(i));
      }
      inverse.addElement(new PrologNil());
      PrologDot inv = new PrologDot(inverse);

      /*
       * On place une variable comme premier element du but pour accelerer
       * l'ecriture sur la socket puis on place la liste inverse comme second
       * argument.
       */
      //but.setTerm(0, new PrologVar()); // prolog IV
      but.setTerm(1, inv);

      return but;
    }
    catch (PrologException e)
    {
      System.out.println("exemples.inverse (PrologException) : " + e);
      int my_error = 1004;
      but.setError(my_error);

      return but;
    }
    catch (ClassCastException e)
    {
      System.out.println("exemples.inverse (ClassCastException) : " + e);
      int my_error = 1005;
      but.setError(my_error);

      return but;
    }
    catch (Exception e)
    {
      System.out.println("exemples.inverse (Exception) : " + e);
      int my_error = 1006;
      but.setError(my_error);

      return but;
    }
  }

//-----------------------------------------------------------------------------
//----- decomposeRat/3 --------------------------------------------------------
//-----------------------------------------------------------------------------

  /**
   * BUT PROLOG : decomposeRat/3
   *
   * ARGUMENTS :
   *   1 -- un rationnel
   *   2 -- une variable
   *   3 -- une variable
   *
   * Decompose un rationnel en deux entiers : son numerateur et
   * son denominateur.
   *
   * EXEMPLE CORRECT :
   *   >> decomposeRat(2/5,N,D).
   *   N = 2,
   *   D = 5.
   */

  public static PrologGoal decomposeRat (PrologGoal but)
  {
    try
    {
      /*
       * On recupere le rationnel qui se trouve en premier argument
       * du but Prolog.
       */
      PrologRat term = (PrologRat)(but.getTerm(0));
  
      /*
       * On place en deuxieme argument le numerateur du rationnel et
       * en troisieme argument le denominateur du rationnel.
       */
      but.setTerm(1, new PrologInt(term.getNum()));
      but.setTerm(2, new PrologInt(term.getDen()));
  
      return but;
    }
    catch (ClassCastException e)
    {
      System.out.println("exemples.decomposeRat (ClassCastException) : " + e);
      int my_error = 1007;
      but.setError(my_error);

      return but;
    }
    catch (Exception e)
    {
      System.out.println("exemples.decomposeRat (Exception) : " + e);
      int my_error = 1008;
      but.setError(my_error);

      return but;
    }
  }

//-----------------------------------------------------------------------------
//----- triQuicksort/2 --------------------------------------------------------
//-----------------------------------------------------------------------------

  /**
   * BUT PROLOG : triQuicksort/2
   *
   * ARGUMENTS :
   *   1 -- une liste d'entiers
   *   2 -- une variable
   *
   * Trie la liste donnee en premier argument et met le resultat
   * dans le second argument. On utilise ici un tri rapide recursif.
   */

  public static PrologGoal triQuicksort (PrologGoal but)
  {
    try
    {
      /*
       * On verifie tout d'abord que le premier argument est bien une liste.
       * On recupere le vecteur representant la liste donnee en premier
       * argument puis on place dans la variable "taille" la taille de ce
       * vecteur moins un (on ne compte pas l'atome nil).
       */
      PrologDot firstArg = (PrologDot)(but.getTerm(0));
      if (firstArg.isList() == false)
      {
        throw new PrologException("Le premier argument doit etre une liste !");
      }
      Vector aTrier = firstArg.getVector();
      int taille = aTrier.size()-1;
  
      /*
       * On construit un tableau d'entiers (tab) a partir des elements de
       * ce vecteur (aTrier).
       */
      long t1 = System.currentTimeMillis();
      int tab[] = new int[taille];
      for (int i = 0 ; i < taille ; i++)
      {
        tab[i] = ((PrologInt)(aTrier.elementAt(i))).getValue();
      }
      long t2 = System.currentTimeMillis();
      System.out.println("exemples.triQuicksort : temps pour construire tab = " + (t2-t1));
  
      /*
       * On trie le tableau d'entiers "tab".
       */
      t1 = System.currentTimeMillis();
      quicksort(tab, 0, taille-1);
      t2 = System.currentTimeMillis();
      System.out.println("exemples.triQuicksort : temps pour trier tab = " + (t2-t1));
  
      /*
       * On reconstruit un vecteur d'objets PrologInt a partir du tableau d'entiers
       * puis on rajoute l'atome nil a ce vecteur.
       */
      t1 = System.currentTimeMillis();
      Vector result = new Vector(taille+1);
      for (int i = 0 ; i < taille ; i++)
      {
        result.addElement(new PrologInt(tab[i]));
      }
      result.addElement(new PrologNil());
      t2 = System.currentTimeMillis();
      System.out.println("exemples.triQuicksort : temps pour construire result = " + (t2-t1));
  
      /*
       * On place une variable comme premier element du but pour accelerer
       * l'ecriture sur la socket puis on place la liste triee comme second
       * argument.
       */
      //but.setTerm(0, new PrologVar()); // prolog IV
      but.setTerm(1, new PrologDot(result));

      return but;
    }
    catch (PrologException e)
    {
      System.out.println("exemples.triQuicksort (PrologException) : " + e);
      int my_error = 1009;
      but.setError(my_error);

      return but;
    }
    catch (Exception e)
    {
      System.out.println("exemples.triQuicksort (Exception) : " + e);
      int my_error = 1010;
      but.setError(my_error);

      return but;
    }
  }

  public static void quicksort (int t[], int indice_debut, int indice_fin) throws Exception
  {
    try
    {
      if (indice_debut < indice_fin)
      {
        int place_du_pivot = partitionner(t, indice_debut, indice_fin);
        quicksort(t, indice_debut, place_du_pivot-1);
        quicksort(t, place_du_pivot+1, indice_fin);
      }
    }
    catch (Exception e)
    {
      System.out.println("exemples.quicksort (Exception) : " + e);
      throw e;
    }
  }

  public static int partitionner (int t[], int premier, int dernier) throws Exception
  {
    try
    {
      int pivot = t[premier], i, position, aux;
  
      position = premier;
      for (i = premier+1 ; i <= dernier ; i++)
      {
        if (t[i] < pivot)
        {
          position++;
          aux = t[position];
          t[position] = t[i];
          t[i] = aux;
        }
      }
      aux = t[premier];
      t[premier] = t[position];
      t[position] = aux;
  
      return position;
    }
    catch (Exception e)
    {
      System.out.println("exemples.partitionner (Exception) : " + e);
      throw e;
    }
  }


//***********************************************
//***********************************************


	public static PrologGoal totalInsert (PrologGoal but)
	{

		try
		{
      PrologString dbAddressAndName = (PrologString)(but.getTerm(0));
      PrologString dbUserName       = (PrologString)(but.getTerm(1));
      PrologString dbPassword       = (PrologString)(but.getTerm(2));

      PrologDot listTables = new PrologDot();
      listTables = (PrologDot)(but.getTerm(3));

      PrologDot listResult = new PrologDot();
      
      Database base = new Database ( dbAddressAndName.getValue(),
                                     dbUserName.getValue(),
                                     dbPassword.getValue() );

      for (int i = 0 ; i < listTables.getSize()-1 ; i++)
      {
        PrologDot tempDot = new PrologDot(base.listResultSet(base.selectQuery("select * from " + ((PrologString)(listTables.getElementAt(i))).getValue())));
        listResult.addElement(tempDot);
      }
      listResult.insertElementAt(new PrologNil(), listTables.getSize()-1);

System.out.println("listResult = " + listResult);
      but.setTerm(4, listResult);

      base.close();

      but.setError(0);

      return but;
    }
    catch (Exception e)
    {
      but.setError(2000);

      return but;
    }

  }










}

