% -- dbg.p4 --
% Primitives pour debugging
%

% Les variables:
sysh__showvars([]).
sysh__showvars([[N|V]|L]) :- sysh__showvar(N,V), sysh__showvars(L).

sysh__showvar(Nom,Val) :- 
    wsend(['gd:initshowvar ', Nom]),
    freeze(Val, (
    	lb__get_absboxnb(A1,_),
	wsend2(['gd:showvar ', Nom, ' {', Val, '} ', A1], success(1)))
	).

% Les solutions
green.

%
% Une matrice (list de liste de rationnel)
%
sysh__afficheNxM(L) :- afficheNxM(1, L).

afficheNxM(_n, []) :- !.
afficheNxM(_i, [X|A]) :-
	afficheLigne(_i, 1, X),
	afficheNxM(_i+1, A).
 

afficheLigne(_n, _m, []) :- !.
afficheLigne(_n, _m, [_e|X]) :-
	spot(_n, _m, _e),
	afficheLigne(_n, _m+1, X).

spot(_n, _m, _e) :-
	trunc(_e, _e1),
	color(64-_e1),
	gr_rect(paint, 10*_m+20, 10*_n+20, 10*_m+30, 10*_n+30).

% Pour les intervalles
sysh__iafficheNxM(L) :- iafficheNxM(1, L).

iafficheNxM(_n, []) :- !.
iafficheNxM(_i, [X|A]) :-
	iafficheLigne(_i, 1, X),
	iafficheNxM(_i+1, A).
 

iafficheLigne(_n, _m, []) :- !.
iafficheLigne(_n, _m, [_e|X]) :-
	ispot(_n, _m, _e),
	iafficheLigne(_n, _m+1, X).

ispot(_n, _m, _e) :-
	bounds(_e, Min,Max),
%	trunc(Min, _e1),
%	color(64-_e1),
	trunc(Min, TMin),
	color(64-TMin),
	gr_rect(paint, 10*_m+20, 10*_n+20, 10*_m+30, 10*_n+30),
	trunc(Max, TMax),
	color(64-TMax),
	gr_rect(paint, 10*_m+200, 10*_n+20, 10*_m+210, 10*_n+30)
	.

ispot_dyn(_n, _m, _e) :-
	ispot(_n, _m, _e),
	fail.
ispot_dyn(_n, _m, _e) :-
	sysh__setup_freeze_bis(_e, ispot_dyn(_n,_m,_e)).

sysh__ispot(_n, _m, _e) :-
	bounds(_e, Min,Max),
%	trunc(Min, _e1),
%	color(64-_e1),
	trunc(Min, TMin),
	color(64-TMin),
	gr_rect(paint, 10*_m+20, 10*_n+20, 10*_m+30, 10*_n+30),
	trunc(Max, TMax),
	color(64-TMax),
	gr_rect(paint, 10*_m+200, 10*_n+20, 10*_m+210, 10*_n+30)
	.

%
color(_c) :-
	lelin(0,_c), lelin(_c, 12), !,
	trunc(19.6*_c, T),
	gr_fgcolor3(T, 0, 0).
color(_c) :-
	lelin(13,_c), lelin(_c, 25), !,
	trunc( 19.6*(_c-12), T),
	gr_fgcolor3(255, T, 0).
color(_c) :-
	lelin(26,_c), lelin(_c, 38), !,
	trunc(255-19.6*(_c-25), T),
	gr_fgcolor3(T, 255, 0).
color(_c) :-
	lelin(39,_c), lelin(_c, 51), !,
	trunc(19.6*(_c-38), T),
	gr_fgcolor3(0, 255, T).
color(_c) :-
	lelin(52,_c), lelin(_c, 64), !,
	trunc(255-19.6*(_c-51), T),
	gr_fgcolor3(0, T, 255).
color(_c) :-
	ltlin(_c,0),
	gr_fgcolor3(80, 80, 80).
color(_c) :-
	ltlin(64,_c),
	gr_fgcolor3(160, 160, 160).

%
sysh__setup_freeze(V,P) :-
%	ebounds(V,A,B), write([P,A,B]), nl, 
   ebounds(V, Vl, Vu),
   suite_setup_freeze(V,P, Vl, Vu).

sysh__setup_freeze_bis(V,P) :-
   ebounds(V, Vl, Vu),
   sf_change_lb(Vl, V,P,U),
   sf_change_ub(Vu, V,P,U).

%

suite_setup_freeze(_,P, Vl, Vl) :- !, freeze(1,P).
suite_setup_freeze(_,P, Vl, Vu) :- 
	float_rank(Vl, Rl),
	float_rank(Vu, Ru), 
	Ru < Rl+10,
	!,
	freeze(1,P).
suite_setup_freeze(V,P, Vl, Vu) :-
   sf_change_lb(Vl, V,P,U),
   sf_change_ub(Vu, V,P,U).

% detection d'une variation de borne pour Al :
% minfinity 	-> up -max_float
% up F		-> next F
% F 		-> up F
sf_change_lb(minfinity,A,P,U) :- !,
   max_float(MF),
   bgt(B,A,-MF),
   freezeifvar(B,onlyonce(U,P)).
sf_change_lb(up(Al),A,P,U) :- !,
   next_float(Al,Alf),
   bge(B,A,Alf),
   freezeifvar(B,onlyonce(U,P)).
sf_change_lb(Al,A,P,U) :-
   bgt(B,A,Al),
   freezeifvar(B,onlyonce(U,P)).
  
% detection d'une variation de borne pour Au :
% minfinity 	-> up -max_float
% up F		-> next F
% F 		-> up F
sf_change_ub(pinfinity,A,P,U) :- !,
   max_float(MF),
   blt(B,A,MF),
   freezeifvar(B,onlyonce(U,P)).
sf_change_ub(dn(Au),A,P,U) :- !,
   previous_float(Au,Auf),
   ble(B,A,Auf),
   freezeifvar(B,onlyonce(U,P)).
sf_change_ub(Au,A,P,U) :-
   blt(B,A,Au),
   freezeifvar(B,onlyonce(U,P)).


% Pour ne pas degeler P trop souvent (voir le freeze au dessus) 
onlyonce(U,P) :- var(U), !, U = 1, P.
onlyonce(_,_).

% si V est pris a ce moment,on ne gele pas (c'est qu'on avait affaire
% plus haut a une constante,captee par un bxx).
freezeifvar(V,P) :- var(V), !, freeze(V,P).
freezeifvar(_,_).

next_float(A,B) :- float_rank(A,R), float_rank(B,R+1).
previous_float(A,B) :- float_rank(A,R), float_rank(B,R-1).

xx(X,up(L),dn(U)) :- !, oo(X,L,U).
xx(X,minfinity,dn(U)) :- !, lt(X,U).
xx(X,minfinity,U) :- !, le(X,U).
xx(X,L,dn(U)) :- !, co(X,L,U).
xx(X,up(L),pinfinity) :- !, gt(X,L).
xx(X,L,pinfinity) :- !, ge(X,L).
xx(X,up(L),U) :- !, oc(X,L,U).
xx(X,L,U) :- !, cc(X,L,U).




% -- EOF dbg.p4 -------------------------------------