% -- drawtree.p4 --
% Dessin d'arbres finis Prolog IV.
% Prolog IV Finite tree drawing.
% Pascal Bouvier, le 15 Mai 1993 (for Prolog III)
%     26/01/95 (revue Prince)
%     08/07/95 (pour Prolog IV)
% 
% (c) PrologIA 1993, 1995-1997
%
% Pour executer les exemples go/1, ce fichier ainsi que "graf.p4" doivent
% etre compiles, le le but "gr_init" doit avoir ete lance.
% To run go/1 examples, this file and "graf.p4" must be compiled, and
% goal "gr_init" must be launched.
%
% >> compile("graf.p4").
% true.
% >> compile("drawtree.p4").
% true.
% >> gr_init.		% creates a graphic window
% true.
% >> gr_erase, go(I).
% ...

go(0) :-
	R = [100,20,X,Y],
	drawtree(toto(1,hh), R, L).

go(1) :-
	R = [100,20,X,Y],
	drawtree(toto(1,hh(jj("chaine",_X,_Y)),t(gg(hh),3/4)), R, L).
go(2) :-
	R = [100,20,X,Y],
	drawtree(poto("toto", """tutu""", "ti\"ti"), R, L).
go(3) :-
	R = [100,20,X,Y],
	drawtree(poto(1,2,3,4,5,6,7,8,9,10), R, L).
go(4) :-
	R = [100,20,X,Y],
	drawtree(poto([1,2,3,4,5,6,7,8,9,10]), R, L).
go(5) :-
	R = [100,20,X,Y],
	drawtree(poto([[1,2,3],2,3,4,5,6,7,8,9,10]), R, L).
go(6) :-
	R = [100,20,X,Y],
	drawtree('-->'(toto,','(titi,','('{}'(tutu),','(tata,'{}'(tunk))))), R, L).
go(66) :-
	R = [100,20,X,Y],
	drawtree('''-->'''(toto,''','''(titi,''','''('''{}'''(tutu),''','''(tata,'''{}'''(tunk))))), R, L).
go(7) :-
	R = [100,20,X,Y],
	drawtree([str("toto",[417/2,20,469/2,40]),fle(443/2,40,106,60),str("1",[100,60,112,80]),fle(443/2,40,393/2,60)
,str("hh",[187,60,206,80]),fle(393/2,80,393/2,100),str("jj",[191,100,202,120]),fle(393/2,120,317/2,140),str("\"chaine\"",[132,140,
185,160]),fle(393/2,120,214,140),str("_x",[205,140,223,160]),fle(393/2,120,252,140),str("_x",[243,140,261,160]),fle(443/2,40,312,60),str("[]",[302,60,322,80]),fle(312,80,581/2,100),str("gg",[281,100,300,120]),fle(581/2,120,581/2,140),str("hh",[281,140,300,160]
),fle(312,80,663/2,100),str("3/4",[320,100,343,120])], R, L).

%
% drawtree(Arbre, Rectangle, liste_instr)
%
drawtree(A, R, L) :-
	calctree(A, R, L, []),
	write('generating drawing ...'),nl,
	drawop(L).

%
drawop([]) :- !.
drawop([fle(X,Y,X2,Y2) | L]) :-
	gr_arrow(X,Y,X2,Y2),
	drawop(L).
drawop([str(S,[A,B,C,D]) | L]) :-
	gr_drawstring_in(S, A,B,C,D),
	drawop(L).

%
calctree(A, R, [str(S,R) | L], L) :-
	R = [X, Y, X+_dx, Y+_dy],
	feuille(A, S, Len), !,
	gr_stringbox(S, Len, _dx, _dy).
calctree(T, [X0,Y0, X1,Y1u], L1, L2) :-
	T =.. [E | U],
	X_c = X0e + (X1e-X0e)/2,
	X_c = X0u + (X1u-X0u)/2,
	X_c = X0 + (X1-X0)/2,
	Y0u = Y1e + 20,
	calctree(E, [X0e,Y0, X1e,Y1e], L1, L3),
	calclisttree(U, [X0u,Y0u, X1u,Y1u], [X_c, Y1e], L3, L2),
	mmax(X1e-X0e, X1u-X0u, X1-X0)
	.
% Assuming a char is 8 pixels width, and 12 pixels high.
% (should be changed to reflect the true font)
gr_stringbox(S, L, 8*L, 12).

%
feuille(X,"_x", 2) :- var(X), !.
feuille(X, X, Len) :- atom(X), !, atom_length(X, Len).
feuille(X, X, Len) :- number(X), !, number_chars(X, Lst), Len = size(Lst). 
/*feuille(X, X) :- integer(X), !.
feuille(X, X) :- real(X), !.
feuille(X, X) :- rational(X), !.
*/


%
mmax(A,B,A) :- A gelin B, !.
mmax(A,B,B).

%
calclisttree([U], R, [E,F], [fle(E,F, X+(X2-X)/2, Y) | L1], L2) :- !,
	R = [X,Y,X2,Y2],
	calctree(U, R, L1, L2).
calclisttree([U | L], R, P, [fle(E,F, X1+(X1p-X1)/2, Y1) | L1], L2) :-
	 R1 = [X1,Y1,X1p,Y1p],
	 R2 = [X2,Y2,X2p,Y2p],
	 R = [X,Y, Xp,Yp],
	 Y1 = Y2, Y2 = Y,
	 X2 = X1p + 10,
	 X = X1, Xp = X2p,
	 P = [E,F],
	calctree(U, R1, L1, L3),
	calclisttree(L, R2, P, L3, L2),
	mmax(Y1p-Y1, Y2p-Y2, Yp-Y).

/* USELESS
trait(x,y,x',y') :-
	gr_gray(1, 1.0)
	gr_moveto(x,y)
	gr_lineto(x',y')
	gr_gray(1, 0.0)
	gr_moveto(x+1,y+1)
	gr_lineto(x'+1,y'+1)
	.
roue(x,y,d,p) :-
	enum(a, 0, d-1)
	val(mul(p,sin(2*3.1416*a/d)), s)
	val(mul(p,cos(2*3.1416*a/d)), c)
	trait(x,y,x+c,y+s).
*/

% -- EOF drawtree.p4 ---------------------------------------------------------%
