% -- graf.p4 --
% Primitives graphiques de Prolog IV en Prolog, relais vers TCL/TK.
% Graphical Prolog IV primitives, relay to TCL/TK.
%
% Pascal Bouvier, le 08/07/95
% (c) PrologIA 1995,1996,1997
%


%%%
%%% Initialisaton : doit etre lance avant toute autre commande graphique.
%%% Initialization: should be placed before any other graphical command.
%%%
gr_init :-
	wsend(['tcl_inigraphic']).

%%%
%%% Deplace le crayon au point X,Y.
%%% Move the pen to point X,Y.
%%%
gr_moveto(X,Y) :-
	wsend(['tcl_gr_moveto {', X, '} {', Y, '}']).

%%%
%%% Deplace le crayon de dX,dY.
%%% Move the pen of dX,dY.
%%%
gr_move(X,Y) :-
	wsend(['tcl_gr_move {', X, '} {', Y, '}']).

%%%
%%% Recupere les coordonnes du crayon (X,Y).
%%% Get pen coordinates (X,Y).
%%%
gr_penloc(X,Y) :-
	wsend2(['tcl_gr_penloc'], [X,Y]).

%%%
%%% Dessine un trait de l'endroit courant jusqu'au point X,Y.
%%% Draw a line from current point to point X,Y.
%%%
gr_lineto(X,Y) :-
	wsend(['tcl_gr_lineto {', X, '} {', Y, '}']).

%%%
%%% Dessine un polygone a partir d'une liste de coordonnees [X1,Y1,X2,Y2,...].
%%% Draw a polygon from a list of coordinates [X1,Y1,X2,Y2,...].
%%%
gr_polyline(L) :-
	gr__list_to_polyline(L, L1),
	wsend(['tcl_gr_polyline ' | L1]).

%
gr__list_to_polyline([], []) :-!.
gr__list_to_polyline([X,Y | L], [' {',X, '} {', Y,'}' | L1]) :-
	gr__list_to_polyline(L, L1).
	
%%%
%%% Dessine un trait de l'endroit courant, de longueur DX,DY.
%%% Draw a line from current point, with length DX,DY.
%%%
gr_line(DX,DY) :-
	wsend(['tcl_gr_line {', DX, '} {', DY, '}']).

%%%
%%% Cree un rectangle de coins X,Y, X2,Y2. Le verbe V est 'frame' ou 'paint'.
%%% Draw a rectangle with corners X,Y, X2,Y2. Verb V can be 'frame' or 'paint'.
%%%
gr_rect(V, [X,Y,X2,Y2]) :- gr_rect(V, X,Y,X2,Y2).

gr_rect(V, X,Y, X2,Y2) :-
	wsend(['tcl_gr_rect {', V, '} {', X, '} {', Y, '} {', X2, '} {', Y2, '}']).

%%%
%%% Cree le texte S au point courant du crayon (en bas a droite (nw)).
%%% Draw text S at current pen point (bottom right (nw)).
%%%
gr_drawstring(S) :-
	wsend(['tcl_gr_drawstring {', S, '}']).

%%%
%%% Cree le texte S au point X,Y.
%%% Draw text S at point X,Y.
%%%
gr_drawstring_at(S, X,Y) :-
	wsend(['tcl_gr_drawstring_at {', S, '} {', X, '} {', Y, '}']).

%%%
%%% Cree le texte S dans le rectangle de coins X,Y et X2,Y2.
%%% Draw text S in the rectangle with corners X,Y and X2,Y2.
%%%
gr_drawstring_in(S, X,Y, X2,Y2) :-
	wsend(['tcl_gr_drawstring_in {', S, '} {', X, '} {', Y, '} {', X2, '} {', Y2, '}']).

%%%
%%% gr_stringbox(S, 8w, 18) :- {S :: w}.
%%%

%%%
%%% Dessine une fleche de X,Y vers X2,Y2.
%%% Draw an arrow from X,Y to X2,Y2.
%%%
gr_arrow(X,Y, X2,Y2) :-
	wsend(['tcl_gr_arrow {', X, '} {', Y, '} {', X2, '} {', Y2, '}']).

%%%
%%% Met la fonte (X11) des textes crees ulterieurement.
%%% Set font (X11) for next texts.
%%%
gr_setxfont(F) :-
	wsend(['tcl_gr_setxfont {', F, '}']).

%%%
%%% Met la fonte des textes crees ulterieurement.
%%% Set the font for next texts. 
%%% F = Fontname {Helvetica, times, courier}
%%% Size {12, 14, 18, 24}
%%% Bold {Bold, Medium}
%%%
gr_setfont(F) :-
	wsend(['tcl_gr_setfont {', F, '}']).
gr_setfont(F, S) :-
	wsend(['tcl_gr_setfont {', F, '} {', S, '}']).
gr_setfont(F, S, B) :-
	wsend(['tcl_gr_setfont {', F, '} {', S, '} {', B, '}']).


%%%
%%% Met la couleur des objets ulterieurement crees a #RGB.
%%% Set the foreground color for next objects to #RGB.
%%%
gr_fgcolor3(R, G, B) :-
	wsend(['tcl_gr_fgcolor3 {', R, '} {', G, '} {', B, '}']).

%%%
%%% Met la couleur des objets ulterieurement crees a Coul.
%%% Set the foreground color for next objects to Coul.
%%%
gr_fgcolor(Coul) :-
	wsend(['tcl_gr_fgcolor {', Coul, '}']).

%%%
%%% Met la couleur des objets ulterieurement crees a Gray.
%%% Set the foreground color for next objects to Gray.
%%%
gr_fggray(Gray) :-
	wsend(['tcl_gr_fggray {', Gray, '}']).

%%%
%%% Met le tag Tag pour les prochains objets a creer. Le tag {} l'enleve.
%%% Set tag Tag for next objects.  Tag {} unsets current tag.
%%%
gr_settag(Tag) :-
	wsend(['tcl_gr_settag {', Tag, '}']).

%%%
%%% Detruit tous les objets de la fenetre graphique courante.
%%% Destroy all object from the current graphics window.
%%%
gr_erase :-
	wsend(['tcl_gr_erase']).

%%%
%%% Detruit tous les objets de tag Tag de la fenetre graphique courante
%%% Destroy all object with tag Tag from the current graphics window.
%%%
gr_erase(Tag) :-
	wsend(['tcl_gr_erase {', Tag, '}']).

%%%
%%% Dessine l'arbre fini T dans le rectangle (partiellement inconnu) R.
%%% Draw finite tree T in the rectangle (partially known) R.
%%%
gr_drawtree(T, R) :- drawtree(T, R, _).


%%%
%%% Recupere les coordonnes d'un clic dans la fenetre (X,Y).
%%% Get click coordinates (X,Y).
%%%
% 1 = attends si aucun click d'avance
gr_click(1, X,Y) :-
	wsend2(['tcl_gr_click_wait'], [X,Y]).
%
% 0 = si pas de click d'avance, echoue.
gr_click(0, X,Y) :-
	wsend2(['tcl_gr_click_nowait'], [X,Y]).

%%%
%%% Enleve tous les clics tapes d'avance.
%%% Flush all clicks.
%%%
% 1 = attends si aucun click d'avance
gr_flushclicks :-
	wsend(['tcl_gr_flush_clicks']).

%%%
%%% Illumination de lignes de texte (entre L1 et L2 comprises) d'une fenetre dont on donne le fichier F.
%%%
gr_illuminer(F, L1,L2, LTerms) :-
	wsend(['tcl_open_file_and_highlight_lines {',F,'} ',L1,' ',L2,' '] o LTerms).

%-- EOF graf.p4 --------------------------------------------------------------%
