% -- random.p4 --
% un generateur de nombres pseudo-aleatoires.
% A pseudo-random number generator.
%
% Pascal Bouvier, d'apres Prolog III
% (c) PrologIA 1996,1997

% initializer
randomize(S) :- integer(S) ,!, record(seed, S).
randomize(S) :- var(S), record(seed, 1).

% number generator
% (formula got from C-ANSI random() ?)
random(X) :-
        recorded(seed,S),
        (integer(S) -> S1 = S ; S1 = 1),
        Z = S1*1103515245 + 12345,
        modulo(Z, 65536*65536-1, Z1),
        record(seed,Z1),
        modulo(Z,32767, X).

% just to test random/1.  This simulate 6000 throws of a dice.
% >> test, print_stats.
%
test :-
	def_array(stat, 6),
	init_stats,
	conttest.
	
% to pursue a test without reinitializing the statistics array
% >> conttest, print_stats.
conttest :-
	enum(_I, 1, 6000),
	random(X),
	modulo(X, 6, DES -1),
	recorded(stat(DES), VDES),
	record(stat(DES), VDES + 1),
	fail.
conttest.

print_stats :-
	enum(I, 1,6),
	recorded(stat(I), V),
	write(I), write(' : '), write(V), nl,
	fail.
print_stats.

init_stats:-
	enum(I, 1, 6),
	record(stat(I), 0),
	fail.
init_stats.

% -- EOF random.p4 ---------------------------------------------%